@@ -34,7 +34,7 @@ extern "C" {
 typedef struct _GMimeHeader GMimeHeader;
 
 typedef void (*GMimeHeaderForeachFunc) (const char *name, const char *value, gpointer data);
-typedef ssize_t (*GMimeHeaderWriteFunc) (GMimeStream *stream, const char *name, const char *value);
+typedef gssize (*GMimeHeaderWriteFunc) (GMimeStream *stream, const char *name, const char *value);
 
 GMimeHeader *g_mime_header_new (void);
 
@@ -48,7 +48,7 @@ const char *g_mime_header_get (const GMimeHeader *header, const char *name);
 
 void g_mime_header_remove (GMimeHeader *header, const char *name);
 
-ssize_t g_mime_header_write_to_stream (const GMimeHeader *header, GMimeStream *stream);
+gssize g_mime_header_write_to_stream (const GMimeHeader *header, GMimeStream *stream);
 
 char *g_mime_header_to_string (const GMimeHeader *header);
 
