@@ -21,20 +21,26 @@
  *
  */
 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <ctype.h>
+#include <errno.h>
+
 #include "gmime-utils.h"
 #include "gmime-table-private.h"
 #include "gmime-part.h"
 #include "gmime-charset.h"
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <ctype.h>
+#include "gmime-iconv.h"
+#include "gmime-iconv-utils.h"
 
 #define d(x)
+#define w(x) x
 
 #ifndef HAVE_ISBLANK
 #define isblank(c) (c == ' ' || c == '\t')
@@ -43,10 +49,12 @@
 #define GMIME_UUENCODE_CHAR(c) ((c) ? (c) + ' ' : '`')
 #define	GMIME_UUDECODE_CHAR(c) (((c) - ' ') & 077)
 
+#define GMIME_FOLD_PREENCODED  (GMIME_FOLD_LEN / 2)
+
 /* date parser macros */
 #define NUMERIC_CHARS          "1234567890"
-#define WEEKDAY_CHARS          "Sun,Mon,Tue,Wed,Thu,Fri,Sat,"
-#define MONTH_CHARS            "JanFebMarAprMayJunJulAugSepOctNovDec"
+#define WEEKDAY_CHARS          "SundayMondayTuesdayWednesdayThursdayFridaySaturday"
+#define MONTH_CHARS            "JanuaryFebruaryMarchAprilMayJuneJulyAugustSeptemberOctoberNovemberDecember"
 #define TIMEZONE_ALPHA_CHARS   "UTCGMTESTEDTCSTCDTMSTPSTPDTZAMNY()"
 #define TIMEZONE_NUMERIC_CHARS "-+1234567890"
 #define TIME_CHARS             "1234567890:"
@@ -60,6 +68,7 @@
 #define DATE_TOKEN_NON_TIMEZONE_NUMERIC (1 << 6)
 #define DATE_TOKEN_HAS_SIGN             (1 << 7)
 
+
 static char *base64_alphabet =
 "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 
@@ -109,12 +118,12 @@ static unsigned char gmime_uu_rank[256] = {
 static unsigned char gmime_datetok_table[256] = {
 	128,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,
 	111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,
-	111,111,111,111,111,111,111,111, 79, 79,111,175,109,175,111,111,
+	111,111,111,111,111,111,111,111, 79, 79,111,175,111,175,111,111,
 	 38, 38, 38, 38, 38, 38, 38, 38, 38, 38,119,111,111,111,111,111,
 	111, 75,111, 79, 75, 79,105, 79,111,111,107,111,111, 73, 75,107,
 	 79,111,111, 73, 77, 79,111,109,111, 79, 79,111,111,111,111,111,
-	111,105,107,107,109,105,111,107,109,109,111,111,107,111,105,105,
-	107,111,105,111,105,105,107,111,111,107,111,111,111,111,111,111,
+	111,105,107,107,109,105,111,107,105,105,111,111,107,107,105,105,
+	107,111,105,105,105,105,107,111,111,105,111,111,111,111,111,111,
 	111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,
 	111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,
 	111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,
@@ -174,7 +183,11 @@ g_mime_utils_header_format_date (time_t time, int offset)
 	
 	time += ((offset / 100) * (60 * 60)) + (offset % 100) * 60;
 	
+#ifdef HAVE_GMTIME_R
+	gmtime_r (&time, &tm);
+#else
 	memcpy (&tm, gmtime (&time), sizeof (tm));
+#endif
 	
 	return g_strdup_printf ("%s, %02d %s %04d %02d:%02d:%02d %+05d",
 				tm_days[tm.tm_wday], tm.tm_mday,
@@ -203,14 +216,18 @@ datetok (const char *date)
 	start = date;
 	while (*start) {
 		/* kill leading whitespace */
-		for ( ; *start && isspace ((int) *start); start++);
-		
-		mask = 0;
+		while (*start && isspace ((int) *start))
+			start++;
+	
+		if (*start == '\0')
+			break;
+
+		mask = gmime_datetok_table[*start];
 		
 		/* find the end of this token */
-		for (end = start; *end && !isspace ((int) *end) && (end > start ? !strchr ("-/", *end) : TRUE); end++) {
-			mask |= gmime_datetok_table[*end];
-		}
+		end = start+1;
+		while (*end && !strchr ("-/,\t\r\n ", *end))
+			mask |= gmime_datetok_table[*end++];
 		
 		if (end != start) {
 			token = g_malloc (sizeof (struct _date_token));
@@ -300,7 +317,7 @@ get_wday (const unsigned char *in, unsigned int inlen)
 		return -1;
 	
 	for (wday = 0; wday < 7; wday++)
-		if (!strncasecmp (in, tm_days[wday], 3))
+		if (!g_strncasecmp (in, tm_days[wday], 3))
 			return wday;
 	
 	return -1;  /* unknown week day */
@@ -332,7 +349,7 @@ get_month (const unsigned char *in, unsigned int inlen)
 		return -1;
 	
 	for (i = 0; i < 12; i++)
-		if (!strncasecmp (in, tm_months[i], 3))
+		if (!g_strncasecmp (in, tm_months[i], 3))
 			return i;
 	
 	return -1;  /* unknown month */
@@ -709,6 +726,7 @@ g_mime_utils_header_decode_date (const char *in, int *saveoffset)
 char *
 g_mime_utils_header_fold (const char *in)
 {
+	gboolean last_was_lwsp = FALSE;
 	register const char *inptr;
 	size_t len, outlen, i;
 	GString *out;
@@ -724,7 +742,10 @@ g_mime_utils_header_fold (const char *in)
 	while (*inptr) {
 		len = strcspn (inptr, " \t");
 		
-		if (outlen + len > GMIME_FOLD_LEN) {
+		if (outlen + len > GMIME_FOLD_LEN) {			
+			if (last_was_lwsp)
+				g_string_truncate (out, out->len - 1);
+			
 			g_string_append (out, "\n\t");
 			outlen = 1;
 			
@@ -737,21 +758,33 @@ g_mime_utils_header_fold (const char *in)
 				g_string_append (out, "\n\t");
 				outlen = 1;
 			}
+			last_was_lwsp = FALSE;
 		} else if (len > 0) {
 			outlen += len;
-			for (i = 0; i < len; i++) {
-				g_string_append_c (out, inptr[i]);
-			}
+			g_string_append_len (out, inptr, len);
 			inptr += len;
+			last_was_lwsp = FALSE;
 		} else {
-			g_string_append_c (out, *inptr++);
+			if (*inptr == '\t') {
+				/* tabs are a good place to fold, odds
+                                   are that this is where the previous
+                                   mailer folded it */
+				g_string_append (out, "\n\t");
+				outlen = 1;
+				inptr++;
+				last_was_lwsp = FALSE;
+			} else {
+				g_string_append_c (out, *inptr++);
+				outlen++;
+				last_was_lwsp = TRUE;
+			}
 		}
 	}
 	
 	ret = out->str;
 	g_string_free (out, FALSE);
 	
-	return ret;	
+	return ret;
 }
 
 
@@ -893,7 +926,7 @@ g_mime_utils_unquote_string (char *string)
  * Determines if @text contains 8bit characters within the first @len
  * bytes.
  *
- * Returns TRUE if the text contains 8bit characters or FALSE
+ * Returns %TRUE if the text contains 8bit characters or %FALSE
  * otherwise.
  **/
 gboolean
@@ -920,7 +953,7 @@ g_mime_utils_text_is_8bit (const unsigned char *text, size_t len)
  * Determines the best content encoding for the first @len bytes of
  * @text.
  *
- * Returns a GMimePartEncodingType that is determined to be the best
+ * Returns a #GMimePartEncodingType that is determined to be the best
  * encoding type for the specified block of text. ("best" in this
  * particular case means best compression)
  **/
@@ -942,14 +975,13 @@ g_mime_utils_best_encoding (const unsigned char *text, size_t len)
 }
 
 /* this decodes rfc2047's version of quoted-printable */
-static ssize_t
+static gssize
 quoted_decode (const unsigned char *in, size_t len, unsigned char *out)
 {
 	register const unsigned char *inptr;
 	register unsigned char *outptr;
 	const unsigned char *inend;
 	unsigned char c, c1;
-	gboolean err = FALSE;
 	
 	inend = in + len;
 	outptr = out;
@@ -965,77 +997,108 @@ quoted_decode (const unsigned char *in, size_t len, unsigned char *out)
 					| ((c1 >= 'A' ? c1 - 'A' + 10 : c1 - '0') & 0x0f);
 			} else {
 				/* data was truncated */
-				err = TRUE;
-				break;
+				return -1;
 			}
 		} else if (c == '_') {
 			/* _'s are an rfc2047 shortcut for encoding spaces */
 			*outptr++ = ' ';
-		} else if (isblank (c) || strchr (CHARS_ESPECIAL, c)) {
-			/* FIXME: this is an error! ignore for now ... */
-			err = TRUE;
-			break;
 		} else {
 			*outptr++ = c;
 		}
 	}
 	
-	if (!err) {
-		return (outptr - out);
-	}
-	
-	return -1;
+	return (outptr - out);
 }
 
-#define is_8bit_word_encoded(atom, len) (len >= 7 && !strncmp (atom, "=?", 2) && !strncmp (atom + len - 2, "?=", 2))
+#define is_rfc2047_encoded_word(atom, len) (len >= 7 && !strncmp (atom, "=?", 2) && !strncmp (atom + len - 2, "?=", 2))
 
 static unsigned char *
-decode_encoded_8bit_word (const unsigned char *word)
+rfc2047_decode_word (const unsigned char *in, size_t inlen)
 {
 	const register unsigned char *inptr;
 	const unsigned char *inend;
+	const char *charset;
+	unsigned char *buf;
+	char *charenc, *p;
 	size_t len;
+	iconv_t cd;
 	
-	len = strlen (word);
-	
-	inptr = word + 2;
-	inend = word + len - 2;
+	inptr = in + 2;
+	inend = in + inlen - 2;
 	
 	inptr = memchr (inptr, '?', inend - inptr);
 	if (inptr && inptr[2] == '?') {
 		unsigned char *decoded;
-		ssize_t declen;
+		gssize declen;
 		int state = 0;
 		int save = 0;
 		
-		d(fprintf (stderr, "encoding is '%c'\n", inptr[0]));
-		
 		inptr++;
+		
 		switch (*inptr) {
 		case 'B':
 		case 'b':
 			inptr += 2;
-			decoded = alloca (inend - inptr);
+			decoded = g_alloca (inend - inptr);
 			declen = g_mime_utils_base64_decode_step (inptr, inend - inptr, decoded, &state, &save);
-			return g_strndup (decoded, declen);
 			break;
 		case 'Q':
 		case 'q':
 			inptr += 2;
-			decoded = alloca (inend - inptr);
+			decoded = g_alloca (inend - inptr);
 			declen = quoted_decode (inptr, inend - inptr, decoded);
 			
 			if (declen == -1) {
 				d(fprintf (stderr, "encountered broken 'Q' encoding\n"));
 				return NULL;
 			}
-			
-			return g_strndup (decoded, declen);
 			break;
 		default:
 			d(fprintf (stderr, "unknown encoding\n"));
 			return NULL;
 		}
+		
+		len = (inptr - 3) - (in + 2);
+		charenc = g_alloca (len + 1);
+		memcpy (charenc, in + 2, len);
+		charenc[len] = '\0';
+		charset = charenc;
+		
+		/* rfc2231 updates rfc2047 encoded words...
+		 * The ABNF given in RFC 2047 for encoded-words is:
+		 *   encoded-word := "=?" charset "?" encoding "?" encoded-text "?="
+		 * This specification changes this ABNF to:
+		 *   encoded-word := "=?" charset ["*" language] "?" encoding "?" encoded-text "?="
+		 */
+		
+		/* trim off the 'language' part if it's there... */
+		p = strchr (charset, '*');
+		if (p)
+			*p = '\0';
+		
+		/* slight optimization */
+		if (!g_strcasecmp (charset, "UTF-8"))
+			return g_strndup (decoded, declen);
+		
+		cd = g_mime_iconv_open ("UTF-8", charset);
+		if (cd == (iconv_t) -1) {
+			w(g_warning ("Cannot convert from %s to UTF-8, header display may "
+				     "be corrupt: %s", charset, g_strerror (errno)));
+			charset = g_mime_charset_locale_name ();
+			cd = g_mime_iconv_open ("UTF-8", charset);
+			if (cd == (iconv_t) -1)
+				return NULL;
+		}
+		
+		buf = g_mime_iconv_strndup (cd, decoded, declen);
+		g_mime_iconv_close (cd);
+		
+		if (!buf) {
+			w(g_warning ("Failed to convert \"%.*s\" to UTF-8, display may be "
+				     "corrupt: %s", declen, decoded, g_strerror (errno)));
+		}
+		
+		return buf;
 	}
 	
 	return NULL;
@@ -1073,8 +1136,8 @@ g_mime_utils_8bit_header_decode (const unsigned char *in)
 			const unsigned char *word;
 			gboolean was_encoded;
 			
-			if ((was_encoded = is_8bit_word_encoded (atom->str, atom->len)))
-				word = dword = decode_encoded_8bit_word (atom->str);
+			if ((was_encoded = is_rfc2047_encoded_word (atom->str, atom->len)))
+				word = dword = rfc2047_decode_word (atom->str, atom->len);
 			else
 				word = atom->str;
 			
@@ -1128,8 +1191,8 @@ g_mime_utils_8bit_header_decode (const unsigned char *in)
 		const unsigned char *word;
 		gboolean was_encoded;
 		
-		if ((was_encoded = is_8bit_word_encoded (atom->str, atom->len)))
-			word = dword = decode_encoded_8bit_word (atom->str);
+		if ((was_encoded = is_rfc2047_encoded_word (atom->str, atom->len)))
+			word = dword = rfc2047_decode_word (atom->str, atom->len);
 		else
 			word = atom->str;
 		
@@ -1187,21 +1250,38 @@ quoted_encode (const unsigned char *in, size_t len, unsigned char *out, gushort
 	return (outptr - out);
 }
 
-static char *
-encode_8bit_word (const unsigned char *word, gushort safemask, gboolean *this_was_encoded)
+static void
+rfc2047_encode_word (GString *string, const unsigned char *word, size_t len,
+		     const char *charset, gushort safemask)
 {
 	unsigned char *encoded, *ptr;
-	size_t enclen, pos, len;
+	unsigned char *uword = NULL;
+	iconv_t cd = (iconv_t) -1;
+	size_t enclen, pos;
 	int state = 0;
 	int save = 0;
 	char encoding;
 	
-	len = strlen (word);
+	
+	if (g_strcasecmp (charset, "UTF-8") != 0)
+		cd = g_mime_iconv_open (charset, "UTF-8");
+	
+	if (cd != (iconv_t) -1) {
+		uword = g_mime_iconv_strndup (cd, word, len);
+		g_mime_iconv_close (cd);
+	}
+	
+	if (uword) {
+		len = strlen (uword);
+		word = uword;
+	} else {
+		charset = "UTF-8";
+	}
 	
 	switch (g_mime_utils_best_encoding (word, len)) {
 	case GMIME_PART_ENCODING_BASE64:
 		enclen = BASE64_ENCODE_LEN (len);
-		encoded = alloca (enclen);
+		encoded = g_alloca (enclen);
 		
 		encoding = 'b';
 		
@@ -1216,7 +1296,7 @@ encode_8bit_word (const unsigned char *word, gushort safemask, gboolean *this_wa
 		break;
 	case GMIME_PART_ENCODING_QUOTEDPRINTABLE:
 		enclen = QP_ENCODE_LEN (len);
-		encoded = alloca (enclen);
+		encoded = g_alloca (enclen);
 		
 		encoding = 'q';
 		
@@ -1225,16 +1305,12 @@ encode_8bit_word (const unsigned char *word, gushort safemask, gboolean *this_wa
 		
 		break;
 	default:
-		if (this_was_encoded)
-			*this_was_encoded = FALSE;
-		
-		return g_strdup (word);
+		g_assert_not_reached ();
 	}
 	
-	if (this_was_encoded)
-		*this_was_encoded = TRUE;
+	g_free (uword);
 	
-	return g_strdup_printf ("=?%s?%c?%s?=", g_mime_charset_locale_name (), encoding, encoded);	
+	g_string_sprintfa (string, "=?%s?%c?%s?=", charset, encoding, encoded);
 }
 
 
@@ -1250,7 +1326,241 @@ encode_8bit_word (const unsigned char *word, gushort safemask, gboolean *this_wa
 char *
 g_mime_utils_8bit_header_encode_phrase (const unsigned char *in)
 {
-	return encode_8bit_word (in, IS_PSAFE, NULL);
+	const char *charset;
+	GString *string;
+	size_t len;
+	char *str;
+	
+	if (in == NULL)
+		return NULL;
+	
+	len = strlen (in);
+	
+	charset = g_mime_charset_best (in, len);
+	charset = charset ? charset : "iso-8859-1";
+	
+	string = g_string_new ("");
+	
+	rfc2047_encode_word (string, in, strlen (in), charset, IS_ESAFE);
+	
+	str = string->str;
+	g_string_free (string, FALSE);
+	
+	return str;
+}
+
+
+enum _phrase_word_t {
+	WORD_ATOM,
+	WORD_2047
+};
+
+struct _phrase_word {
+	struct _phrase_word *next;
+	const unsigned char *start, *end;
+	enum _phrase_word_t type;
+	int encoding;
+};
+
+static gboolean
+word_types_compatable (enum _phrase_word_t type1, enum _phrase_word_t type2)
+{
+	switch (type1) {
+	case WORD_ATOM:
+		return FALSE;
+	case WORD_2047:
+		return type2 == WORD_2047;
+	default:
+		return FALSE;
+	}
+}
+
+static struct _phrase_word *
+rfc2047_encode_phrase_get_words (const unsigned char *in)
+{
+	const unsigned char *inptr, *start, *last;
+	struct _phrase_word *words, *tail, *word;
+	enum _phrase_word_t type = WORD_ATOM;
+	int count = 0, encoding = 0;
+	
+	words = NULL;
+	tail = (struct _phrase_word *) &words;
+	
+	last = start = inptr = in;
+	while (inptr && *inptr) {
+		const char *newinptr;
+		gunichar c;
+		
+		newinptr = g_utf8_next_char (inptr);
+		c = g_utf8_get_char (inptr);
+		if (newinptr == NULL || !g_unichar_validate (c)) {
+			w(g_warning ("Invalid UTF-8 sequence encountered"));
+			inptr++;
+			continue;
+		}
+		
+		inptr = newinptr;
+		
+		if (g_unichar_isspace (c)) {
+			if (count > 0) {
+				word = g_new (struct _phrase_word, 1);
+				word->next = NULL;
+				word->start = start;
+				word->end = last;
+				word->type = type;
+				word->encoding = encoding;
+				
+				tail->next = word;
+				tail = word;
+				count = 0;
+			}
+			
+			start = inptr;
+			type = WORD_ATOM;
+			encoding = 0;
+		} else {
+			count++;
+			if (c > 127 && c < 256) {
+				type = WORD_2047;
+				encoding = MAX (encoding, 2);
+			} else if (c >= 256) {
+				type = WORD_2047;
+				encoding = 2;
+			}
+		}
+		
+		last = inptr;
+	}
+	
+	if (count > 0) {
+		word = g_new (struct _phrase_word, 1);
+		word->next = NULL;
+		word->start = start;
+		word->end = last;
+		word->type = type;
+		word->encoding = encoding;
+		
+		tail->next = word;
+		tail = word;
+	}
+	
+	return words;
+}
+
+static gboolean
+rfc2047_encode_phrase_merge_words (struct _phrase_word **wordsp)
+{
+	struct _phrase_word *word, *next, *words = *wordsp;
+	gboolean merged = FALSE;
+	
+	/* scan the list, checking for words of similar types that can be merged */
+	word = words;
+	while (word) {
+		next = word->next;
+		
+		while (next) {
+			/* merge nodes of the same type AND we are not creating too long a string */
+			if (word_types_compatable (word->type, next->type)) {
+				if (next->end - word->start < GMIME_FOLD_PREENCODED) {
+					/* the resulting word type is the MAX of the 2 types */
+					word->type = MAX (word->type, next->type);
+					
+					word->end = next->end;
+					word->next = next->next;
+					
+					g_free (next);
+					
+					next = word->next;
+					
+					merged = TRUE;
+				} else {
+					/* if it is going to be too long, make sure we include the
+					   separating whitespace */
+					word->end = next->start;
+					break;
+				}
+			} else {
+				break;
+			}
+		}
+		
+		word = word->next;
+	}
+	
+	*wordsp = words;
+	
+	return merged;
+}
+
+static char *
+rfc2047_encode_phrase (const unsigned char *in)
+{
+	struct _phrase_word *words, *word, *prev = NULL;
+	GString *out;
+	char *outstr;
+	
+	if (in == NULL)
+		return NULL;
+	
+	words = rfc2047_encode_phrase_get_words (in);
+	if (!words)
+		return NULL;
+	
+	while (rfc2047_encode_phrase_merge_words (&words))
+		;
+	
+	out = g_string_new ("");
+	
+	/* output words now with spaces between them */
+	word = words;
+	while (word) {
+		const char *start;
+		size_t len;
+		
+		/* append correct number of spaces between words */
+		if (prev && !(prev->type == WORD_2047 && word->type == WORD_2047)) {
+			/* one or both of the words are not encoded so we write the spaces out untouched */
+			len = word->start - prev->end;
+			g_string_append_len (out, prev->end, len);
+		}
+		
+		switch (word->type) {
+		case WORD_ATOM:
+			g_string_append_len (out, word->start, word->end - word->start);
+			break;
+		case WORD_2047:
+			if (prev && prev->type == WORD_2047) {
+				/* include the whitespace chars between these 2 words in the
+                                   resulting rfc2047 encoded word. */
+				len = word->end - prev->end;
+				start = prev->end;
+				
+				/* encoded words need to be separated by linear whitespace */
+				g_string_append_c (out, ' ');
+			} else {
+				len = word->end - word->start;
+				start = word->start;
+			}
+			
+			if (word->encoding == 1)
+				rfc2047_encode_word (out, start, len, "iso-8859-1", IS_PSAFE);
+			else
+				rfc2047_encode_word (out, start, len,
+						     g_mime_charset_best (start, len), IS_PSAFE);
+			break;
+		}
+		
+		g_free (prev);
+		prev = word;
+		word = word->next;
+	}
+	
+	g_free (prev);
+	
+	outstr = out->str;
+	g_string_free (out, FALSE);
+	
+	return outstr;
 }
 
 
@@ -1266,104 +1576,7 @@ g_mime_utils_8bit_header_encode_phrase (const unsigned char *in)
 char *
 g_mime_utils_8bit_header_encode (const unsigned char *in)
 {
-	register const unsigned char *inptr;
-	GString *out, *word, *lwsp;
-	unsigned char *encoded;
-	gboolean is8bit = FALSE;
-	gboolean last_was_encoded = FALSE;
-	gboolean last_was_space = FALSE;
-	
-	out = g_string_new ("");
-	word = g_string_new ("");
-	lwsp = g_string_new ("");
-	inptr = (guchar *) in;
-	
-	while (inptr && *inptr) {
-		unsigned char c = *inptr++;
-		
-		if (isspace (c) && !last_was_space) {
-			gboolean this_was_encoded = FALSE;
-			char *eword;
-			
-			if (is8bit)
-				eword = encode_8bit_word (word->str, IS_ESAFE, &this_was_encoded);
-			else
-				eword = g_strdup (word->str);
-			
-			/* append any whitespace */
-			if (last_was_encoded && this_was_encoded) {
-				/* we need to encode the whitespace */
-				unsigned char *elwsp;
-				size_t len;
-				
-				elwsp = alloca (lwsp->len * 3 + 4);
-				len = quoted_encode (lwsp->str, lwsp->len, elwsp, IS_SPACE);
-				elwsp[len] = '\0';
-				
-				g_string_sprintfa (out, " =?%s?q?%s?= ", g_mime_charset_locale_name (), elwsp);
-			} else {
-				g_string_append (out, lwsp->str);
-			}
-			
-			/* append the encoded word */
-			g_string_append (out, eword);
-			g_free (eword);
-			
-			g_string_truncate (lwsp, 0);
-			g_string_truncate (word, 0);
-			
-			last_was_encoded = this_was_encoded;
-			is8bit = FALSE;
-		}
-		
-		if (isspace (c)) {
-			g_string_append_c (lwsp, c);
-			last_was_space = TRUE;
-		} else {
-			if (c > 127)
-				is8bit = TRUE;
-			
-			g_string_append_c (word, c);
-			last_was_space = FALSE;
-		}
-	}
-	
-	if (word->len || lwsp->len) {
-		gboolean this_was_encoded = FALSE;
-		char *eword;
-		
-		if (is8bit)
-			eword = encode_8bit_word (word->str, IS_ESAFE, &this_was_encoded);
-		else
-			eword = g_strdup (word->str);
-		
-		/* append any whitespace */
-		if (last_was_encoded && this_was_encoded) {
-			/* we need to encode the whitespace */
-			unsigned char *elwsp;
-			size_t len;
-			
-			elwsp = alloca (lwsp->len * 3 + 4);
-			len = quoted_encode (lwsp->str, lwsp->len, elwsp, IS_SPACE);
-			elwsp[len] = '\0';
-			
-			g_string_sprintfa (out, " =?%s?q?%s?= ", g_mime_charset_locale_name (), elwsp);
-		} else {
-			g_string_append (out, lwsp->str);
-		}
-		
-		/* append the encoded word */
-		g_string_append (out, eword);
-		g_free (eword);
-	}
-	
-	g_string_free (lwsp, TRUE);
-	g_string_free (word, TRUE);
-	
-	encoded = out->str;
-	g_string_free (out, FALSE);
-	
-	return encoded;
+	return rfc2047_encode_phrase (in);
 }
 
 
@@ -1376,8 +1589,8 @@ g_mime_utils_8bit_header_encode (const unsigned char *in)
  * @save: leftover bits that have not yet been encoded
  *
  * Base64 encodes the input stream to the output stream. Call this
- * when finished encoding data with g_mime_utils_base64_encode_step to
- * flush off the last little bit.
+ * when finished encoding data with #g_mime_utils_base64_encode_step
+ * to flush off the last little bit.
  *
  * Returns the number of bytes encoded.
  **/
@@ -1443,8 +1656,6 @@ g_mime_utils_base64_encode_step (const unsigned char *in, size_t inlen, unsigned
 	inptr = in;
 	outptr = out;
 	
-	d(printf("we have %d chars, and %d saved chars\n", inlen, ((gchar *)save)[0]));
-	
 	if (inlen + ((unsigned char *)save)[0] > 2) {
 		const unsigned char *inend = in + inlen - 2;
 		register int c1 = 0, c2 = 0, c3 = 0;
@@ -1505,6 +1716,7 @@ g_mime_utils_base64_encode_step (const unsigned char *in, size_t inlen, unsigned
 	return (outptr - out);
 }
 
+
 /**
  * g_mime_utils_base64_decode_step:
  * @in: input stream
@@ -1557,7 +1769,7 @@ g_mime_utils_base64_decode_step (const unsigned char *in, size_t inlen, unsigned
 	while (inptr > in && i) {
 		inptr--;
 		if (gmime_base64_rank[*inptr] != 0xff) {
-			if (*inptr == '=')
+			if (*inptr == '=' && outptr > out)
 				outptr--;
 			i--;
 		}
@@ -1578,7 +1790,7 @@ g_mime_utils_base64_decode_step (const unsigned char *in, size_t inlen, unsigned
  * @save: leftover bits that have not yet been encoded
  *
  * Uuencodes a chunk of data. Call this when finished encoding data
- * with g_mime_utils_uuencode_step to flush off the last little bit.
+ * with #g_mime_utils_uuencode_step to flush off the last little bit.
  *
  * Returns the number of bytes encoded.
  **/
@@ -1730,7 +1942,7 @@ g_mime_utils_uuencode_step (const unsigned char *in, size_t inlen, unsigned char
 /**
  * g_mime_utils_uudecode_step:
  * @in: input stream
- * @inlen: max length of data to decode (normally strlen (in) ??)
+ * @inlen: max length of data to decode
  * @out: output stream
  * @state: holds the number of bits that are stored in @save
  * @save: leftover bits that have not yet been decoded
@@ -1839,8 +2051,8 @@ g_mime_utils_uudecode_step (const unsigned char *in, size_t inlen, unsigned char
  * @save: leftover bits that have not yet been encoded
  *
  * Quoted-printable encodes a block of text. Call this when finished
- * encoding data with g_mime_utils_quoted_encode_step to flush off the
- * last little bit.
+ * encoding data with #g_mime_utils_quoted_encode_step to flush off
+ * the last little bit.
  *
  * Returns the number of bytes encoded.
  **/
