@@ -22,6 +22,7 @@
 #include <string.h>
 
 #include <pan/base/acache.h>
+#include <pan/base/argset.h>
 #include <pan/base/article.h>
 #include <pan/base/debug.h>
 #include <pan/base/pan-glib-extensions.h>
@@ -208,15 +209,19 @@ article_supersede (GMimeMessage * message)
 	}
 	else
 	{
-		const char * old_message_id;
-		const char * domain;
+		char * domain;
+		char * old_message_id;
 		char * new_message_id;
 
 		/* get the new message-id */
-		old_message_id = g_mime_message_get_message_id (ours);
+		old_message_id = g_strdup (g_mime_message_get_message_id (ours));
 		domain = strchr (old_message_id, '@');
 		if (domain)
-			++domain;
+		{
+			size_t len = strlen (++domain);
+			if (domain[len-1] == '>')
+				domain[len-1]='\0';
+		}
 		new_message_id = gnksa_generate_message_id (domain);
 
 		/* update the headers for superseding */
@@ -228,6 +233,7 @@ article_supersede (GMimeMessage * message)
 
 		/* cleanup */
 		g_free (new_message_id);
+		g_free (old_message_id);
 	}
 
 	debug_exit ("article_supersede");
@@ -243,12 +249,17 @@ article_copy_articles_to_folder_cb (gpointer obj, gpointer arg, gpointer data)
 	int status;
 	Task * task;
 	Group * folder;
+	ArgSet * args;
+	char * acache_key;
 	debug_enter ("article_copy_articles_to_folder_cb");
 
 	/* sanity clause */
 	g_return_if_fail (obj!=NULL);
-	g_return_if_fail (data!=NULL);
-	folder = (Group*) data;
+	args = (ArgSet*) data;
+	g_return_if_fail (args!=NULL);
+	acache_key = (char*) argset_get (args, 0);
+	g_return_if_fail (is_nonempty_string (acache_key));
+	folder = (Group*) argset_get (args, 1);
 	g_return_if_fail (group_is_valid(folder));
 	g_return_if_fail (group_is_folder(folder));
 
@@ -263,14 +274,15 @@ article_copy_articles_to_folder_cb (gpointer obj, gpointer arg, gpointer data)
 		for (i=0; i<task->identifiers->len; ++i)
 		{
 			MessageIdentifier * mid = MESSAGE_IDENTIFIER (g_ptr_array_index (task->identifiers, i));
-			GMimeMessage * message = acache_get_message (&mid, 1);
+			GMimeMessage * message = acache_get_message (acache_key, &mid, 1);
 			folder_add_message (folder, message);
 			g_object_unref (message);
 		}
-
-		/* clean up */
 	}
 
+	/* cleanup */
+	g_free (acache_key);
+	argset_free (args);
 	debug_exit ("articlelist_article_to_folder_cb");
 }
 
@@ -290,7 +302,9 @@ article_copy_articles_to_folder (Group * folder, const Article ** articles, int
 	/* make a task to download the bodies in case we don't already have them */
 	task = TASK(task_bodies_new_from_articles (articles, qty));
 	if (task != NULL) {
-		pan_callback_add (task->task_ran_callback, article_copy_articles_to_folder_cb, folder);
+		const char * source_key = group_get_acache_key (articles[0]->group);
+		ArgSet * args = argset_new2 (g_strdup(source_key), folder);
+		pan_callback_add (task->task_ran_callback, article_copy_articles_to_folder_cb, args);
 		queue_add (task);
 	}
 
@@ -329,7 +343,7 @@ save_attachments_impl (Article ** articles, int len)
 
 		/* queue for decode */
 		task_save_weed_duplicates (tmp);
-		task = task_save_new_from_articles (article_get_subject(a), (const Article**)tmp->pdata, tmp->len);
+		task = task_save_new_from_articles ((const Article**)tmp->pdata, tmp->len);
 		task_save_set_attachments (TASK_SAVE(task), NULL, NULL);
 		tasks = g_slist_prepend (tasks, task);
 
@@ -496,7 +510,7 @@ delete_articles (GPtrArray * articles)
 				Article * a = ARTICLE(g_ptr_array_index(articles,i));
 				g_ptr_array_add (ids, (gpointer)article_get_message_id(a));
 			}
-			acache_expire_messages ((const gchar **)ids->pdata, ids->len);
+			acache_expire_messages (group_get_acache_key(group), (const gchar **)ids->pdata, ids->len);
 			g_ptr_array_free (ids, TRUE);
 		}
 
@@ -563,7 +577,7 @@ article_action_edit_selected (void)
 	if (a != NULL)
 		mid = message_identifier_new_from_article (a);
 	if (mid != NULL)
-		message = acache_get_message (&mid, 1);
+		message = acache_get_message (group_get_acache_key(a->group), &mid, 1);
 	if (message != NULL)
 		message_edit_window (message);
 
