@@ -201,16 +201,21 @@ activate_me_cb (GtkWidget * w, gpointer user_data)
 GtkWidget*
 article_filter_dialog_new (GtkWidget * window, const Article * a)
 {
-	gchar * pch;
-	gint row = 0;
+	int row = 0;
 	GtkWidget * w;
 	GtkWidget * t;
 	GtkWidget * tb;
 	ArticleFilterDialog * d;
-	gchar author[256];
-	const gint atch = GTK_EXPAND | GTK_SHRINK | GTK_FILL;
+	char author[256];
+	char * subject;
+	const int atch = GTK_EXPAND | GTK_SHRINK | GTK_FILL;
 	const int wide_enough = 250;
 
+	/* filter criteria */
+	article_get_author_str (a, author, sizeof(author));
+	subject = g_strdup (article_get_subject(a));
+	g_strstrip (subject);
+
 	/* dialog */
 	d = g_new0 (ArticleFilterDialog, 1);
 	w = d->dialog = gtk_dialog_new_with_buttons (_("Pan: Filter Out Articles"),
@@ -237,8 +242,7 @@ article_filter_dialog_new (GtkWidget * window, const Article * a)
 	w = gtk_entry_new ();
 	gtk_widget_set_usize (w, wide_enough, -1);
 	d->bozo_author_entry = w;
-	article_get_author_str (a, author, sizeof(author));
-	pan_gtk_entry_set_text (w, author);
+	gtk_entry_set_text (GTK_ENTRY(w), author);
 	g_signal_connect (w, "changed", G_CALLBACK(activate_me_cb), tb);
 	gtk_table_attach (GTK_TABLE(t), w, 1, 2, row, row+1, atch, atch, 0, 0);
 	w = gtk_toggle_button_new_with_label (_("Regular Expression"));
@@ -253,8 +257,7 @@ article_filter_dialog_new (GtkWidget * window, const Article * a)
 	w = gtk_entry_new ();
 	gtk_widget_set_usize (w, wide_enough, -1);
 	d->spam_author_entry = w;
-	article_get_author_str (a, author, sizeof(author));
-	pan_gtk_entry_set_text (w, author);
+	gtk_entry_set_text (GTK_ENTRY(w), author);
 	g_signal_connect (w, "changed", G_CALLBACK(activate_me_cb), tb);
 	gtk_table_attach (GTK_TABLE(t), w, 1, 2, row, row+1, atch, atch, 0, 0);
 	w = gtk_toggle_button_new_with_label (_("Regular Expression"));
@@ -269,10 +272,7 @@ article_filter_dialog_new (GtkWidget * window, const Article * a)
 	w = gtk_entry_new ();
 	gtk_widget_set_usize (w, wide_enough, -1);
 	d->spam_subject_entry = w;
-	pch = g_strdup (article_get_subject(a));
-	g_strstrip (pch);
-	pan_gtk_entry_set_text (w, pch);
-	g_free (pch);
+	gtk_entry_set_text (GTK_ENTRY(w), subject);
 	g_signal_connect (w, "changed", G_CALLBACK(activate_me_cb), tb);
 	gtk_table_attach (GTK_TABLE(t), w, 1, 2, row, row+1, atch, atch, 0, 0);
 	w = gtk_toggle_button_new_with_label (_("Regular Expression"));
@@ -280,5 +280,8 @@ article_filter_dialog_new (GtkWidget * window, const Article * a)
 	g_signal_connect (w, "clicked", G_CALLBACK(activate_me_cb), tb);
 	gtk_table_attach (GTK_TABLE(t), w, 2, 3, row, row+1, atch, atch, 0, 0);
 
+	/* cleanup */
+	g_free (subject);
+
 	return d->dialog;
 }
