@@ -40,7 +40,6 @@
 
 static void fire_user_changed_filter (void);
 
-static GtkWidget * _filter_phrase_om = NULL;
 static GtkWidget * _filter_phrase_entry = NULL;
 
 /*
@@ -102,7 +101,10 @@ merge_filter_with_phrase (Filter * filter)
 	if (is_nonempty_string (phrase_str))
 	{
 		Filter * phrase_filter = filter_phrase_new ();
-		Filter * children[2] = {filter, phrase_filter};
+		Filter * children[2];
+
+		children[0] = filter;
+		children[1] = phrase_filter;
 
 		filter_phrase_set (FILTER_PHRASE(phrase_filter),
 		                   get_filter_phrase_state(),
@@ -322,7 +324,6 @@ article_toolbar_new (void)
 		w = option_menu;
 	}
 	gtk_box_pack_start (GTK_BOX(toolbar), w, FALSE, FALSE, 0);
-	_filter_phrase_om = w;
 	_filter_phrase_entry = w = gtk_entry_new ();
 	g_signal_connect (w, "focus_in_event", G_CALLBACK(entry_focus_in_cb), NULL);
 	g_signal_connect (w, "focus_out_event", G_CALLBACK(entry_focus_out_cb), NULL);
