@@ -117,9 +117,9 @@ PanCallback* articlelist_get_group_changed_callback (void);
  * called whenever the article tree's selection state changes.
  *
  * @param call_object the article ctree
- * @param call_arg article of the selected article, or NULL if none
+ * @param call_arg GPtrArray of the selected Article*s
  */
-extern PanCallback* articlelist_selection_changed;
+PanCallback* articlelist_get_selection_changed_callback (void);
 
 
 /***
