@@ -46,13 +46,16 @@
 #include <pan/base/util-mime.h>
 #include <pan/base/log.h>
 
-PanCallback* acache_bodies_added_callback = NULL;
-PanCallback* acache_bodies_removed_callback = NULL;
-
 static GStaticMutex entries_mutex = G_STATIC_MUTEX_INIT;
 static MemChunk * acache_entry_chunk = NULL;
-static GHashTable * messageid_to_entry = NULL;
+static GHashTable * key_to_messageids = NULL;
+
 static GHashTable * key_to_path = NULL;
+static void acache_add_path (const char * key, const char * path);
+
+/***
+****
+***/
 
 /**
  * This represents one directory (folder) in the cache
@@ -87,71 +90,41 @@ const char * ACACHE_DEFAULT_KEY = "cache";
 
 /***
 ****
+****  EVENT NOTIFICATION
+****
 ***/
 
-static void
-acache_path_free (AcachePath * path)
+PanCallback*
+acache_get_bodies_added_callback (void)
 {
-	g_free (path->key);
-	g_free (path->path);
-	g_free (path);
+	static PanCallback * cb = NULL;
+	if (cb==NULL) cb = pan_callback_new ();
+	return cb;
 }
 
-static gboolean
-get_message_id_from_file (char * msgid, int msgid_len, const char * filename)
+static void
+fire_bodies_added (const char ** message_ids, int qty)
 {
-	char buf [2048];
-	char * retval;
-	FILE * fp;
-	debug_enter ("get_message_id_from_file");
+	pan_callback_call (acache_get_bodies_added_callback(), message_ids, GINT_TO_POINTER(qty));
+}
 
-	g_return_val_if_fail (is_nonempty_string(filename), FALSE);
-	g_return_val_if_fail (msgid!=NULL, FALSE);
-	g_return_val_if_fail (msgid_len!=0, FALSE);
+PanCallback*
+acache_get_bodies_removed_callback (void)
+{
+	static PanCallback * cb = NULL;
+	if (cb==NULL) cb = pan_callback_new ();
+	return cb;
+}
 
-	/* try to find the line */
-	*buf = '\0';
-	fp = fopen (filename, "r");
-	if (fp != NULL) {
-		const char * key = "Message-I";
-		const size_t key_len = 9;
-		gboolean found;
-		for (found=FALSE; !found && fgets(buf,sizeof(buf),fp); ) {
-			if (buf[0]=='\0' || buf[1]=='\0') /* watch for header/body delimiter */
-				break;
-			if (!memcmp (key, buf, key_len))
-				found = TRUE;
-		}
-		if (!found)
-			*buf = '\0';
-		fclose (fp);
-	}
-
-	/* parse the line */
-	retval = NULL;
-	if (*buf != '\0') {
-		char * pch = strchr (buf, ':');
-		if (pch != NULL) {
-			++pch;
-			while (*pch && isspace((guchar)*pch))
-				++pch;
-			retval = pch;
-			while (*pch && !isspace((guchar)*pch))
-				++pch;
-			*pch = '\0';
-		}
-	}
-
-	if (retval != NULL) 
-		g_snprintf (msgid, msgid_len, "%s", retval);
-
-	debug_exit ("get_message_id_from_file");
-	return retval != NULL;
+static void
+fire_bodies_removed (const char ** message_ids, int qty)
+{
+	pan_callback_call (acache_get_bodies_removed_callback(), message_ids, GINT_TO_POINTER(qty));
 }
 
 /***
 ****
-****  PRIVATE UTIL
+****  DIRECTORIES & FILENAMES
 ****
 ***/
 
@@ -169,44 +142,266 @@ acache_get_folder_base (void)
 	return retval;
 }
 
+/**
+ * Some characters in message-ids don't work well in file names,
+ * so we transform them to a safer name.
+ */
+static char*
+acache_message_id_to_basename (char * buf, int len, const char * message_id)
+{
+	const char * in;
+	char * out;
+	debug_enter ("acache_message_id_to_basename");
+
+	/* sanity clause */
+	g_return_val_if_fail (is_nonempty_string(message_id), NULL);
+	g_return_val_if_fail (buf!=NULL, NULL);
+	g_return_val_if_fail (len>0, NULL);
+
+	/* transform */
+	for (in=message_id, out=buf; *in; ++in) {
+		switch (*in) {
+			case '_': *out++ = '_'; *out++ = '_'; break;
+			case '%': *out++ = '_'; *out++ = 'p'; break;
+			case '$': *out++ = '_'; *out++ = 'd'; break;
+			case '/': *out++ = '_'; *out++ = 's'; break;
+			case '\\': *out++ = '_'; *out++ = 'b'; break;
+			case '<': break;
+			case '>': break;
+			default: *out++ = *in; break;
+		}
+	}
+	strcpy (out, ".msg");
+
+	debug_exit ("acache_message_id_to_basename");
+	return buf;
+}
+
+/**
+ * Message-IDs are transformed via acache_message_id_to_basename()
+ * to play nicely with some filesystems, so to extract the Message-ID
+ * from a basename we need to reverse the transform.
+ */
+static char*
+acache_basename_to_message_id (char * buf, int len, const char * basename)
+{
+	const char * in;
+	char * out;
+	char * pch;
+	char tmp_basename[PATH_MAX];
+	debug_enter ("acache_basename_to_message_id");
+
+	/* sanity clause */
+	g_return_val_if_fail (is_nonempty_string(basename), NULL);
+	g_return_val_if_fail (buf!=NULL, NULL);
+	g_return_val_if_fail (len>0, NULL);
+
+	/* remove the trailing ".msg" */
+	g_strlcpy (tmp_basename, basename, sizeof(tmp_basename));
+	if ((pch = g_strrstr (tmp_basename, ".msg")))
+		*pch = '\0';
+	g_strstrip (tmp_basename);
+
+	/* transform */
+	out = buf;
+	*out++ = '<';
+	for (in=tmp_basename; *in; ++in) {
+		if (*in != '_')
+			*out++ = *in;
+		else switch (*++in) {
+			case '_': *out++ = '_'; break;
+			case 'p': *out++ = '%'; break;
+			case 'd': *out++ = '$'; break;
+			case 's': *out++ = '/'; break;
+			case 'b': *out++ = '\\'; break;
+		}
+	}
+	*out++ = '>';
+	*out = '\0';
+
+	/* cleanup */
+	debug_exit ("acache_basename_to_message_id");
+	return buf;
+}
+
 static char*
 acache_get_filename (char * buf, int len, const char * path, const char * message_id)
 {
-	register char * pch;
-	const register char * cpch;
-	const int path_len = strlen (path);
-	debug_enter ("acache_get_filename");
+	char basename[PATH_MAX] = { '\0' };
 
 	/* sanity clause */
-	g_return_val_if_fail (buf!=NULL, NULL);
 	g_return_val_if_fail (is_nonempty_string(message_id), NULL);
-	g_return_val_if_fail (len >= (path_len + 1 + strlen(message_id) + 4), NULL);
+	g_return_val_if_fail (buf!=NULL, NULL);
+	g_return_val_if_fail (len>0, NULL);
 
-	/* build acache string.  path/msg-id.msg */
-	pch = g_stpcpy (buf, path);
-	*pch++ = G_DIR_SEPARATOR;
-	for (cpch=message_id; *cpch!='\0'; ++cpch)
-		if (*cpch!='%' && *cpch!='$' && *cpch!='<' && *cpch!='>' && *cpch!='/' && *cpch!='\\')
-			*pch++ = *cpch;
-	strcpy (pch, ".msg");
-	pan_file_normalize_inplace (buf);
+	/* build the filename */
+	*buf = '\0';
+	if (acache_message_id_to_basename (basename, sizeof(basename), message_id)) {
+		g_snprintf (buf, len, "%s%c%s", path, G_DIR_SEPARATOR, basename);
+		pan_file_normalize_inplace (buf);
+	}
 
-	debug_exit ("acache_get_filename");
 	return is_nonempty_string(buf) ? buf : NULL;
 }
 
-static gpointer
-add_folder_directories (gpointer data)
+
+/***
+****
+****  AcacheEntry Management
+****
+***/
+
+static GHashTable*
+acache_get_key_path_hash (const char * key_path)
 {
-	Server * server = (Server *) data;
+	GHashTable * retval = NULL;
+
+	/* sanity clause */
+	g_return_val_if_fail (is_nonempty_string (key_path), NULL);
+
+	/* ensure there's a hash table for this path. */
+	retval = g_hash_table_lookup (key_to_messageids, key_path);
+	if (retval == NULL) {
+		retval = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+		g_hash_table_insert (key_to_messageids, g_strdup(key_path), retval);
+	}
+	
+	return retval;
+}
+
+static AcacheEntry*
+acache_lookup_entry (const char* key_path, const char* message_id)
+{
+	GHashTable * message_ids;
+
+	g_return_val_if_fail (is_nonempty_string(key_path), NULL);
+	g_return_val_if_fail (is_nonempty_string(message_id), NULL);
+
+	message_ids = acache_get_key_path_hash (key_path);
+
+	return (AcacheEntry*) g_hash_table_lookup (message_ids, message_id);
+}
+
+static AcacheEntry*
+acache_get_entry (const char * key_path, const char * message_id)
+{
+	AcacheEntry * entry = NULL;
+	GHashTable * message_ids;
+
+	g_return_val_if_fail (is_nonempty_string(key_path), NULL);
+	g_return_val_if_fail (is_nonempty_string(message_id), NULL);
+
+	message_ids = acache_get_key_path_hash (key_path);
+	entry = g_hash_table_lookup (message_ids, message_id);
+	if (entry == NULL)
+	{
+		entry = (AcacheEntry*) memchunk_alloc (acache_entry_chunk);
+		entry->path = NULL;
+		entry->refcount = 0;
+		entry->message_id = g_strdup (message_id);
+		entry->size = 0;
+		entry->date = time (NULL);
+		g_hash_table_replace (message_ids, entry->message_id, entry);
+		debug1 (DEBUG_ACACHE, "Added new entry %d", entry->message_id);
+	}
+
+	return entry;
+}
+
+/**
+ * This must be called from inside an entries_mutex lock.
+ */
+static void
+acache_remove_entry (AcacheEntry * entry)
+{
+	GHashTable * message_ids;
+	char filename[PATH_MAX] = { '\0' };
+
+	/* sanity clause */
+	g_return_if_fail (entry!=NULL);
+	g_return_if_fail (entry->path!=NULL);
+	g_return_if_fail (is_nonempty_string(entry->message_id));
+
+	/* notify listeners that this message is going away */
+	debug2 (DEBUG_ACACHE, "Removing %s from cache with refcount %d", entry->message_id, entry->refcount);
+	fire_bodies_removed ((const char**)&entry->message_id, 1);
+
+	/* remove the file */
+	acache_get_filename (filename, sizeof(filename), entry->path->path, entry->message_id);
+	debug1 (DEBUG_ACACHE, "unlinking [%s]", filename);
+	unlink (filename);
+	entry->path->size -= entry->size;
+
+	/* remove it from the lookup table */
+	message_ids = acache_get_key_path_hash (entry->path->key);
+	g_hash_table_remove (message_ids, entry->message_id);
+	entry->message_id = NULL;
+
+	/* free the entry struct */
+	memchunk_free (acache_entry_chunk, entry);
+	entry = NULL;
+}
+/***
+****
+****
+****
+***/
+
+static void
+acache_path_free (AcachePath * path)
+{
+	g_free (path->key);
+	g_free (path->path);
+	g_free (path);
+}
+
+static gboolean
+get_message_id_from_file (char * msgid, int msgid_len, const char * filename)
+{
+	const char * basename;
+	gboolean retval = FALSE;
+
+	/* sanity clause */
+	g_return_val_if_fail (is_nonempty_string(filename), FALSE);
+	g_return_val_if_fail (msgid!=NULL, FALSE);
+	g_return_val_if_fail (msgid_len>0, FALSE);
+	
+	/* get the message-id */
+	*msgid = '\0';
+	basename = g_basename (filename);
+	if (basename != NULL)
+		retval = acache_basename_to_message_id (msgid, msgid_len, basename) != NULL;
+
+	return retval;
+}
+
+void
+acache_init (void)
+{
+	Server *server;
 	char * message_base;
 	char * folder_base;
 	char * path;
 	GDir * dir;
 	GError * err;
-	debug_enter ("add_folder_directories");
+	debug_enter ("acache_init");
 
-	/* ensure the default directories exist */
+	/* sanity clause -- don't init twice */
+	if (acache_entry_chunk != NULL)
+		return;
+
+	/* init the message id hash  */
+	acache_entry_chunk = memchunk_new (sizeof(AcacheEntry), ACACHE_ENTRY_CHUNK_SIZE, FALSE);
+	key_to_path = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, (GDestroyNotify)acache_path_free);
+	key_to_messageids = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, (GDestroyNotify)g_hash_table_destroy);
+
+	/* add the folder server */
+	server = server_new ();
+	server->address = g_strdup ("Mock Server for Folders");
+	server->name = g_strdup (INTERNAL_SERVER_NAME);
+	serverlist_add_server (server);
+
+	/* ensure the folder directories exist */
 	message_base = acache_get_message_base ();
 	path = g_build_filename (message_base, ACACHE_DEFAULT_KEY, NULL);
 	acache_add_path (ACACHE_DEFAULT_KEY, path);
@@ -256,46 +451,18 @@ add_folder_directories (gpointer data)
 	/* cleanup */
 	g_free (folder_base);
 	g_free (message_base);
-	debug_exit ("add_folder_directories");
-	return NULL;
+	debug_exit ("acache_init");
 }
 
-void
-acache_init (void)
-{
-	Server *server;
-
-	/* sanity clause -- don't init twice */
-	if (acache_entry_chunk != NULL)
-		return;
-
-	/* init the callbacks */
-	acache_bodies_added_callback = pan_callback_new ();
-	acache_bodies_removed_callback = pan_callback_new ();
-
-	/* init the message id hash  */
-	acache_entry_chunk = memchunk_new (sizeof(AcacheEntry), ACACHE_ENTRY_CHUNK_SIZE, FALSE);
-	messageid_to_entry = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
-	key_to_path = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, (GDestroyNotify)acache_path_free);
-
-	/* add the folder server */
-	server = server_new ();
-	server->address = g_strdup ("Mock Server for Folders");
-	server->name = g_strdup (INTERNAL_SERVER_NAME);
-	serverlist_add_server (server);
-
-	/* add all the folder directories */
-	run_in_worker_thread (add_folder_directories, server);
-}
 
 void
 acache_shutdown (void)
 {
-	if (messageid_to_entry) {
+	if (key_to_messageids) {
 		if (acache_flush_on_exit)
 			acache_expire_all ();
-		g_hash_table_destroy (messageid_to_entry);
-		messageid_to_entry = NULL;
+		g_hash_table_destroy (key_to_messageids);
+		key_to_messageids = NULL;
 	}
 
 	if (key_to_path) {
@@ -315,47 +482,6 @@ acache_shutdown (void)
 ****
 ***/
 
-/**
- * This must be called from inside an entries_mutex lock.
- */
-static void
-acache_remove_entry_from_path (AcacheEntry * entry)
-{
-	char filename[PATH_MAX] = { '\0' };
-
-	g_return_if_fail (entry!=NULL);
-	g_return_if_fail (entry->path!=NULL);
-
-	/* update the acache size */
-	entry->path->size -= entry->size;
-
-	/* remove the file */
-	acache_get_filename (filename, sizeof(filename), entry->path->path, entry->message_id);
-	unlink (filename);
-}
-
-/**
- * This must be called from inside an entries_mutex lock.
- */
-static void
-acache_remove_entry (AcacheEntry * entry)
-{
-	/* sanity clause */
-	g_return_if_fail (entry!=NULL);
-
-	/* let everyone know */
-	debug2 (DEBUG_ACACHE, "Removing %s from cache with refcount %d", entry->message_id, entry->refcount);
-	pan_callback_call (acache_bodies_removed_callback, (gpointer)entry->message_id, NULL);
-
-	/* if this entry had a file associated with it ... */
-	if (entry->path)
-		acache_remove_entry_from_path (entry);
-
-	/* update acache tables */
-	g_hash_table_remove (messageid_to_entry, entry->message_id);
-	memchunk_free (acache_entry_chunk, entry);
-}
-
 static int
 compare_ppEntry_ppEntry_by_youth (gconstpointer a, gconstpointer b, gpointer unused)
 {
@@ -365,9 +491,12 @@ compare_ppEntry_ppEntry_by_youth (gconstpointer a, gconstpointer b, gpointer unu
 }
 
 void
-acache_expire_messages (const char   ** message_ids,
+acache_expire_messages (const char   *  path_key,
+                        const char   ** message_ids,
 	                int             message_id_qty)
 {
+	/* sanity clause */
+	g_return_if_fail (is_nonempty_string (path_key));
 	g_return_if_fail (message_ids!=NULL);
 	g_return_if_fail (message_id_qty>0);
 
@@ -377,14 +506,7 @@ acache_expire_messages (const char   ** message_ids,
 
 		for (i=0; i<message_id_qty; ++i)
 		{
-			AcacheEntry * entry;
-			const char * message_id;
-
-			/* find the entry */
-			message_id = message_ids[i];
-			entry = (AcacheEntry*) g_hash_table_lookup (messageid_to_entry, message_id);
-
-			/* if we have such an entry, remove it */
+			AcacheEntry * entry = acache_lookup_entry (path_key, message_ids[i]);
 			if (entry != NULL)
 				acache_remove_entry (entry);
 		}
@@ -394,31 +516,25 @@ acache_expire_messages (const char   ** message_ids,
 	g_static_mutex_unlock (&entries_mutex);
 }
 
-
 static int
-acache_expire_to_size (double max_megs)
+acache_expire_to_size (AcachePath * apath, double max_megs)
 {
 	int files_removed = 0;
 	const size_t cache_max = (size_t) max_megs * (1024 * 1024);
-	AcachePath * apath;
 	debug_enter ("acache_expire_to_size");
 
-	/* get the scratch directory path */
-	apath = (AcachePath*) g_hash_table_lookup (key_to_path, ACACHE_DEFAULT_KEY);
-	g_return_val_if_fail (apath!=NULL, 0);
-
 	g_static_mutex_lock (&entries_mutex);
 	{
-		debug2 (DEBUG_ACACHE, "expiring to %lu; current size is %lu", cache_max, apath->size);
+		debug3 (DEBUG_ACACHE, "expiring to %lu; current size is %lu (%ld free)", cache_max, apath->size, (long)cache_max-(long)apath->size);
 
 		if (cache_max < apath->size)
 		{
 			int i;
 			GPtrArray * entries;
 
-			/* get an array of files sorted from oldest to youngest*/
+			/* get an array of files sorted from oldest to youngest */
 			entries = g_ptr_array_new ();
-			pan_hash_to_ptr_array (messageid_to_entry, entries);
+			pan_hash_to_ptr_array (acache_get_key_path_hash(apath->key), entries);
 			g_ptr_array_sort_with_data (entries, compare_ppEntry_ppEntry_by_youth, NULL);
 
 			/* Start blowing away files */
@@ -437,13 +553,6 @@ acache_expire_to_size (double max_megs)
 	}
 	g_static_mutex_unlock (&entries_mutex);
 
-	/* log 
-	if (files_removed != 0)
-		log_add_va (LOG_INFO, _("Removed %d articles from \"%s\", which now has %.1f MB"),
-		                      files_removed,
-		                      apath->path,
-		                      (double)(apath->size/(1024.0*1024.0)));*/
-
 	/* done */
 	debug_exit ("acache_expire_to_size");
 	return files_removed;
@@ -452,17 +561,23 @@ acache_expire_to_size (double max_megs)
 int 
 acache_expire (void)
 {
-	return acache_expire_to_size (acache_max_megs * 0.8);
+	AcachePath * apath = (AcachePath*) g_hash_table_lookup (key_to_path, ACACHE_DEFAULT_KEY);
+	const double desired_size = acache_max_megs * 0.8;
+	return acache_expire_to_size (apath, desired_size);
 }
 
 int
 acache_expire_all (void)
 {
+	AcachePath * apath;
 	int retval;
-	printf ("Pan is flushing article cache... ");
-	retval = acache_expire_to_size(0);
-	printf ("%d files erased.\n", retval);
+
+	g_message (_("Flushing article cache... "));
+       	apath = (AcachePath*) g_hash_table_lookup (key_to_path, ACACHE_DEFAULT_KEY);
+	retval = acache_expire_to_size (apath, 0.0);
+	g_message (_("%d files erased."), retval);
 	fflush (NULL);
+
 	return retval;
 }
 
@@ -474,67 +589,75 @@ acache_expire_all (void)
 
 /* this must be called inside an entries_mutex lock */
 static void
-acache_update_refcount_nolock (MessageIdentifier ** identifiers,
+acache_update_refcount_nolock (const char         * path_key,
+                               MessageIdentifier ** identifiers,
                                int                  identifier_qty,
                                int                  inc)
 {
 	int i;
-	g_return_if_fail (identifiers != NULL);
+	debug_enter ("acache_update_refcount_nolock");
+
 	g_return_if_fail (identifier_qty >= 1);
-	for (i=0; i<identifier_qty; ++i)
-		g_return_if_fail (message_identifier_is_valid(identifiers[i]));
+	g_return_if_fail (message_identifiers_are_valid ((const MessageIdentifier**)identifiers, identifier_qty));
 
 	for (i=0; i<identifier_qty; ++i)
 	{
-		const char * message_id = identifiers[i]->message_id;
-		AcacheEntry * entry;
+		AcacheEntry * entry = acache_get_entry (path_key, identifiers[i]->message_id);
 
-		entry = g_hash_table_lookup (messageid_to_entry, message_id);
-		if (entry == NULL)
-		{
-			entry = (AcacheEntry*) memchunk_alloc (acache_entry_chunk);
-			entry->path = NULL;
-			entry->refcount = 0;
-			entry->message_id = g_strdup (message_id);
-			entry->size = 0;
-			entry->date = time (NULL);
-			g_hash_table_replace (messageid_to_entry, entry->message_id, entry);
-			debug1 (DEBUG_ACACHE, "Added new entry %d", entry->message_id);
-		}
-
-		/* If we're checking it out, then move it to the safe end of the
-		   least-recently-used kill heuristic */
+		/* If we're checking it out,
+		 * then move it to the safe end of the least-recently-used kill heuristic */
 		if (inc > 0)
 			entry->date = time (NULL);
 
 		entry->refcount += inc;
 		debug3 (DEBUG_ACACHE, "%s refcount - inc by %d to %d", entry->message_id, inc, entry->refcount);
 	}
+
+	debug_exit ("acache_update_refcount_nolock");
 }
 
 void
-acache_checkout (MessageIdentifier  ** identifiers,
+acache_checkout (const char          * key,
+                 MessageIdentifier  ** identifiers,
                  int                   identifier_qty)
 {
+	debug_enter ("acache_checkout");
+
+	/* sanity clause */
+	g_return_if_fail (is_nonempty_string(key));
 	g_return_if_fail (identifiers!=NULL);
 	g_return_if_fail (identifier_qty >= 1);
+	g_return_if_fail (message_identifiers_are_valid ((const MessageIdentifier**)identifiers, identifier_qty));
 
+	/* ref the entries */
 	g_static_mutex_lock (&entries_mutex);
-	acache_update_refcount_nolock (identifiers, identifier_qty, 1);
+	acache_update_refcount_nolock (key, identifiers, identifier_qty, 1);
 	g_static_mutex_unlock (&entries_mutex);
+
+	debug_exit ("acache_checkout");
 }
+
 void
-acache_checkin (MessageIdentifier  ** identifiers,
+acache_checkin (const char          * key,
+                MessageIdentifier  ** identifiers,
                 int                   identifier_qty)
 {
+	debug_enter ("acache_checkin");
+
+	/* sanity clause */
+	g_return_if_fail (is_nonempty_string (key));
 	g_return_if_fail (identifiers!=NULL);
 	g_return_if_fail (identifier_qty >= 1);
+	g_return_if_fail (message_identifiers_are_valid ((const MessageIdentifier**)identifiers, identifier_qty));
 
+	/* unref the entries */
 	g_static_mutex_lock (&entries_mutex);
-	acache_update_refcount_nolock (identifiers, identifier_qty, -1);
+	acache_update_refcount_nolock (key, identifiers, identifier_qty, -1);
 	g_static_mutex_unlock (&entries_mutex);
 
 	acache_expire ();
+
+	debug_exit ("acache_checkin");
 }
 
 /***
@@ -583,30 +706,16 @@ acache_set_message (const char    * key,
 		{
 			AcacheEntry * entry;
 
+			/* file an entry for this message */
 			g_static_mutex_lock (&entries_mutex);
-			{
-				/* if this mesasge existed before and is moving ... */
-				entry = g_hash_table_lookup (messageid_to_entry, message_id);
-				if (entry!=NULL && entry->path!=NULL && entry->path!=apath)
-					acache_remove_entry_from_path (entry);
-
-				/* if this message didn't exist before ... */
-				if (entry == NULL) {
-					MessageIdentifier * mid = mid = message_identifier_new (message_id);
-					acache_update_refcount_nolock (&mid, 1, 0);
-					g_object_unref (mid);
-					entry = g_hash_table_lookup (messageid_to_entry, message_id);
-				}
-
-				g_assert (entry!=NULL);
-				entry->size = message_len;
-				entry->date = time (NULL);
-				entry->path = apath;
-			}
+			entry = acache_get_entry (key, message_id);
+			entry->size = message_len;
+			entry->date = time (NULL);
+			entry->path = apath;
 			g_static_mutex_unlock (&entries_mutex);
 
 			/* notify everyone that this message has been added */
-			pan_callback_call (acache_bodies_added_callback, (gpointer)message_id, NULL);
+			fire_bodies_added (&message_id, 1);
 
 			/* if the acache is too big, purge the old stuff */
 			apath->size += message_len;
@@ -681,7 +790,8 @@ acache_get_message_mem_stream (const AcacheEntry * entry)
 }
 
 GMimeMessage*
-acache_get_message  (MessageIdentifier  ** mids,
+acache_get_message  (const char          * path_key,
+                     MessageIdentifier  ** mids,
                      int                   mid_qty)
 {
 	int i;
@@ -690,18 +800,16 @@ acache_get_message  (MessageIdentifier  ** mids,
 	GMimeMessage * retval = NULL;
 
 	/* sanity clause */
+	g_return_val_if_fail (is_nonempty_string(path_key), NULL);
 	g_return_val_if_fail (mid_qty>0, NULL);
-	g_return_val_if_fail (mids!=NULL, NULL);
-	for (i=0; i<mid_qty; ++i)
-		g_return_val_if_fail (message_identifier_is_valid(mids[i]), NULL);
+	g_return_val_if_fail (message_identifiers_are_valid ((const MessageIdentifier**)mids, mid_qty), NULL);
 
 	/* get streams */
 	streams = g_alloca (sizeof(GMimeStream*) * mid_qty);
 	for (i=stream_qty=0; i<mid_qty; ++i)
 	{
 		GMimeStream * stream;
-		const AcacheEntry * entry = (const AcacheEntry*)
-			g_hash_table_lookup (messageid_to_entry, mids[i]->message_id);
+		const AcacheEntry * entry = acache_lookup_entry (path_key, mids[i]->message_id);
 		if (entry == NULL)
 			break;
 
@@ -726,14 +834,18 @@ acache_get_message  (MessageIdentifier  ** mids,
 }
 
 gboolean
-acache_has_message (const char * message_id)
+acache_has_message (const char * path_key,
+                    const char * message_id)
 {
 	gboolean retval = FALSE;
 	AcacheEntry * entry = NULL;
 
+	/* sanity clause */
+	g_return_val_if_fail (is_nonempty_string(path_key), FALSE);
 	g_return_val_if_fail (is_nonempty_string(message_id), FALSE);
 
-	entry = g_hash_table_lookup (messageid_to_entry, message_id);
+	/* lookup the message */
+	entry = acache_lookup_entry (path_key, message_id);
 	if (entry != NULL)
 		retval = entry->size != 0;
 
@@ -792,7 +904,7 @@ acache_path_foreach (const char             * key,
 	tmp.path = apath;
 	tmp.func = func;
 	tmp.user_data = user_data;
-	g_hash_table_foreach (messageid_to_entry, acache_path_hash_foreach, &tmp);
+	g_hash_table_foreach (acache_get_key_path_hash(key), acache_path_hash_foreach, &tmp);
 }
 
 void
@@ -807,11 +919,11 @@ acache_add_folder (const char * key)
 	g_free (folder_base);
 }
 
-void
+static void
 acache_add_path (const char * key,
                  const char * path)
 {
-	int qty;
+	int qty = 0;
 	GDir * dir;
 	GError * err;
 	AcachePath * apath;
@@ -844,43 +956,28 @@ acache_add_path (const char * key,
 		char message_id[2048];
 		char filename[PATH_MAX];
 
-		qty = 0;
 		while ((fname = g_dir_read_name (dir)))
 		{
 			struct stat stat_p;
 
-			if (!string_ends_with (fname, ".msg"))
-				continue;
-
 			g_snprintf (filename, sizeof(filename), "%s%c%s", path, G_DIR_SEPARATOR, fname);
 			if (stat (filename, &stat_p) == 0)
 			{
 				if (get_message_id_from_file (message_id, sizeof(message_id), filename))
 				{
-					AcacheEntry * entry;
-
-					/* FIXME: if Pan is shut down while 
-					 * this thread is still indexing the 
-					 * cache, acache_shutdown() will clear
-					 * acache_entry_chunk and calling 
-					 * memchunk_alloc() will segfault. 
-		 			 * This check can be removed once a 
-					 * faster cache index can be done in 
-					 * the main thread.
+					/*
+					 * FIXME: this may segfault if acache 
+					 * is shut down while this thread is 
+					 * still running.
 					 */
-					if (!acache_entry_chunk)
-						break;
 
-					entry = (AcacheEntry*) memchunk_alloc (acache_entry_chunk);
+					AcacheEntry * entry = acache_get_entry (key, message_id);
 					entry->path = apath;
-					entry->message_id = g_strdup (message_id);
 					entry->size = stat_p.st_size;
 					entry->date = stat_p.st_mtime;
 					entry->refcount = 0;
-					g_hash_table_replace (messageid_to_entry, entry->message_id, entry);
 
 					apath->size += entry->size;
-
 					++qty;
 				}
 			}
