@@ -195,7 +195,7 @@ articles_set_new_impl (Article ** articles, int article_qty, gboolean is_new, gb
 	g_return_if_fail (articles_are_valid ((const Article **)articles, article_qty));
 
 	/* mark each article's newness field */
-	changed = g_ptr_array_new ();
+	changed = g_ptr_array_sized_new (article_qty);
 	for (i=0; i<article_qty; ++i) {
 		Article * a = articles[i];
 		if (!!is_new != !!a->is_new) {
@@ -741,7 +741,7 @@ article_header_is_extra (const char * key)
 	return TRUE;
 }
 
-void
+static void
 xref_foreach (const char         * xref_string,
               Server             * server,
               ServerGroupsType     set,
