@@ -131,13 +131,13 @@ create_filename (const gchar       * path,
 		}
 		else if (is_nonempty_string(path))
 		{
-			unique = g_strdup_printf ("%s%c%s_%d%s",
+			unique = g_strdup_printf ("%s%c%s_copy_%d%s",
 			                          path, G_DIR_SEPARATOR,
 			                          lead, i, tail);
 		}
 		else
 		{
-			unique = g_strdup_printf ("%s_%d%s", lead, i, tail);
+			unique = g_strdup_printf ("%s_copy_%d%s", lead, i, tail);
 		}
 
 		/* cleanup */
@@ -192,22 +192,21 @@ decode_article (const decode_data * dd)
 	GString * filenames;
 	char * filename = NULL;
 	GMimeStream * stream;
-	GMimeStream * stream_file;
+	GMimeStream * stream_file = NULL;
 	GMimeFilterYenc * yenc_filter;
 	debug_enter ("decode_article");
 
 	/* sanity clause */
 	g_return_val_if_fail (dd!=NULL, FALSE);
 	g_return_val_if_fail (server_is_valid (dd->server), FALSE);
-	g_return_val_if_fail (dd->mids!=NULL, FALSE);
 	g_return_val_if_fail (dd->mid_qty>0, FALSE);
-	for (i=0; i<dd->mid_qty; ++i)
-		g_return_val_if_fail (message_identifier_is_valid (dd->mids[i]), FALSE);
+	g_return_val_if_fail (is_nonempty_string(dd->acache_key), FALSE);
+	g_return_val_if_fail (message_identifiers_are_valid ((const MessageIdentifier**)dd->mids, dd->mid_qty), FALSE);
 
-	/* checkout the messages */
+	/* get the message from the cache, and make an array of its attachments */
 	attachments = g_ptr_array_new ();
 	status_item_emit_status_va (dd->item, _("Decoding \"%s\""), dd->subject);
-	mm = acache_get_message (dd->mids, dd->mid_qty);
+	mm = acache_get_message (dd->acache_key, dd->mids, dd->mid_qty);
 	g_mime_message_foreach_part (mm, get_array_of_decodable_parts_func, attachments);
 
 	/* decode & save the parts */
@@ -266,9 +265,9 @@ decode_article (const decode_data * dd)
 					}
 				}
 				else {
-					log_add_va (LOG_ERROR, _("Decode unable to create file \"%s\" %s"),
-							filename,
-							errno==0 ? "" : g_strerror(errno));
+					log_add_va (LOG_ERROR, _("Can't create file \"%s\" %s"),
+					            filename,
+					            errno==0 ? "" : g_strerror(errno));
 					success = FALSE;
 				}
 			}
@@ -312,7 +311,7 @@ decode_article (const decode_data * dd)
 				yenc_filter->state = GMIME_YENCODE_STATE_INIT;
 			}
 
-			if (!ferror (GMIME_STREAM_FILE(stream_file)->fp))
+			if (stream_file!=NULL && !ferror (GMIME_STREAM_FILE(stream_file)->fp))
 				status_item_emit_status_va (dd->item, _("Saved \"%s\""), filename);
 			else {
 				log_add_va (LOG_ERROR, _("Error saving file \"%s\" - is the disk full?"), filename);
@@ -326,14 +325,9 @@ decode_article (const decode_data * dd)
 
 	/* log the success */
 	if (success) {
-		const char * group_name;
-		char * msg;
-	       	group_name = message_identifier_get_primary_group (dd->mids[0]);
-		if (!group_name)
-			group_name = "???";
-		msg = g_strdup_printf (
-			_("Decoded \"%s\" from group \"%s\", \"%s\""),
-			filename, group_name, dd->subject);
+		const char * group_name = message_identifier_get_primary_group (dd->mids[0]);
+		char * msg = g_strdup_printf (_("Decoded \"%s\" from group \"%s\", \"%s\""),
+			filename, group_name ? group_name : "???", dd->subject);
 		log_add (LOG_INFO, msg);
 		debug0 (DEBUG_DECODE, msg);
 		g_free (msg);
@@ -346,34 +340,36 @@ decode_article (const decode_data * dd)
 	g_free (filename);
 	filename = NULL;
 
-	/* if the Article objects are in memory, update them */
+	/* if the Article objects are in memory, update them  as decoded or failed */
 	if (1) {
-		/* mark as decoded or failed */
-		const char * group_name = message_identifier_get_primary_group (dd->mids[0]);
-		Group * group = server_get_named_group (dd->server, group_name);
-		if (group != NULL) {
-			if (group_ref_articles_if_loaded (group)) {
-				int qty = 0;
-				Article ** articles = g_alloca (sizeof(Article*) * dd->mid_qty);
-				for (i=0; i<dd->mid_qty; ++i) {
-					Article * a = group_get_article_by_message_id (group, dd->mids[i]->message_id);
-					if (a != NULL)
-						articles[qty++] = a;
-				}
-				articles_add_flag (articles, qty, (success?STATE_DECODED:STATE_DECODE_FAILED));
-				group_unref_articles (group, NULL);
+		const char * group_name = NULL;
+		Group * group = NULL;
+
+		group_name = message_identifier_get_primary_group (dd->mids[0]);
+
+		if (is_nonempty_string (group_name))
+			group = server_get_named_group (dd->server, group_name);
+
+		if (group!=NULL && group_ref_articles_if_loaded (group)) {
+			int qty = 0;
+			Article ** articles = g_alloca (sizeof(Article*) * dd->mid_qty);
+			for (i=0; i<dd->mid_qty; ++i) {
+				Article * a = group_get_article_by_message_id (group, dd->mids[i]->message_id);
+				if (a != NULL)
+					articles[qty++] = a;
 			}
+			articles_add_flag (articles, qty, (success?STATE_DECODED:STATE_DECODE_FAILED));
+			group_unref_articles (group, NULL);
 		}
 	}
 
 	if (success)
 		message_identifiers_mark_read (dd->mids, dd->mid_qty, TRUE, SERVER_GROUPS_SUBSCRIBED);
 
-	/* cleanup */
+	/* cleanup some more */
 	if (mm != NULL)
 		g_mime_object_unref (GMIME_OBJECT(mm));
 	g_string_free (filenames, TRUE);
-	acache_checkin (dd->mids, dd->mid_qty);
 	g_ptr_array_free (attachments, TRUE);
 	debug_exit ("decode_article");
 
