@@ -51,7 +51,7 @@ file_grouplist_097_style_exists (const Server * server)
 	ServerGroupsType retval = 0;
 
 	/* sanity clause */
-	g_return_val_if_fail (server_is_valid (server), FALSE);
+	g_return_val_if_fail (server_is_valid (server), retval);
 
 	/* check for sub */
 	g_snprintf (fname, sizeof(fname), "%s%c%s_sub.idx", get_data_dir(), G_DIR_SEPARATOR, server_get_name (server));
@@ -87,7 +87,7 @@ file_grouplist_exists (const Server* server)
 	ServerGroupsType retval = 0;
 
 	/* sanity clause */
-	g_return_val_if_fail (server_is_valid (server), 0);
+	g_return_val_if_fail (server_is_valid (server), retval);
 
 	/* first check the new style... */
 	retval = file_grouplist_097_style_exists (server);
@@ -211,7 +211,7 @@ read_group_data (Server        * server,
 
 			/* old sort style */
 			if (version >= 10)
-				g->old_sort_style = get_next_token_int (march_idx, '\n', &march_idx);
+				g->old_sort_style = (gint8) get_next_token_int (march_idx, '\n', &march_idx);
 
 			g->new_sort_style = (gint8) get_next_token_int (march_idx, '\n', &march_idx);
 
@@ -481,7 +481,7 @@ save_groups_to_file (Server * server, GPtrArray * groups, gchar * filemarker, St
 	/* sanity clause */
 	g_return_if_fail (server_is_valid (server));
 	g_return_if_fail (groups!=NULL);
-	g_return_if_fail (0<=groups->len);
+	g_return_if_fail (0<groups->len);
 	g_return_if_fail (is_nonempty_string(filemarker));
 
 	if (groups->len == 0)
