@@ -391,12 +391,10 @@ file_headers_save_group (Group * group, StatusItem * status)
 
 	if (1) {
 		char * pch = g_strdup_printf ("%s%c%s", get_data_dir(), G_DIR_SEPARATOR, server_get_name (group->server));
-		if (!pan_file_ensure_path_exists (pch))
-			log_add_va (LOG_ERROR, _("Can't create directory \"%s\""), pch);
+		pan_file_ensure_path_exists (pch);
 		g_free (pch);
 	}
 
-
 	/* open index file */
 	idx_path = g_strdup_printf ("%s%c%s%c%s.idx.tmp",
 		get_data_dir(), G_DIR_SEPARATOR,
