@@ -209,7 +209,7 @@ read_unquoted_word (const char * pch, const char ** end)
 {
 	/* 1*unquoted-char */
 	if (!is_unquoted_char(*pch)) return FALSE;
-	while (is_unquoted_char((int)*pch)) ++pch;
+	while (is_unquoted_char((guchar)*pch)) ++pch;
 	*end = pch;
 	return TRUE;
 }
@@ -223,7 +223,7 @@ read_quoted_word (const char * pch, const char ** end)
 	for (;;) {
 		if (read_space(pch,&pch))
 			continue;
-		if (is_quoted_char ((int)*pch)) {
+		if (is_quoted_char ((guchar)*pch)) {
 			++pch;
 			continue;
 		}
@@ -264,7 +264,7 @@ read_plain_phrase (const char * pch, const char ** end)
 static gboolean
 read_paren_char (const char * pch, const char ** end)
 {
-	if (!is_paren_char((int)*pch))
+	if (!is_paren_char((guchar)*pch))
 		return FALSE;
 	*end = pch + 1;
 	return TRUE;
@@ -396,7 +396,6 @@ gnksa_check_domain (const char * domain, guint domain_len)
 
 			/* single-letter TLDs dont exist */
 			return GNKSA_ILLEGAL_DOMAIN;
-			break;
 
 		case 2:
 			if (isdigit((guchar)top[0]) || isdigit((guchar)top[1]))
@@ -794,8 +793,7 @@ gnksa_check_message_id (const char* message_id, gint message_id_len)
 static void
 remove_broken_message_ids_from_references (const char * references, GString * setme)
 {
-	const gchar * pch;
-	const gchar * tmp;
+	const char * pch;
 
 	/* sanity checks */
 	g_return_if_fail (setme!=NULL);
@@ -804,11 +802,11 @@ remove_broken_message_ids_from_references (const char * references, GString * se
 
 	/* remove broken message-ids
 	   FIXME: this tokenizing works, but it's hard to read. */
-       	pch = tmp = references;
+       	pch = references;
 	for (;;)
 	{
-		const gchar * begin;
-		const gchar * end;
+		const char * begin;
+		const char * end;
 
 		/* find the beginning of the message-id */
 		while (*pch!='<')
@@ -978,7 +976,7 @@ gnksa_generate_message_id_from_email_addr (const gchar * addr)
 static gchar*
 gnksa_trim_references (const gchar* refs)
 {
-	const int GNKSA_CUTOFF = 998; /* gnksa rule 7 */
+	const int GNKSA_CUTOFF = 986; /* GNKSA rule 7: 998 chars - 12 for "References: " */
 	return gnksa_trim_references_to_len (refs, GNKSA_CUTOFF);
 }
 
