@@ -67,7 +67,6 @@ group_constructor (Group                * g,
 	pan_warn_if_fail (g!=NULL);
 	pan_warn_if_fail (is_nonempty_string(name));
 
-	debug1 (DEBUG_PAN_OBJECT, "group_constructor: %p", g);
 	g->article_mutex = g_mutex_new ();
 	g->loaded_since_last_fetch = FALSE;
 	g->articles_dirty = FALSE;
@@ -483,6 +482,20 @@ group_set_is_folder (Group * group, gboolean folder)
 	}
 }
 
+const char*
+group_get_acache_key (const Group * group)
+{
+	const char * retval = ACACHE_DEFAULT_KEY;
+
+	g_return_val_if_fail (group_is_valid(group), retval);
+
+	if (group_is_folder (group))
+		retval = group_get_name (group);
+
+	return retval;
+}
+
+
 /***
 ****
 ****  DELETE GROUP
@@ -534,7 +547,7 @@ folder_add_message (Group            * folder,
 
 	/* do we already have this message? */
 	message_id = g_mime_message_get_message_id (message);
-	is_new = acache_has_message (message_id);
+	is_new = acache_has_message (group_get_acache_key(folder), message_id);
 
 	/* write the message */
 	text = g_mime_message_to_string (message);
@@ -574,7 +587,7 @@ folder_remove_message  (Group         * folder,
 
 	/* delete the message from our cache */
 	message_id = g_mime_message_get_message_id (message);
-	acache_expire_messages (&message_id, 1);
+	acache_expire_messages (group_get_acache_key(folder), &message_id, 1);
 
 	/* delete the message from the group's tables */
 	if (!group_ref_articles_if_loaded (folder))
@@ -715,7 +728,7 @@ group_add_articles_impl (Group             * group,
 	{
 		Article * a = ARTICLE(g_ptr_array_index(articles,i));
 		const gchar * message_id = article_get_message_id (a);
-		gboolean is_purged = newsrc_is_article_read(group->_purged, a->number);
+		gboolean is_purged = newsrc_is_article_read(purged, a->number);
 
 		if (!is_purged && g_hash_table_lookup(ours,message_id) == NULL)
 		{
@@ -1083,8 +1096,8 @@ group_unref_articles (Group * group, StatusItem * status)
 	if (group->_articles_refcount == 1)
 	{
 		g_mutex_lock (group->article_mutex);
-		file_headers_save_noref (group, NULL);
-		server_save_grouplist_if_dirty (group->server, NULL);
+		file_headers_save_noref (group, status);
+		server_save_grouplist_if_dirty (group->server, status);
 
 		if (group->_articles!=NULL) {
 		       	g_hash_table_foreach (group->_articles, destroy_articles_ghfunc, NULL);
