@@ -320,41 +320,6 @@ check_all_quoted (GPtrArray      * setme_errors,
 	*setme_goodness = MAX (*setme_goodness, REFUSE);
 }
 
-/**
- * Check the charset is correct for the body.
- * Since we've already g_converted at this point (article_get_body()),
- * body will be NULL if this is the case.
- */
-static void
-check_charset (GPtrArray      * setme_errors,
-               GoodnessLevel  * setme_goodness,
-               const gchar    * body,
-               GMimeMessage   * message)
-{
-#if 0
-	if (body == NULL)
-	{
-		/* FIXME: this should move to article.c */
-		const char * raw_body;
-		const char * user_charset;
-		const char * best_charset;
-	       
-		user_charset = NULL;
-		raw_body = NULL;
-		if (is_nonempty_string (raw_body))
-			best_charset = g_mime_charset_best (raw_body, strlen (raw_body));
-		else
-			best_charset = _("Unknown");
-
-		g_ptr_array_add (setme_errors, g_strdup_printf(
-			_("ERROR: message uses characters not specified in charset '%s' - possibly use '%s' instead?"), user_charset, best_charset));
-		*setme_goodness = MAX (*setme_goodness, REFUSE);
-	}
-#else
-#warning check_charset is broken
-#endif
-}
-
 static void
 check_body (GPtrArray       * setme_errors,
 	    GoodnessLevel   * setme_goodness,
@@ -367,7 +332,6 @@ check_body (GPtrArray       * setme_errors,
 	g_return_if_fail (setme_errors != NULL);
 	g_return_if_fail (message != NULL);
 
-	check_charset (setme_errors, setme_goodness, body, message);
 	if (posting && setme_errors->len == 0)
 	{
 		check_empty (setme_errors, setme_goodness, body);
