@@ -216,9 +216,8 @@ newsrc_export (Server          * server,
 	if (fp == NULL)
 	{
 		log_add_va (LOG_ERROR,
-			_("Couldn't create file \"%s\" to export newsrc file %s"),
-			filename,
-			g_strerror(errno));
+		            _("Could not create temporary file: %s"),
+		            g_strerror(errno));
 		g_free (filename);
 		return;
 	}
