@@ -469,7 +469,7 @@ parse_and_load_file (xmlDoc *doc, xmlNode *node, const char * path)
 				config_set_or_add_node (new_key, IS_INT, atoi (contents));
 				break;
 			case 'b':
-				if (isdigit(contents[0]))
+				if (isdigit((guchar)contents[0]))
 					config_set_or_add_node (new_key, IS_BOOL, contents[0] == '1');
 				else
 					config_set_or_add_node (new_key, IS_BOOL, tolower(contents[0]) == 't');
