@@ -41,7 +41,7 @@ static void fire_groups_removed        (Server*, Group**, gint group_qty);
 
 /* this option for turning off saving exists for the regression tests,
  * which need to be able to turn it off */
-gint server_saving_enabled = TRUE;
+int server_saving_enabled = TRUE;
 
 /**
 ***  Server Object Life Cycle
