@@ -1,13 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Author: Charles Kerr <charles@rebelbase.com>
- *
- * Copyright (C) 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #ifndef __SERVER_H__
@@ -50,19 +47,19 @@ typedef struct
 
 	/*Fields*/
 	guint posting              : 1;  /* is posting allowed? */
-	guint gen_msgid            : 1;
 	guint need_auth            : 1;  /* need authorization ? */
-	guint online               : 1;  /* is this server online? */
-	guint online_prompted      : 1;  /* true if user's been asked to go online for some tasks. */
+	guint use_newsrc           : 1;  /* import/export .newsrc files? */
+	guint newsrc_imported      : 1;  /* have we already imported it? */
 	guint reserve_connection_for_bodies     : 1;
 	gint8 max_connections;           /* max connections to server */
         gint16 idle_secs_before_timeout;
 	gint port;	                 /* port number.  This is usually 119 for NNTP. */
 	time_t last_newgroup_list_time;
-	gchar * address;                 /* server net address */
-	gchar * name;	                 /* pretty name, i.e. Erols */
-	gchar * username;                /* auth username */
-	gchar * password;                /* auth password */
+	char * address;                  /* server net address */
+	char * name;	                 /* pretty name, i.e. Erols */
+	char * username;                 /* auth username */
+	char * password;                 /* auth password */
+	char * newsrc_filename;          /* import/export this */
 
 	ServerGroupsType _groups_dirty;
 	ServerGroupsType _groups_loaded;
@@ -87,33 +84,26 @@ void       server_destructor             (PanObject*);
 
 Server*       server_new (void);
 
-gboolean      server_has_group           (const Server*, const char* groupname);
-
-gboolean      server_has_group_in_type   (const Server      * server,
-                                          const char        * groupname,
-                                          ServerGroupsType    type);
-
 gboolean      server_is_valid            (const Server*);
 
 const gchar * server_get_name            (const Server*);
 
 
-/**
-***  PUBLIC - Online
-**/
-
-void       server_set_online             (Server*, gboolean);
-
 /**
 ***  PUBLIC - Server Groups
 **/
 
 struct _Group;
 
-struct _Group* server_alloc_new_group    (Server            * server);
+struct _Group* server_alloc_new_group          (Server            * server);
+
+struct _Group* server_get_named_group          (Server            * server,
+                                                const char        * name);
+
+struct _Group* server_get_named_group_in_type  (Server            * server,
+                                                const char        * name,
+                                                ServerGroupsType    type);
 
-struct _Group* server_get_named_group    (Server            * server,
-                                          const gchar       * name);
 
 void       server_init_groups             (Server            * server,
                                           struct _Group    ** groups,
@@ -162,11 +152,5 @@ PanCallback*   server_get_groups_removed_callback (void);
  */
 PanCallback*   server_get_groups_added_callback (void);
 
-/**
- * @call_obj: Server*
- * @call_arg: NULL if offline, non-NULL if online
- */
-PanCallback*   server_get_online_status_changed_callback (void);
-
 
 #endif /* __SERVER_H__ */
