@@ -216,6 +216,10 @@ serverlist_set_active_server (Server * server)
 
 	/* activate it */
 	_active = server;
+	if (_active!=NULL && is_nonempty_string(_active->name))
+		pan_config_set_string ("/Pan/State/Server", _active->name);
+	else
+		pan_config_clean_key ("/Pan/State/Server");
 	pan_callback_call (serverlist_get_server_activated_callback(), _active, NULL);
 }
 
@@ -343,7 +347,11 @@ serverlist_save (void)
 		const char * str_begin = NULL;
 		const char * walk = p;
 		while (get_next_token_run (walk, ',', &walk, &str_begin, &str_len)) {
-			g_snprintf (buf, sizeof(buf), "/Pan/%*.*s", str_len, str_len, str_begin);
+			char * line = g_alloca (str_len+1);
+			memcpy (line, str_begin, str_len);
+			line[str_len] = '\0';
+
+			g_snprintf (buf, sizeof(buf), "/Pan/%s", line);
 			pan_config_clean_section (buf);
 		}
 		g_free (p);
