@@ -94,21 +94,13 @@ get_credits_translators (void)
 {
 	const char * str =
 "\n\tcs - Michal Bukovjan"
-"\n\tde - Benjamin Greiner, Christian Neumair"
-"\n\tes - Pablo del Campo"
+"\n\tde - Benjamin Greiner"
+"\n\tes - Pablo Gonzalo del Campo"
 "\n\tet - Tivo Leedjrv"
-"\n\tfr - Christophe Merlet"
-"\n\thu - Andras Timar"
 "\n\tja - Takayuki KUSANO"
-"\n\tko - Nam SungHyun"
 "\n\tnl - Reinout van Schouwen"
-"\n\tno - Kjartan Maraas"
-"\n\tpl - GNOME PL Team"
-"\n\tpt - Duarte Loreto"
-"\n\tsk - Marcel Telka"
 "\n\tsv - Christian Rose"
-"\n\tvi - pclouds"
-"\n\tzh_CN - He Qiangqiang" ;
+"\n\tvi - pclouds";
 	return g_convert (str, strlen(str), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
 }
 
@@ -116,9 +108,7 @@ static char *
 get_credits_code (void)
 {
 	const char * str =
-"\n\tJacob Ils Christensen"
-"\n\tFrederic Crozat"
-"\n\tPanagiotis Issaris";
+"\n\tBrandon Corey";
 	return g_convert (str, strlen(str), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
 }
 
@@ -126,31 +116,30 @@ static char *
 get_credits_users (void)
 {
 	const char * str = 
-"\n\tStephen Beahm (#91212)"
-"\n\tGabi Davar (#85442)"
-"\n\tMarc Deslauriers (#88260)"
-"\n\tDevilKin (#92833)"
-"\n\tJoe Drew (#89002)"
-"\n\tDuncan (#94684)"
-"\n\tLaurent Duperval (#88522)"
-"\n\tSteve Fox (#89539)"
-"\n\tNicolas Girard (#93092)"
-"\n\tJohann Glaser (#92645, #91212)"
-"\n\tGollum (#93449, #93452)"
-"\n\tAdam Huffman (#80276)"
-"\n\tjohannes (#91308)"
-"\n\tKang Jeong-Hee (#79270)"
-"\n\tJohn King (#94497)"
-"\n\tChristophe Lambin (#89290)"
-"\n\tKaj-Michael Lang (#91523)"
-"\n\tChristian Lohmaier (#90760, #91532)"
-"\n\toolon (#93538)"
-"\n\to.t. (#90087, #90388)"
-"\n\tJohn Portnoy (#91916)"
-"\n\trapskat (#93229)"
-"\n\tjseymour (#91652)"
-"\n\thverkuil (#84913)"
-"\n\tAdam Warner (#95159)"
-"\n\tAlen Williams (#91348)";
+"\n\tAlan Bailward (#91365)"
+"\n\tJames Brost (#100904)"
+"\n\tBobby D. Bryant (#100695)"
+"\n\tChill"
+"\n\tSid Cowles (#99218)"
+"\n\tWendell Dingus (#90378)"
+"\n\tDuncan"
+"\n\tTom F."
+"\n\tBo Flynn (#100605)"
+"\n\tGollum (#93713, #93894)"
+"\n\tDon Horsman (#97527)"
+"\n\tKang Jeong-Hee (#87553)"
+"\n\tJohan (#909070)"
+"\n\tCarl Hudkins (#85153)"
+"\n\tChristian Lohmaier (#89281)"
+"\n\tJean-Marc (#91536)"
+"\n\tMaurizio (#101013)"
+"\n\tBas Mevissen (#89435)"
+"\n\tLeonard Nutley (#93169)"
+"\n\tChris Petersen (#94863)"
+"\n\tJens Petersen (#99873)"
+"\n\tWill Robinson (#97710)"
+"\n\tMartin Schrode (#98463)"
+"\n\tJerry Segers (#97778)"
+"\n\tMarcin Wojtczuk (#67554)";
 	return g_convert (str, strlen(str), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
 }
