@@ -39,14 +39,14 @@
 #include <pan/base/serverlist.h>
 
 #include <pan/globals.h>
-#include <pan/pan.h>
-#include <pan/gui.h>
 #include <pan/prefs.h>
 #include <pan/util.h>
 
 #include <pan/identities/identity.h>
 #include <pan/identities/identity-manager.h>
 
+#include <pan/pan.h>
+#include <pan/gui.h>
 #include <pan/dialogs/dialogs.h>
 #include <pan/xpm/pan-pixbufs.h>
 
@@ -291,6 +291,7 @@ druid_name_page (Druid * druid)
 {
 	GtkWidget * w;
 	GtkWidget * table;
+	char buf[128];
 
 	table = gtk_table_new (5, 2, FALSE);
 	gtk_table_set_row_spacings (GTK_TABLE(table), GUI_PAD_SMALL);
@@ -320,7 +321,8 @@ druid_name_page (Druid * druid)
 	gtk_label_set_justify (GTK_LABEL(w), GTK_JUSTIFY_LEFT);
 	gtk_table_attach (GTK_TABLE(table), w, 0, 2, 3, 4, GTK_FILL, GTK_FILL, GUI_PAD, GUI_PAD_SMALL);
 
-	w = gtk_label_new (_("Organization:"));
+	g_snprintf (buf, sizeof(buf), "%s:", _("Organization"));
+	w = gtk_label_new (buf);
 	gtk_misc_set_alignment (GTK_MISC(w), 1.0, 0.5);
 	gtk_table_attach (GTK_TABLE (table), w, 0, 1, 4, 5, GTK_FILL, GTK_FILL, GUI_PAD, GUI_PAD_SMALL);
 
@@ -671,10 +673,13 @@ dialog_newuser (void)
 	if (1) {
 		GdkPixbuf * pixbuf = gdk_pixbuf_new_from_inline (-1, icon_pan_druid, FALSE, NULL);
 		GtkWidget * child = gtk_image_new_from_pixbuf (pixbuf);
+		char * welcome = g_strdup_printf (_("Thank you for trying Pan!\n \nIf you are new to Usenet,\n%s\nis a starting point to see what Usenet is,\nwhere to find a free news server,\nand how to use Pan.\n \nSince this is your first time running Pan,\nwe'll need to gather some information.\n"), "http://pan.rebelbase.com/intro.html");
+
 		g_object_unref (G_OBJECT(pixbuf));
 		gtk_box_pack_start (GTK_BOX(w), child, FALSE, FALSE, 0);
-		child = gtk_label_new (_("Thank you for trying Pan!\n \nIf you are new to Usenet,\nhttp://pan.rebelbase.com/intro.php\nis a starting point to see what Usenet is,\nwhere to find a free news server,\nand how to use Pan.\n \nSince this is your first time running Pan,\n we'll need to gather some information.\n"));
+		child = gtk_label_new (welcome);
 		gtk_box_pack_start (GTK_BOX(w), child, FALSE, FALSE, 0);
+		g_free (welcome);
 	}
 
 	/* add user name */
