@@ -1,14 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Author: Charles Kerr <charles@rebelbase.com>
- *
- * Pan - A Newsreader for X
- * Copyright (C) 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,7 +25,8 @@
 
 #include <string.h>
 
-#include <libgnomeui/libgnomeui.h>
+#include <glib.h>
+#include <gtk/gtk.h>
 
 #include <pan/base/debug.h>
 #include <pan/base/pan-i18n.h>
@@ -44,7 +42,6 @@
 #include <pan/filters/filter-linecount.h>
 #include <pan/filters/filter-new.h>
 #include <pan/filters/filter-phrase.h>
-#include <pan/filters/filter-read.h>
 #include <pan/filters/filter-saved.h>
 #include <pan/filters/filter-score.h>
 #include <pan/filters/filter-thread.h>
@@ -102,6 +99,7 @@ typedef enum
 	ARTICLE_IS_MINE,
 	ARTICLE_IS_CACHED,
 	ARTICLE_IS_NEW,
+	ARTICLE_IS_UNREAD,
 	ARTICLE_IS_READ
 }
 ArticleIs;
@@ -141,26 +139,26 @@ ArticleIs;
 typedef struct
 {
 	Filter * filter;
-	gchar * string;
+	char * string;
 }
 FilterView;
 
 static void
-indent_to_depth (GString * str, gint depth)
+indent_to_depth (GString * str, int depth)
 {
-	gint i;
+	int i;
 	for (i=0; i<depth; ++i)
 		g_string_append (str, "          ");
 }
 static void
-get_filter_view_impl (GPtrArray * a, Filter * filter, gint depth)
+get_filter_view_impl (GPtrArray * a, Filter * filter, int depth)
 {
 	const gboolean is_aggregate = filter_isa (filter, FILTER_AGGREGATE_CLASS_ID);
 
 	/* add this item */
 	if (1)
 	{
-		gchar * s;
+		char * s;
 		GString * str = g_string_new (NULL);
 		FilterView * view;
 
@@ -174,10 +172,10 @@ get_filter_view_impl (GPtrArray * a, Filter * filter, gint depth)
 
 		/* maybe add empty notice */
 		if (is_aggregate) {
-			gint children = filter_aggregate_child_size (FILTER_AGGREGATE(filter));
+			int children = filter_aggregate_child_size (FILTER_AGGREGATE(filter));
 			if (children==0) {
 				if (filter->negate)
-					g_string_append (str, _("  (No conditions -- no articles will match"));
+					g_string_append (str, _("  (No conditions -- no articles will match)"));
 				else
 					g_string_append (str, _("  (No conditions -- all articles will match)"));
 			}
@@ -185,9 +183,8 @@ get_filter_view_impl (GPtrArray * a, Filter * filter, gint depth)
 
 	       	view = g_new (FilterView, 1);
 		view->filter = filter;
-		view->string = str->str;
+		view->string = g_string_free (str, FALSE);
 		g_ptr_array_add (a, view);
-		g_string_free (str, FALSE);
 	}
 
 	/* add children */
@@ -216,7 +213,7 @@ get_filter_view (Filter * filter)
 static Filter*
 get_parent (FilterEditDialog * d, Filter * f)
 {
-	gint i;
+	int i;
 	GtkCList * clist;
 	Filter * retval = NULL;
 
@@ -237,10 +234,10 @@ get_parent (FilterEditDialog * d, Filter * f)
 	return retval;
 }
 
-static gint
+static int
 get_selected_index (FilterEditDialog * d)
 {
-	gint sel;
+	int sel;
 	GtkCList * clist;
 
 	/* find the selected filter */
@@ -255,7 +252,7 @@ get_selected_index (FilterEditDialog * d)
 }
 
 static Filter*
-get_filter_at (FilterEditDialog * d, gint index)
+get_filter_at (FilterEditDialog * d, int index)
 {
 	Filter * retval = NULL;
 
@@ -270,7 +267,7 @@ get_filter_at (FilterEditDialog * d, gint index)
 static Filter*
 get_selected_filter (FilterEditDialog * d)
 {
-	gint sel = get_selected_index (d);
+	int sel = get_selected_index (d);
 	if (sel == -1)
 		sel = 0;
 
@@ -289,9 +286,9 @@ get_selected_aggregate (FilterEditDialog * d)
 }
 
 static void
-update_clist_move_sel (FilterEditDialog * d, gint move_sel)
+update_clist_move_sel (FilterEditDialog * d, int move_sel)
 {
-	gint sel;
+	int sel;
 	guint i;
 	Filter * f;
 	GPtrArray * a;
@@ -309,9 +306,9 @@ update_clist_move_sel (FilterEditDialog * d, gint move_sel)
                 sel = GPOINTER_TO_INT(clist->selection->data);
 	gtk_clist_clear (clist);
 	for (i=0; i<a->len; ++i) {
-		gint row;
+		int row;
 		FilterView * view = (FilterView*) g_ptr_array_index (a, i);
-		row = gtk_clist_insert (clist, -1, (gchar**)&view->string);
+		row = gtk_clist_insert (clist, -1, (char**)&view->string);
 		gtk_clist_set_row_data (clist, row, view->filter);
 		g_free (view->string);
 		g_free (view);
@@ -345,7 +342,7 @@ get_new_filter (FilterEditDialog * d)
 		gboolean is_regex = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(d->phrase_regexp_checkbutton));
 		GtkWidget * active = gtk_menu_get_active (GTK_MENU(d->phrase_menu));
 		FilterPhraseState state = GPOINTER_TO_INT(gtk_object_get_user_data (GTK_OBJECT(active)));
-		gchar * phrase = gtk_editable_get_chars (GTK_EDITABLE(d->phrase_entry), 0, -1);
+		char * phrase = gtk_editable_get_chars (GTK_EDITABLE(d->phrase_entry), 0, -1);
                 g_strstrip (phrase);
 		if (is_nonempty_string(phrase)) {
 			f = filter_phrase_new ();
@@ -409,10 +406,13 @@ get_new_filter (FilterEditDialog * d)
 				f = filter_cached_new ();
 				break;
 			case ARTICLE_IS_NEW:
-				f = filter_new_article_new ();
+				f = filter_new_article_new (FILTER_NEW_NEW);
+				break;
+			case ARTICLE_IS_UNREAD:
+				f = filter_new_article_new (FILTER_NEW_UNREAD);
 				break;
 			case ARTICLE_IS_READ:
-				f = filter_read_new ();
+				f = filter_new_article_new (FILTER_NEW_READ);
 				break;
 			default:
 				pan_warn_if_reached ();
@@ -427,13 +427,12 @@ get_new_filter (FilterEditDialog * d)
 ******
 *****/
 
-static gint
-dialog_close_cb (GnomeDialog * dialog, gpointer data)
+static void
+dialog_close_cb (GtkDialog * dialog, gpointer data)
 {
 	FilterEditDialog * d = (FilterEditDialog*) data;
 	pan_object_unref (PAN_OBJECT(d->top));
 	g_free (d);
-	return FALSE;
 }
 
 static void
@@ -442,11 +441,11 @@ up_button_cb (GtkButton * button, gpointer data)
 	FilterEditDialog * d = (FilterEditDialog*) data;
 	Filter * f = get_selected_filter (d);
 	FilterAggregate * p = FILTER_AGGREGATE(get_parent (d, f));
-	gint move_sel = 0;
+	int move_sel = 0;
 
 	if (p != NULL)
 	{
-		gint index = filter_aggregate_child_index (p, f);
+		int index = filter_aggregate_child_index (p, f);
 		if (index > 0)
 		{
 			Filter * sib = filter_aggregate_get_child_at (p, index-1);
@@ -465,7 +464,7 @@ up_button_cb (GtkButton * button, gpointer data)
 			Filter * p2 = get_parent (d, FILTER(p));
 			if (p2 != NULL)
 			{
-				gint index = filter_aggregate_child_index (FILTER_AGGREGATE(p2), FILTER(p));
+				int index = filter_aggregate_child_index (FILTER_AGGREGATE(p2), FILTER(p));
 				pan_object_ref (PAN_OBJECT(f));
 				filter_aggregate_remove (p, &f, 1);
 				filter_aggregate_insert (FILTER_AGGREGATE(p2), &f, 1, index);
@@ -481,15 +480,15 @@ up_button_cb (GtkButton * button, gpointer data)
 static void
 down_button_cb (GtkButton * button, gpointer data)
 {
-	gint move_sel = 0;
+	int move_sel = 0;
 	FilterEditDialog * d = (FilterEditDialog*) data;
 	Filter * f = get_selected_filter (d);
 	FilterAggregate * p = FILTER_AGGREGATE(get_parent (d, f));
 
 	if (p != NULL)
 	{
-		gint index = filter_aggregate_child_index (p, f);
-		gint size = filter_aggregate_child_size (p);
+		int index = filter_aggregate_child_index (p, f);
+		int size = filter_aggregate_child_size (p);
 		if (index < size-1) /* down */
 		{
 			Filter * sib = filter_aggregate_get_child_at (p, index+1);
@@ -507,7 +506,7 @@ down_button_cb (GtkButton * button, gpointer data)
 			Filter * p2 = get_parent (d, FILTER(p));
 			if (p2 != NULL)
 			{
-				gint index = filter_aggregate_child_index (FILTER_AGGREGATE(p2), FILTER(p));
+				int index = filter_aggregate_child_index (FILTER_AGGREGATE(p2), FILTER(p));
 				pan_object_ref (PAN_OBJECT(f));
 				filter_aggregate_remove (p, &f, 1);
 				filter_aggregate_insert (FILTER_AGGREGATE(p2), &f, 1, index+1);
@@ -545,7 +544,7 @@ aggregate_action (FilterEditDialog * d, FilterAggregateType type)
 	{
 		Filter * p = get_parent (d, f);
 		Filter * a = filter_aggregate_new ();
-		gint index = filter_aggregate_child_index (FILTER_AGGREGATE(p), f);
+		int index = filter_aggregate_child_index (FILTER_AGGREGATE(p), f);
 		filter_aggregate_set_type (FILTER_AGGREGATE(a), type);
 		filter_aggregate_insert (FILTER_AGGREGATE(p), &a, 1, index+1);
 		pan_object_unref (PAN_OBJECT(a));
@@ -603,7 +602,7 @@ static void
 name_editable_changed_cb (GtkEditable * editable, gpointer data)
 {
 	FilterEditDialog * d = (FilterEditDialog*) data;
-	gchar * pch = gtk_editable_get_chars (editable, 0, -1);
+	char * pch = gtk_editable_get_chars (editable, 0, -1);
 	filter_top_set_name (FILTER_TOP(d->top), pch);
 	g_free (pch);
 }
@@ -631,13 +630,12 @@ tb_activate (GtkToggleButton * tb)
 
 
 GtkWidget*
-filter_edit_dialog_new (const FilterTop * top)
+filter_edit_dialog_new (GtkWindow * window, const FilterTop * top)
 {
-	const gint ROWS = 2;
-	const gint COLS = 10;
-	gint row;
-	gint col;
-	guint uint;
+	const int ROWS = 2;
+	const int COLS = 10;
+	int row;
+	int col;
 	GtkWidget * w;
 	GtkWidget * w2;
 	GtkWidget * tmp;
@@ -647,7 +645,6 @@ filter_edit_dialog_new (const FilterTop * top)
 	GtkWidget * vbox_main;
 	GtkObject * adj;
 	GtkTooltips * tips = gtk_tooltips_new ();
-	GtkAccelGroup * accel_group = gtk_accel_group_new ();
 	GSList * rb_group = NULL;
 	FilterEditDialog * d;
 
@@ -655,30 +652,33 @@ filter_edit_dialog_new (const FilterTop * top)
 	d->top = top!=NULL ? filter_dup(FILTER(top)) : filter_top_new ();
 
 	/* dialog */
-	w = d->dialog = gnome_dialog_new (_("Pan: Edit Filter"),
-		GNOME_STOCK_BUTTON_OK,
-		GNOME_STOCK_BUTTON_APPLY,
-		GNOME_STOCK_BUTTON_CANCEL,
-		NULL);
-	vbox_main = GNOME_DIALOG(w)->vbox;
+	w = d->dialog = gtk_dialog_new_with_buttons (_("Pan: Edit Filter"),
+	                                             GTK_WINDOW(window),
+	                                             GTK_DIALOG_DESTROY_WITH_PARENT,
+	                                             GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+	                                             GTK_STOCK_APPLY, GTK_RESPONSE_APPLY,
+	                                             GTK_STOCK_OK, GTK_RESPONSE_OK,
+	                                             NULL);
+	vbox_main = GTK_DIALOG(w)->vbox;
+	gtk_box_set_spacing (GTK_BOX(vbox_main), GUI_PAD);
+	gtk_container_set_border_width (GTK_CONTAINER(vbox_main), GUI_PAD);
 	gtk_window_set_policy (GTK_WINDOW(w), TRUE, TRUE, TRUE);
-	gtk_signal_connect (GTK_OBJECT(w), "close",
-	                    GTK_SIGNAL_FUNC(dialog_close_cb), d);
+	g_signal_connect (GTK_OBJECT(w), "destroy", G_CALLBACK(dialog_close_cb), d);
 
 	/* top row: filter name, show in filter menu */
-	hbox = gtk_hbox_new (FALSE, GNOME_PAD);
+	hbox = gtk_hbox_new (FALSE, GUI_PAD);
 	gtk_box_pack_start (GTK_BOX(vbox_main), hbox, FALSE, TRUE, 1);
-	w = gtk_label_new (_("Filter Name: "));
+	w = gtk_label_new (_("Filter name: "));
 	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
 	w = d->filter_name_entry = gtk_entry_new ();
-	gtk_entry_set_text (GTK_ENTRY(w), FILTER_TOP(d->top)->name);
-	gtk_signal_connect (GTK_OBJECT(w), "changed",
-	                    GTK_SIGNAL_FUNC(name_editable_changed_cb), d);
+	pan_gtk_entry_set_text (w, FILTER_TOP(d->top)->name);
+	g_signal_connect (GTK_OBJECT(w), "changed",
+	                    G_CALLBACK(name_editable_changed_cb), d);
 	gtk_box_pack_start (GTK_BOX(hbox), w, TRUE, TRUE, 0);
 	w = d->show_in_menu_checkbutton = gtk_check_button_new_with_label (_("Show in Filter Menu"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), FILTER_TOP(d->top)->is_visible);
-	gtk_signal_connect (GTK_OBJECT(w), "toggled",
-	                    GTK_SIGNAL_FUNC(in_menu_togglebutton_toggled_cb), d);
+	g_signal_connect (GTK_OBJECT(w), "toggled",
+	                    G_CALLBACK(in_menu_togglebutton_toggled_cb), d);
 	gtk_box_pack_end (GTK_BOX(hbox), w, FALSE, FALSE, 0);
 
 	/* separator */
@@ -686,9 +686,10 @@ filter_edit_dialog_new (const FilterTop * top)
 	gtk_box_pack_start (GTK_BOX(vbox_main), w, FALSE, FALSE, 10);
 
 	/* rules clist + its buttons */
-	hbox = gtk_hbox_new (FALSE, GNOME_PAD);
+	hbox = gtk_hbox_new (FALSE, GUI_PAD);
 	gtk_box_pack_start (GTK_BOX(vbox_main), hbox, TRUE, TRUE, 0);
 	w = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(w), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_box_pack_start (GTK_BOX(hbox), w, TRUE, TRUE, 0);
 	w2 = w;
 	w = d->view_clist = gtk_clist_new (1);
@@ -696,36 +697,34 @@ filter_edit_dialog_new (const FilterTop * top)
 	gtk_container_add (GTK_CONTAINER(w2), w);
 	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
-	w = gtk_button_new_with_label (_("Move Up"));
-	gtk_signal_connect (GTK_OBJECT(w), "clicked", GTK_SIGNAL_FUNC(up_button_cb), d);
+	w = gtk_button_new_from_stock (GTK_STOCK_GO_UP);
+	g_signal_connect (GTK_OBJECT(w), "clicked", G_CALLBACK(up_button_cb), d);
 	gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
 	gtk_tooltips_set_tip (tips, w, _("Move the selected line up one line"), NULL);
-	w = gtk_button_new_with_label (_("Move Down"));
-	gtk_signal_connect (GTK_OBJECT(w), "clicked", GTK_SIGNAL_FUNC(down_button_cb), d);
+	w = gtk_button_new_from_stock (GTK_STOCK_GO_DOWN);
+	g_signal_connect (GTK_OBJECT(w), "clicked", G_CALLBACK(down_button_cb), d);
 	gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
 	gtk_tooltips_set_tip (tips, w, _("Move the selected line down one line"), NULL);
 	w = gtk_button_new_with_label (_("Negate"));
-	gtk_signal_connect (GTK_OBJECT(w), "clicked", GTK_SIGNAL_FUNC(negate_button_cb), d);
+	g_signal_connect (GTK_OBJECT(w), "clicked", G_CALLBACK(negate_button_cb), d);
 	gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
 	gtk_tooltips_set_tip (tips, w, _("Negate the selected line"), NULL);
 	w = gtk_button_new_with_label (_("AND"));
-	gtk_signal_connect (GTK_OBJECT(w), "clicked", GTK_SIGNAL_FUNC(and_button_cb), d);
+	g_signal_connect (GTK_OBJECT(w), "clicked", G_CALLBACK(and_button_cb), d);
 	gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
 	w = gtk_button_new_with_label (_("OR"));
-	gtk_signal_connect (GTK_OBJECT(w), "clicked", GTK_SIGNAL_FUNC(or_button_cb), d);
+	g_signal_connect (GTK_OBJECT(w), "clicked", G_CALLBACK(or_button_cb), d);
 	gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
-	w = gtk_button_new_with_label (_("Delete Line"));
-	gtk_signal_connect (GTK_OBJECT(w), "clicked", GTK_SIGNAL_FUNC(delete_button_cb), d);
+	w = gtk_button_new_from_stock (GTK_STOCK_DELETE);
+	g_signal_connect (GTK_OBJECT(w), "clicked", G_CALLBACK(delete_button_cb), d);
 	gtk_box_pack_end (GTK_BOX(vbox), w, FALSE, FALSE, 0);
 	gtk_tooltips_set_tip (tips, w, _("Delete the selected line"), NULL);
 
 	/* add-condition button */
 	hbox = gtk_hbutton_box_new ();
 	gtk_box_pack_start (GTK_BOX(vbox_main), hbox, FALSE, FALSE, 0);
-	w = gnome_stock_button (GNOME_STOCK_BUTTON_UP);
-
-	gtk_signal_connect (GTK_OBJECT(w), "clicked", GTK_SIGNAL_FUNC(add_button_cb), d);
-	gtk_tooltips_set_tip (tips, w, _("Add new Line to Filter"), NULL);
+	w = gtk_button_new_with_mnemonic (_("Add New Line to the Filter"));
+	g_signal_connect (GTK_OBJECT(w), "clicked", G_CALLBACK(add_button_cb), d);
 	gtk_container_add (GTK_CONTAINER(hbox), w);
 
 	/* hoo-boy, the conditions */
@@ -738,11 +737,9 @@ filter_edit_dialog_new (const FilterTop * top)
 	row = col = 0;
 
 	/* phrase */
-	hbox = gtk_hbox_new (FALSE, GNOME_PAD);
+	hbox = gtk_hbox_new (FALSE, GUI_PAD);
 	gtk_table_attach (GTK_TABLE(table), hbox, 0, COLS, row, row+1, GTK_FILL, GTK_FILL, 0, 0);
-	w = d->phrase_rb = gtk_radio_button_new_with_label (rb_group, "");
-	uint = gtk_label_parse_uline (GTK_LABEL(GTK_BIN(w)->child), _("_Article"));
-	gtk_widget_add_accelerator (w, "clicked", accel_group, uint, GDK_MOD1_MASK, 0);
+	w = d->phrase_rb = gtk_radio_button_new_with_mnemonic (rb_group, _("_Article"));
 	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
 	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
 	w = gtk_option_menu_new ();
@@ -750,193 +747,178 @@ filter_edit_dialog_new (const FilterTop * top)
 	w2 = d->phrase_menu = gtk_menu_new ();
 	tmp = gtk_menu_item_new_with_label (_("Subject"));
 	gtk_object_set_user_data (GTK_OBJECT(tmp), GINT_TO_POINTER(FILTER_PHRASE_SUBJECT));
-	gtk_signal_connect_object (GTK_OBJECT(tmp), "activate", tb_activate, GTK_OBJECT(d->phrase_rb));
+	g_signal_connect_swapped (GTK_OBJECT(tmp), "activate", G_CALLBACK(tb_activate), GTK_OBJECT(d->phrase_rb));
 	gtk_widget_show (tmp);
 	gtk_menu_append (GTK_MENU(w2), tmp);
 	tmp = gtk_menu_item_new_with_label (_("Author"));
 	gtk_object_set_user_data (GTK_OBJECT(tmp), GINT_TO_POINTER(FILTER_PHRASE_AUTHOR));
-	gtk_signal_connect_object (GTK_OBJECT(tmp), "activate", tb_activate, GTK_OBJECT(d->phrase_rb));
+	g_signal_connect_swapped (GTK_OBJECT(tmp), "activate", G_CALLBACK(tb_activate), GTK_OBJECT(d->phrase_rb));
 	gtk_widget_show (tmp);
 	gtk_menu_append (GTK_MENU(w2), tmp);
 	tmp = gtk_menu_item_new_with_label (_("Message-ID"));
 	gtk_object_set_user_data (GTK_OBJECT(tmp), GINT_TO_POINTER(FILTER_PHRASE_MESSAGE_ID));
-	gtk_signal_connect_object (GTK_OBJECT(tmp), "activate", tb_activate, GTK_OBJECT(d->phrase_rb));
+	g_signal_connect_swapped (GTK_OBJECT(tmp), "activate", G_CALLBACK(tb_activate), GTK_OBJECT(d->phrase_rb));
 	gtk_widget_show (tmp);
 	gtk_menu_append (GTK_MENU(w2), tmp);
 	gtk_option_menu_set_menu (GTK_OPTION_MENU(w), w2);
 	w = gtk_label_new (_("contains the phrase"));
 	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
 	w = d->phrase_entry = gtk_entry_new ();
-	gtk_signal_connect_object (GTK_OBJECT(w), "focus_in_event", tb_activate, GTK_OBJECT(d->phrase_rb));
+	g_signal_connect_swapped (GTK_OBJECT(w), "focus_in_event", G_CALLBACK(tb_activate), GTK_OBJECT(d->phrase_rb));
 	gtk_box_pack_start (GTK_BOX(hbox), w, TRUE, TRUE, 0);
 	w = d->phrase_regexp_checkbutton = gtk_check_button_new_with_label (_("Regular Expression"));
 	gtk_tooltips_set_tip (tips, w,_("See http://pan.rebelbase.com/regexp.txt for a tutorial on how to use regular expressions.  If regular expressions are turned off, a simple case-insensitive wildcard match is used."), NULL);
-	gtk_signal_connect_object (GTK_OBJECT(w), "focus_in_event", tb_activate, GTK_OBJECT(d->phrase_rb));
+	g_signal_connect_swapped (GTK_OBJECT(w), "focus_in_event", G_CALLBACK(tb_activate), GTK_OBJECT(d->phrase_rb));
 	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
-#if 0
-	w = gtk_label_new (_("(%s for user-specified text)"));
-	gtk_table_attach (GTK_TABLE(table), w, 2, 4, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
-#endif
 
 	/* saved */
 	++row;
-	w = d->saved_rb = gtk_radio_button_new_with_label (rb_group, "");
+	w = d->saved_rb = gtk_radio_button_new_with_mnemonic (rb_group, _("Article has been"));
 	gtk_table_attach (GTK_TABLE(table), w, col, col+1, row, row+1, GTK_FILL, 0, 0, 0);
-	uint = gtk_label_parse_uline (GTK_LABEL(GTK_BIN(w)->child), _("Article has been"));
-	gtk_widget_add_accelerator (w, "clicked", accel_group, uint, GDK_MOD1_MASK, 0);
 	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
 	w = gtk_option_menu_new ();
 	gtk_table_attach (GTK_TABLE(table), w, col+1, col+2, row, row+1, GTK_FILL, 0, 0, 0);
 	w2 = d->saved_menu = gtk_menu_new ();
 	tmp = gtk_menu_item_new_with_label (_("Saved"));
 	gtk_object_set_user_data (GTK_OBJECT(tmp), GINT_TO_POINTER(FILTER_SAVED_SAVED));
-	gtk_signal_connect_object (GTK_OBJECT(tmp), "activate", tb_activate, GTK_OBJECT(d->saved_rb));
+	g_signal_connect_swapped (GTK_OBJECT(tmp), "activate", G_CALLBACK(tb_activate), GTK_OBJECT(d->saved_rb));
 	gtk_widget_show (tmp);
 	gtk_menu_append (GTK_MENU(w2), tmp);
 	tmp = gtk_menu_item_new_with_label (_("Queued for Saving"));
 	gtk_object_set_user_data (GTK_OBJECT(tmp), GINT_TO_POINTER(FILTER_SAVED_QUEUED));
-	gtk_signal_connect_object (GTK_OBJECT(tmp), "activate", tb_activate, GTK_OBJECT(d->saved_rb));
+	g_signal_connect_swapped (GTK_OBJECT(tmp), "activate", G_CALLBACK(tb_activate), GTK_OBJECT(d->saved_rb));
 	gtk_widget_show (tmp);
 	gtk_menu_append (GTK_MENU(w2), tmp);
 	tmp = gtk_menu_item_new_with_label (_("Neither Saved nor Queued"));
 	gtk_object_set_user_data (GTK_OBJECT(tmp), GINT_TO_POINTER(FILTER_SAVED_NOT_SAVED));
-	gtk_signal_connect_object (GTK_OBJECT(tmp), "activate", tb_activate, GTK_OBJECT(d->saved_rb));
+	g_signal_connect_swapped (GTK_OBJECT(tmp), "activate", G_CALLBACK(tb_activate), GTK_OBJECT(d->saved_rb));
 	gtk_widget_show (tmp);
 	gtk_menu_append (GTK_MENU(w2), tmp);
 	gtk_option_menu_set_menu (GTK_OPTION_MENU(w), w2);
 
 	/* thread */
 	++row;
-	w = d->thread_rb = gtk_radio_button_new_with_label (rb_group, "");
+	w = d->thread_rb = gtk_radio_button_new_with_mnemonic (rb_group, _("Article _thread is"));
 	gtk_table_attach (GTK_TABLE(table), w, col, col+1, row, row+1, GTK_FILL, 0, 0, 0);
-	uint = gtk_label_parse_uline (GTK_LABEL(GTK_BIN(w)->child), _("Article _thread is"));
-	gtk_widget_add_accelerator (w, "clicked", accel_group, uint, GDK_MOD1_MASK, 0);
 	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
 	w = gtk_option_menu_new ();
 	gtk_table_attach (GTK_TABLE(table), w, col+1, col+2, row, row+1, GTK_FILL, 0, 0, 0);
 	w2 = d->thread_menu = gtk_menu_new ();
 	tmp = gtk_menu_item_new_with_label (_("Watched"));
 	gtk_object_set_user_data (GTK_OBJECT(tmp), GINT_TO_POINTER(FILTER_THREAD_WATCHED));
-	gtk_signal_connect_object (GTK_OBJECT(tmp), "activate", tb_activate, GTK_OBJECT(d->thread_rb));
+	g_signal_connect_swapped (GTK_OBJECT(tmp), "activate", G_CALLBACK(tb_activate), GTK_OBJECT(d->thread_rb));
 	gtk_widget_show (tmp);
 	gtk_menu_append (GTK_MENU(w2), tmp);
 	tmp = gtk_menu_item_new_with_label (_("Ignored"));
 	gtk_object_set_user_data (GTK_OBJECT(tmp), GINT_TO_POINTER(FILTER_THREAD_IGNORED));
-	gtk_signal_connect_object (GTK_OBJECT(tmp), "activate", tb_activate, GTK_OBJECT(d->thread_rb));
+	g_signal_connect_swapped (GTK_OBJECT(tmp), "activate", G_CALLBACK(tb_activate), GTK_OBJECT(d->thread_rb));
 	gtk_widget_show (tmp);
 	gtk_menu_append (GTK_MENU(w2), tmp);
 	tmp = gtk_menu_item_new_with_label (_("Neither Watched nor Ignored"));
 	gtk_object_set_user_data (GTK_OBJECT(tmp), GINT_TO_POINTER(FILTER_THREAD_NOTHING));
-	gtk_signal_connect_object (GTK_OBJECT(tmp), "activate", tb_activate, GTK_OBJECT(d->thread_rb));
+	g_signal_connect_swapped (GTK_OBJECT(tmp), "activate", G_CALLBACK(tb_activate), GTK_OBJECT(d->thread_rb));
 	gtk_widget_show (tmp);
 	gtk_menu_append (GTK_MENU(w2), tmp);
 	gtk_option_menu_set_menu (GTK_OPTION_MENU(w), w2);
 
 	/* attachment */
 	++row;
-	w = d->attach_rb = gtk_radio_button_new_with_label (rb_group, "");
+	w = d->attach_rb = gtk_radio_button_new_with_mnemonic (rb_group, _("Article h_as"));
 	gtk_table_attach (GTK_TABLE(table), w, col, col+1, row, row+1, GTK_FILL, 0, 0, 0);
-	uint = gtk_label_parse_uline (GTK_LABEL(GTK_BIN(w)->child), _("Article h_as"));
-	gtk_widget_add_accelerator (w, "clicked", accel_group, uint, GDK_MOD1_MASK, 0);
 	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
 	w = gtk_option_menu_new ();
 	gtk_table_attach (GTK_TABLE(table), w, col+1, col+2, row, row+1, GTK_FILL, 0, 0, 0);
 	w2 = d->attach_menu = gtk_menu_new ();
 	tmp = gtk_menu_item_new_with_label (_("Attachments"));
 	gtk_object_set_user_data (GTK_OBJECT(tmp), GINT_TO_POINTER(FILTER_BINARY_COMPLETE));
-	gtk_signal_connect_object (GTK_OBJECT(tmp), "activate", tb_activate, GTK_OBJECT(d->attach_rb));
+	g_signal_connect_swapped (GTK_OBJECT(tmp), "activate", G_CALLBACK(tb_activate), GTK_OBJECT(d->attach_rb));
 	gtk_widget_show (tmp);
 	gtk_menu_append (GTK_MENU(w2), tmp);
 	tmp = gtk_menu_item_new_with_label (_("Incomplete Attachments"));
 	gtk_object_set_user_data (GTK_OBJECT(tmp), GINT_TO_POINTER(FILTER_BINARY_INCOMPLETE));
-	gtk_signal_connect_object (GTK_OBJECT(tmp), "activate", tb_activate, GTK_OBJECT(d->attach_rb));
+	g_signal_connect_swapped (GTK_OBJECT(tmp), "activate", G_CALLBACK(tb_activate), GTK_OBJECT(d->attach_rb));
 	gtk_widget_show (tmp);
 	gtk_menu_append (GTK_MENU(w2), tmp);
 	tmp = gtk_menu_item_new_with_label (_("No Attachments"));
 	gtk_object_set_user_data (GTK_OBJECT(tmp), GINT_TO_POINTER(FILTER_BINARY_NONBINARY));
-	gtk_signal_connect_object (GTK_OBJECT(tmp), "activate", tb_activate, GTK_OBJECT(d->attach_rb));
+	g_signal_connect_swapped (GTK_OBJECT(tmp), "activate", G_CALLBACK(tb_activate), GTK_OBJECT(d->attach_rb));
 	gtk_widget_show (tmp);
 	gtk_menu_append (GTK_MENU(w2), tmp);
 	gtk_option_menu_set_menu (GTK_OPTION_MENU(w), w2);
 
 	/* read, cached, new, posted-by-me */
 	++row;
-	w = d->article_is_rb = gtk_radio_button_new_with_label (rb_group, "");
+	w = d->article_is_rb = gtk_radio_button_new_with_mnemonic (rb_group, _("Article _is"));
 	gtk_table_attach (GTK_TABLE(table), w, col, col+1, row, row+1, GTK_FILL, 0, 0, 0);
-	uint = gtk_label_parse_uline (GTK_LABEL(GTK_BIN(w)->child), _("Article _is"));
-	gtk_widget_add_accelerator (w, "clicked", accel_group, uint, GDK_MOD1_MASK, 0);
 	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
 	w = gtk_option_menu_new ();
 	gtk_table_attach (GTK_TABLE(table), w, col+1, col+2, row, row+1, GTK_FILL, 0, 0, 0);
 	w2 = d->article_is_menu = gtk_menu_new ();
-	tmp = gtk_menu_item_new_with_label (_("Read"));
-	gtk_object_set_user_data (GTK_OBJECT(tmp), GINT_TO_POINTER(ARTICLE_IS_READ));
-	gtk_signal_connect_object (GTK_OBJECT(tmp), "activate", tb_activate, GTK_OBJECT(d->article_is_rb));
-	gtk_widget_show (tmp);
-	gtk_menu_append (GTK_MENU(w2), tmp);
 	tmp = gtk_menu_item_new_with_label (_("New"));
 	gtk_object_set_user_data (GTK_OBJECT(tmp), GINT_TO_POINTER(ARTICLE_IS_NEW));
-	gtk_signal_connect_object (GTK_OBJECT(tmp), "activate", tb_activate, GTK_OBJECT(d->article_is_rb));
+	g_signal_connect_swapped (GTK_OBJECT(tmp), "activate", G_CALLBACK(tb_activate), GTK_OBJECT(d->article_is_rb));
 	gtk_widget_show (tmp);
 	gtk_menu_append (GTK_MENU(w2), tmp);
-	tmp = gtk_menu_item_new_with_label (_("Cached Locally"));
+	tmp = gtk_menu_item_new_with_label (_("Unread"));
+	gtk_object_set_user_data (GTK_OBJECT(tmp), GINT_TO_POINTER(ARTICLE_IS_UNREAD));
+	g_signal_connect_swapped (GTK_OBJECT(tmp), "activate", G_CALLBACK(tb_activate), GTK_OBJECT(d->article_is_rb));
+	gtk_widget_show (tmp);
+	gtk_menu_append (GTK_MENU(w2), tmp);
+	tmp = gtk_menu_item_new_with_label (_("Read"));
+	gtk_object_set_user_data (GTK_OBJECT(tmp), GINT_TO_POINTER(ARTICLE_IS_READ));
+	g_signal_connect_swapped (GTK_OBJECT(tmp), "activate", G_CALLBACK(tb_activate), GTK_OBJECT(d->article_is_rb));
+	gtk_widget_show (tmp);
+	gtk_menu_append (GTK_MENU(w2), tmp);
+	tmp = gtk_menu_item_new_with_label (_("Cached"));
 	gtk_object_set_user_data (GTK_OBJECT(tmp), GINT_TO_POINTER(ARTICLE_IS_CACHED));
-	gtk_signal_connect_object (GTK_OBJECT(tmp), "activate", tb_activate, GTK_OBJECT(d->article_is_rb));
+	g_signal_connect_swapped (GTK_OBJECT(tmp), "activate", G_CALLBACK(tb_activate), GTK_OBJECT(d->article_is_rb));
 	gtk_widget_show (tmp);
 	gtk_menu_append (GTK_MENU(w2), tmp);
 	tmp = gtk_menu_item_new_with_label (_("Posted by Me"));
 	gtk_object_set_user_data (GTK_OBJECT(tmp), GINT_TO_POINTER(ARTICLE_IS_MINE));
-	gtk_signal_connect_object (GTK_OBJECT(tmp), "activate", tb_activate, GTK_OBJECT(d->article_is_rb));
+	g_signal_connect_swapped (GTK_OBJECT(tmp), "activate", G_CALLBACK(tb_activate), GTK_OBJECT(d->article_is_rb));
 	gtk_widget_show (tmp);
 	gtk_menu_append (GTK_MENU(w2), tmp);
 	gtk_option_menu_set_menu (GTK_OPTION_MENU(w), w2);
 
 	/* lines */
 	++row;
-	w = d->lines_rb = gtk_radio_button_new_with_label (rb_group, "");
+	w = d->lines_rb = gtk_radio_button_new_with_mnemonic (rb_group, _("Article has at _least N lines:"));
 	gtk_table_attach (GTK_TABLE(table), w, col, col+1, row, row+1, GTK_FILL, 0, 0, 0);
-	uint = gtk_label_parse_uline (GTK_LABEL(GTK_BIN(w)->child), _("Article has at _least N lines:"));
-	gtk_widget_add_accelerator (w, "clicked", accel_group, uint, GDK_MOD1_MASK, 0);
 	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
 	adj = gtk_adjustment_new (1, 0, 300000, 1, 10, 10);
 	w = d->lines_sb = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1, 0);
-	gtk_signal_connect_object (GTK_OBJECT(w), "focus_in_event", tb_activate, GTK_OBJECT(d->lines_rb));
+	g_signal_connect_swapped (GTK_OBJECT(w), "focus_in_event", G_CALLBACK(tb_activate), GTK_OBJECT(d->lines_rb));
 	gtk_table_attach (GTK_TABLE(table), w, col+1, col+2, row, row+1, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 
 	/* crosspost */
 	++row;
-	w = d->xpost_rb = gtk_radio_button_new_with_label (rb_group, "");
+	w = d->xpost_rb = gtk_radio_button_new_with_mnemonic (rb_group, _("Article was _posted to at least N groups:"));
 	gtk_table_attach (GTK_TABLE(table), w, col, col+1, row, row+1, GTK_FILL, 0, 0, 0);
-	uint = gtk_label_parse_uline (GTK_LABEL(GTK_BIN(w)->child), _("Article was _posted to at least N groups:"));
-	gtk_widget_add_accelerator (w, "clicked", accel_group, uint, GDK_MOD1_MASK, 0);
 	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
 	adj = gtk_adjustment_new (1, 0, 300000, 1, 10, 10);
 	w = d->xpost_sb = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1, 0);
-	gtk_signal_connect_object (GTK_OBJECT(w), "focus_in_event", tb_activate, GTK_OBJECT(d->xpost_rb));
+	g_signal_connect_swapped (GTK_OBJECT(w), "focus_in_event", G_CALLBACK(tb_activate), GTK_OBJECT(d->xpost_rb));
 	gtk_table_attach (GTK_TABLE(table), w, col+1, col+2, row, row+1, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 
 	/* age */
 	++row;
-	w = d->age_rb = gtk_radio_button_new_with_label (rb_group, "");
+	w = d->age_rb = gtk_radio_button_new_with_mnemonic (rb_group, _("Article is at least N days _old"));
 	gtk_table_attach (GTK_TABLE(table), w, col, col+1, row, row+1, GTK_FILL, 0, 0, 0);
-	uint = gtk_label_parse_uline (GTK_LABEL(GTK_BIN(w)->child), _("Article is at least N days _old"));
-	gtk_widget_add_accelerator (w, "clicked", accel_group, uint, GDK_MOD1_MASK, 0);
 	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
 	adj = gtk_adjustment_new (1, 0, 300000, 1, 10, 10);
 	w = d->age_sb = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1, 0);
-	gtk_signal_connect_object (GTK_OBJECT(w), "focus_in_event", tb_activate, GTK_OBJECT(d->age_rb));
+	g_signal_connect_swapped (GTK_OBJECT(w), "focus_in_event", G_CALLBACK(tb_activate), GTK_OBJECT(d->age_rb));
 	gtk_table_attach (GTK_TABLE(table), w, col+1, col+2, row, row+1, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 
 	/* score */
 	++row;
-	w = d->score_rb = gtk_radio_button_new_with_label (rb_group, "");
+	w = d->score_rb = gtk_radio_button_new_with_mnemonic (rb_group, _("Article _score is at least"));
 	gtk_table_attach (GTK_TABLE(table), w, col, col+1, row, row+1, GTK_FILL, 0, 0, 0);
-	uint = gtk_label_parse_uline (GTK_LABEL(GTK_BIN(w)->child), _("Article _score is at least"));
-	gtk_widget_add_accelerator (w, "clicked", accel_group, uint, GDK_MOD1_MASK, 0);
 	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
 	adj = gtk_adjustment_new (0, -32768, 32767, 1, 1, 1);
 	w = d->score_sb = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1, 0);
-	gtk_signal_connect_object (GTK_OBJECT(w), "focus_in_event", tb_activate, GTK_OBJECT(d->score_rb));
+	g_signal_connect_swapped (GTK_OBJECT(w), "focus_in_event", G_CALLBACK(tb_activate), GTK_OBJECT(d->score_rb));
 	gtk_table_attach (GTK_TABLE(table), w, col+1, col+2, row, row+1, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 
 	/* prepare the ui state */
