@@ -382,9 +382,12 @@ show_only_mine_cb (GtkToggleButton * tb, gpointer user_data)
 }
 
 static void
-show_toggled_cb (GtkToggleButton * tb, gpointer user_data)
+show_toggled_cb (GtkCheckMenuItem * check, gpointer user_data)
 {
-	set_show (GPOINTER_TO_SIZE(user_data));
+	g_return_if_fail (GTK_IS_CHECK_MENU_ITEM(check));
+
+	if (check->active)
+		set_show (GPOINTER_TO_SIZE(user_data));
 }
 
 /***
