@@ -94,31 +94,12 @@ add_any_needed_canned_filters (GPtrArray * filters)
 	}
 }
 
-
-/*static GStaticMutex _fm_lock = G_STATIC_MUTEX_INIT;
-static gboolean     _fm_inited = FALSE;*/
-
-/*static gboolean
-is_inited (void)
-{
-	gboolean inited = FALSE;
-	g_static_mutex_lock (&_fm_lock);
-	inited = _fm_inited;
-	g_static_mutex_unlock (&_fm_lock);
-
-	return inited;
-}*/
-
 void
 filter_manager_init (void)
 {
 
 	char * fname;
 
-
-
-	
-
 	bozo_filter_name = _("Bozos");
 	spam_filter_name = _("Spam");
 
