@@ -24,7 +24,6 @@
 #include <glib.h>
 
 #include <pan/base/debug.h>
-#include <pan/base/fnmatch.h>
 #include <pan/base/pan-glib-extensions.h>
 #include <pan/base/pan-i18n.h>
 
@@ -109,8 +108,16 @@ filter_phrase_test_articles (Filter          * filter,
                              gint              article_qty,
                              gboolean        * passfail)
 {
-	gint i;
+	int i;
 	FilterPhrase * f = FILTER_PHRASE(filter);
+	GPatternSpec * pattern = NULL;
+
+	if (!f->is_regex) {
+		char * phrase = strchr(f->phrase,'*') ? g_strdup(f->phrase) : g_strdup_printf ("*%s*", f->phrase);
+		replace_gstr (&phrase, g_utf8_strdown(phrase,-1));
+		pattern = g_pattern_spec_new (phrase);
+		g_free (phrase);
+	}
 
 	for (i=0; i<article_qty; ++i)
 	{
@@ -129,10 +136,17 @@ filter_phrase_test_articles (Filter          * filter,
 				break;
 		}
 
-		passfail[i] = f->is_regex
-			? !regexec (&f->phrase_regex,  checkme, 0, NULL, 0)
-			: !fnmatch (f->phrase, checkme, PAN_CASEFOLD);
+		if (f->is_regex)
+			passfail[i] = !regexec (&f->phrase_regex,  checkme, 0, NULL, 0);
+		else {
+			char * lower = g_utf8_strdown (checkme, -1);
+			passfail[i] = g_pattern_match (pattern, strlen(lower), lower, NULL);
+			g_free (lower);
+		}
 	}
+
+	if (pattern != NULL)
+		g_pattern_spec_free (pattern);
 }
 
 static void
@@ -197,17 +211,10 @@ filter_phrase_set (FilterPhrase        * filter,
 	filter->is_regex = is_regex;
 	if (is_nonempty_string (phrase))
 	{
+		filter->phrase = g_strdup (phrase);
+
 		if (filter->is_regex)
-		{
-			filter->phrase = g_strdup (phrase);
 			regcomp (&filter->phrase_regex, filter->phrase, REG_EXTENDED|REG_ICASE);
-		}
-		else
-		{
-			filter->phrase = strchr(phrase,'*')!=NULL
-				? g_strdup (phrase)
-				: g_strdup_printf ("*%s*", phrase);
-		}
 	}
 
 	debug_exit ("filter_phrase_set");
