@@ -39,6 +39,7 @@
 #include <pan/base/pan-i18n.h>
 #include <pan/base/serverlist.h>
 #include <pan/base/status-item.h>
+#include <pan/base/base-prefs.h>
 
 #include <pan/rules/rule-ui.h>
 
@@ -94,7 +95,6 @@ typedef enum
 	ACTION_GET_NEW_HEADERS_FOR_SUBSCRIBED,
 	ACTION_GET_NEW_HEADERS_FOR_SELECTED,
 	ACTION_GET_BODIES_FOR_SELECTED,
-	/*ACTION_GET_BODIES_FOR_FLAGGED,*/
 	ACTION_READ_MORE,
 	ACTION_READ_LESS,
 	ACTION_READ_NEXT_UNREAD_ARTICLE,
@@ -136,7 +136,6 @@ typedef struct
 StatusIcon;
 static StatusType _current_status;
 static GtkWidget * _status_image = NULL;
-static GtkWidget * _status_button;
 static StatusIcon _status_icons[STATUS_QTY] = {
 	{ icon_inform, NULL, NULL },
 	{ icon_caution, NULL, NULL }
@@ -302,14 +301,9 @@ articles_changed_cb (gpointer call_obj, gpointer call_arg, gpointer user_data)
 		gui_set_title (agroup);
 }
 
-static void
-text_fill_body_changed_cb (gpointer call_obj,
-                           gpointer call_arg,
-                           gpointer user_data)
-{
-	menu_set_checked (_main_menu_factory, "/View/Wrap Article Body", call_arg!=NULL);
-}
-
+/**
+***  Status Log
+**/
 
 static void
 set_current_status_nolock (StatusType status)
@@ -323,10 +317,12 @@ set_current_status_nolock (StatusType status)
 }
 
 static int
-log_button_clicked_idle (gpointer data)
+set_log_button_status_idle (gpointer data)
 {
+	const int status = GPOINTER_TO_INT(data);
+
 	pan_lock ();
-	set_current_status_nolock (STATUS_OK);
+	set_current_status_nolock (status);
 	pan_unlock ();
 
 	return 0;
@@ -335,51 +331,24 @@ log_button_clicked_idle (gpointer data)
 static void
 log_button_clicked (void)
 {
-	gui_queue_add (log_button_clicked_idle, NULL);
-
+	gui_queue_add (set_log_button_status_idle, GINT_TO_POINTER(STATUS_OK));
 	dialog_log_viewer ();
 }
 
-/**
-***  Status Log
-**/
-
-static int
-log_list_appended_idle (gpointer data)
-{
-	pan_lock ();
-	set_current_status_nolock (STATUS_ERROR);
-	pan_unlock ();
-
-	return 0;
-}
-static void
-log_list_appended_cb (gpointer call_obj, gpointer call_arg, gpointer user_data)
-{
-	if (((LogEntry*)call_obj)->severity== LOG_ERROR)
-		gui_queue_add (log_list_appended_idle, NULL);
-}
-
-static int
-log_entry_added_idle (gpointer data)
-{
-	LogEntry * e = (LogEntry*) data;
-
-	if (e->severity & LOG_ERROR)
-		pan_error_dialog (e->message);
-	else /* LOG_INFO */
-		pan_info_dialog (e->message);
-
-	return 0;
-}
-
 static void
 log_entry_added_cb (gpointer entry_data, gpointer unused1, gpointer unused2)
 {
 	LogEntry* e = (LogEntry*) entry_data;
 
-	if (e && (e->severity&LOG_URGENT))
-		gui_queue_add (log_entry_added_idle, e);
+	if (e != NULL)
+	{
+		if (e->severity & (LOG_URGENT&LOG_ERROR))
+			pan_error_dialog (e->message);
+		if (e->severity & (LOG_URGENT&LOG_INFO))
+			pan_info_dialog (e->message);
+		if (e->severity & LOG_ERROR)
+			gui_queue_add (set_log_button_status_idle, GINT_TO_POINTER(STATUS_ERROR));
+	}
 }
 
 /**
@@ -389,7 +358,6 @@ log_entry_added_cb (gpointer entry_data, gpointer unused1, gpointer unused2)
 static int
 update_menus_idle (gpointer data)
 {
-	//int i;
 	const Article * article;
 	gboolean have_article;
 	gboolean have_current_article;
@@ -398,17 +366,14 @@ update_menus_idle (gpointer data)
 	gboolean have_group;
 	gboolean have_alist_folder;
 	gboolean have_glist_folder;
-	const Group * sendlater;
-	gboolean sendlater_empty;
-	gboolean posted_here;
 	gboolean is_paned = current_layout == GUI_PANED;
 	GtkItemFactory * gif = _main_menu_factory;
 	GMimeMessage * message;
+	const int visible_pane_qty = (show_group_pane?1:0) + (show_header_pane?1:0) + (show_body_pane?1:0);
 	debug_enter ("update_menus_idle");
 
 	article = articlelist_get_selected_article_nolock ();
 	have_article = article!=NULL;
-	posted_here = article!=NULL;
 
 	message = get_current_message ();
 	if ((have_current_article = message!=NULL))
@@ -420,22 +385,19 @@ update_menus_idle (gpointer data)
 	have_alist_folder = alist_group!=NULL && group_is_folder(alist_group);
 	have_glist_folder = glist_group!=NULL && group_is_folder(glist_group);
 
-	sendlater = serverlist_get_named_folder (PAN_SENDLATER);
-	sendlater_empty = FALSE;//sendlater!=NULL && !sendlater->article_qty;
-
 	pan_lock ();
 
-	/* update view menu */
-	menu_set_sensitive (gif, "/View/Show Group Pane", is_paned);
-	menu_set_sensitive (gif, "/View/Show Header Pane", is_paned);
-	menu_set_sensitive (gif, "/View/Show Body Pane", is_paned);
+	/* pane visibility.  don't let them turn off _all_ the panes; that's too confusing. */
+	menu_set_sensitive (gif, "/View/Show Group Pane", is_paned && (!show_group_pane || visible_pane_qty>1));
+	menu_set_sensitive (gif, "/View/Show Header Pane", is_paned && (!show_header_pane || visible_pane_qty>1));
+	menu_set_sensitive (gif, "/View/Show Body Pane", is_paned && (!show_body_pane || visible_pane_qty>1));
 
 	/* post menu */
 	menu_set_sensitive (gif, "/Post/Followup to Newsgroup", have_current_article);
 	menu_set_sensitive (gif, "/Post/Reply by Email", have_current_article);
 	menu_set_sensitive (gif, "/Post/Followup and Reply", have_current_article);
 	menu_set_sensitive (gif, "/Post/Forward Article by Email", have_current_article);
-	menu_set_sensitive (gif, "/Post/Send Pending Messages", !sendlater_empty);
+	menu_set_sensitive (gif, "/Post/Send Pending Messages", TRUE);
 
 	/* navigate menu */
 	menu_set_sensitive (gif, "/Go/Next Unread Article", have_group);
@@ -541,9 +503,11 @@ gui_refresh_connection_label_idle (gpointer data)
 	return 0;
 }
 
-void
-gui_set_connecting_flag (gboolean is_connecting)
+static void
+queue_connecting_state_changed_cb (gpointer call_obj, gpointer call_arg, gpointer user_data)
 {
+	const gboolean is_connecting = call_obj != NULL;
+
 	if (_is_connecting != is_connecting)
 	{
 		_is_connecting = is_connecting;
@@ -563,12 +527,14 @@ gui_set_connection_size (int size)
 	}
 }
 
-void
-gui_inc_connection_size (int inc)
+static void
+queue_connection_size_changed_cb (gpointer call_obj, gpointer call_arg, gpointer user_data)
 {
+	const int inc = GPOINTER_TO_INT (call_obj);
 	gui_set_connection_size (connection_size + inc);
 }
 
+
 static int
 gui_refresh_timer_cb (gpointer user_data)
 {
@@ -642,13 +608,19 @@ gui_set_queue_size_idle (gpointer data)
 	return 0;
 }
 
-void
+static void
 gui_set_queue_size (guint running, guint size)
 {
 	gui_queue_add (gui_set_queue_size_idle,
 	               argset_new2 (GUINT_TO_POINTER(running), GUINT_TO_POINTER(size)));
 }
 
+static void
+queue_size_changed_cb (gpointer call_obj, gpointer call_arg, gpointer user_data)
+{
+	gui_set_queue_size (GPOINTER_TO_INT(call_obj), GPOINTER_TO_INT(call_arg));
+}
+
 static int
 gui_add_status_item_idle (gpointer p)
 {
@@ -752,15 +724,21 @@ online_status_changed_cb (gpointer call_obj, gpointer call_arg, gpointer user_da
 }
 
 static void
-mute_quoted_text_changed_cb (gpointer data, gpointer call, gpointer client)
+mute_quoted_text_changed_cb (gpointer call_obj, gpointer call_arg, gpointer client)
 {
-	menu_set_checked (_main_menu_factory, "/View/Mute Quoted Text", call!=NULL);
+	menu_set_checked (_main_menu_factory, "/View/Mute Quoted Text", call_arg!=NULL);
 }
 
 static void
-show_all_headers_changed_cb (gpointer data, gpointer call, gpointer client)
+show_all_headers_changed_cb (gpointer call_obj, gpointer call_arg, gpointer client)
 {
-	menu_set_checked (_main_menu_factory, "/View/Show All Headers in Body Pane", call!=NULL);
+	menu_set_checked (_main_menu_factory, "/View/Show All Headers in Body Pane", call_arg!=NULL);
+}
+
+static void
+text_fill_body_changed_cb (gpointer call_obj, gpointer call_arg, gpointer user_data)
+{
+	menu_set_checked (_main_menu_factory, "/View/Wrap Article Body", call_arg!=NULL);
 }
 
 
@@ -976,7 +954,7 @@ gui_page_activate_nolock (int pane)
 				article_action_selected_save_as ();
 				/* RETURN pressed: save_selected_as */
 			}
-			else if (!queue_is_online() && !acache_has_message(article->message_id))
+			else if (!queue_is_online() && !acache_has_message(group_get_acache_key(article->group), article->message_id))
 			{
 				/* RETURN pressed: do nothing (offline) */
 			}
@@ -1030,7 +1008,6 @@ gui_key_press_cb (GtkWidget      * widget,
 }
 
 
-
 /****
 *****
 *****   SECTION:  SHUTDOWN
@@ -1129,11 +1106,18 @@ gui_save_window_size (GtkWidget     * widget,
 
 }
 
+static void
+get_accel_filename (char * buf, int buf_max)
+{
+	g_snprintf (buf, buf_max, "%s%caccels.txt", get_data_dir(), G_DIR_SEPARATOR);
+}
+
 static void
 gui_shutdown (void)
 {
 	int i;
 	char * pch;
+	char fname[PATH_MAX];
 	GtkWidget * w[3];
 	debug_enter ("gui_shutdown");
 
@@ -1141,36 +1125,27 @@ gui_shutdown (void)
 	***  Stop updating the UI
 	**/
 
-	pan_callback_remove (status_item_get_active_callback(),
-	                     status_item_active_changed_cb, NULL);
+	pan_callback_remove (status_item_get_active_callback(), status_item_active_changed_cb, NULL);
 	turn_off_status_item_views_nolock ();
 
 	/**
 	***  Stop updating the rest of the UI
 	**/
 
-	pan_callback_remove (article_get_articles_changed_callback(),
-	                     articles_changed_cb, NULL);
-	pan_callback_remove (log_get_entry_added_callback(),
-	                     log_entry_added_cb, NULL);
-	pan_callback_remove (log_get_entry_added_callback(),
-	                     log_list_appended_cb, NULL);
-	pan_callback_remove (text_get_fill_body_changed_callback(),
-	                     text_fill_body_changed_cb, NULL);
-	pan_callback_remove (text_get_show_all_headers_changed_callback(),
-	                     show_all_headers_changed_cb, NULL);
-	pan_callback_remove (text_get_mute_quoted_changed_callback(),
-	                     mute_quoted_text_changed_cb, NULL);
-	pan_callback_remove (current_article_changed,
-	                     update_menus_cb, NULL);
-	pan_callback_remove (grouplist_group_selection_changed,
-	                     update_menus_cb, NULL);
-	pan_callback_remove (grouplist_server_set_callback,
-	                     update_menus_cb, NULL);
-	pan_callback_remove (articlelist_get_group_changed_callback(),
-	                     update_menus_cb, NULL);
-	pan_callback_remove (articlelist_selection_changed,
-	                     update_menus_cb, NULL);
+	pan_callback_remove (article_get_articles_changed_callback(), articles_changed_cb, NULL);
+	pan_callback_remove (articlelist_get_group_changed_callback(), update_menus_cb, NULL);
+	pan_callback_remove (articlelist_get_selection_changed_callback(), update_menus_cb, NULL);
+	pan_callback_remove (current_article_changed, update_menus_cb, NULL);
+	pan_callback_remove (grouplist_group_selection_changed, update_menus_cb, NULL);
+	pan_callback_remove (serverlist_get_server_activated_callback(), update_menus_cb, NULL);
+	pan_callback_remove (log_get_entry_added_callback(), log_entry_added_cb, NULL);
+	pan_callback_remove (queue_get_size_changed_callback(), queue_size_changed_cb, NULL);
+	pan_callback_remove (queue_get_connection_size_changed_callback(), queue_connection_size_changed_cb, NULL);
+	pan_callback_remove (queue_get_connecting_state_changed_callback(), queue_connecting_state_changed_cb, NULL);
+	pan_callback_remove (text_get_fill_body_changed_callback(), text_fill_body_changed_cb, NULL);
+	pan_callback_remove (text_get_mute_quoted_changed_callback(), mute_quoted_text_changed_cb, NULL);
+	pan_callback_remove (text_get_show_all_headers_changed_callback(), show_all_headers_changed_cb, NULL);
+	pan_callback_remove (queue_get_online_status_changed_callback(), online_status_changed_cb, NULL);
 
 	g_source_remove (gui_refresh_timer_id);
 	gui_refresh_timer_id = 0;
@@ -1243,6 +1218,13 @@ gui_shutdown (void)
 	gui_save_window_size (Pan.window, "main_window");
 	pan_config_sync();
 
+	/**
+	*** Save accelerators
+	**/
+
+	get_accel_filename (fname, sizeof(fname));
+	gtk_accel_map_save (fname);
+
 	/**
 	***  Clear out the UI pieces
 	**/
@@ -1284,7 +1266,7 @@ gui_restore_column_widths (GtkWidget     * clist,
 		int i;
 		GtkCList * list = GTK_CLIST (clist);
 		const int cols = list->columns;
-		int * widths = g_new (int, cols);
+		int * widths = g_newa (int, cols);
 		char buf[1024];
 		debug_enter ("gui_restore_column_widths");
 
@@ -1302,7 +1284,6 @@ gui_restore_column_widths (GtkWidget     * clist,
 		pan_unlock();
 
 		/* cleanup */
-		g_free (widths);
 		debug_exit ("gui_restore_column_widths");
 	}
 }
@@ -1321,7 +1302,7 @@ gui_restore_column_widths_tree_view (GtkWidget     * tree_view,
 	
 	columns = gtk_tree_view_get_columns(GTK_TREE_VIEW(tree_view));
 	length = g_list_length(columns);
-	widths = g_new (int, length);
+	widths = g_newa (int, length);
 	
 	/* get width from config... */
 	for (i = 0; i != length; ++i) {
@@ -1339,7 +1320,6 @@ gui_restore_column_widths_tree_view (GtkWidget     * tree_view,
 	pan_unlock();
 	
 	/* cleanup */
-	g_free (widths);
 	g_list_free(columns);
 	
 	debug_exit ("gui_restore_column_widths_tree_view");
@@ -1452,7 +1432,7 @@ gui_create_appbar (GtkWidget* window)
 	}
 
 	/* status button */
-	_status_button = w = gtk_button_new ();
+	w = gtk_button_new ();
 	_status_image = gtk_image_new ();
 	gtk_container_add (GTK_CONTAINER(w), _status_image);
 	set_current_status_nolock (STATUS_OK);
@@ -1498,7 +1478,7 @@ menu_translate(const char* path, gpointer data)
 }
 
 
-GtkWidget*
+static GtkWidget*
 create_image (const guint8 * inline_txt)
 {
 	GdkPixbuf * pixbuf;
@@ -1636,10 +1616,6 @@ toolbar_cb (GtkWidget * button, gpointer user_data)
 			group_action_selected_download_new_and_bodies ();
 			break;
 
-		/*case ACTION_GET_BODIES_FOR_FLAGGED:
-			flagset_flush ();
-			break;*/
-
 		case ACTION_READ_MORE:
 			text_read_more ();
 			break;
@@ -1698,17 +1674,20 @@ create_toolbar (void)
 	GtkWidget * toolbar_w;
 	GtkToolbar * toolbar;
 	char buf[512];
+	const gboolean show_labels = pan_config_get_bool ("/Pan/Display/toolbar_labels_visible=TRUE");
 
 	toolbar_w = gtk_toolbar_new ();
 	toolbar = GTK_TOOLBAR(toolbar_w);
 	gtk_toolbar_set_icon_size (toolbar, GTK_ICON_SIZE_SMALL_TOOLBAR);
 	gtk_toolbar_set_style (toolbar, GTK_TOOLBAR_ICONS);
 
-	g_snprintf (buf, sizeof(buf), "%s:", _("Post"));
-	w = gtk_label_new (buf);
-	gtk_misc_set_alignment (GTK_MISC(w), 0.5f, 0.5f);
-	gtk_misc_set_padding (GTK_MISC(w), GUI_PAD, 0);
-	gtk_toolbar_append_widget (toolbar, w, NULL, NULL);
+	if (show_labels) {
+		g_snprintf (buf, sizeof(buf), "%s:", _("Post"));
+		w = gtk_label_new (buf);
+		gtk_misc_set_alignment (GTK_MISC(w), 0.5f, 0.5f);
+		gtk_misc_set_padding (GTK_MISC(w), GUI_PAD, 0);
+		gtk_toolbar_append_widget (toolbar, w, NULL, NULL);
+	}
 	_toolbar_buttons[ACTION_COMPOSE_NEW] = gtk_toolbar_append_item (
 		toolbar, "Post", _("Post to Newsgroup"), NULL,
 		create_image (icon_compose_post),
@@ -1719,11 +1698,13 @@ create_toolbar (void)
 		G_CALLBACK(toolbar_cb), GINT_TO_POINTER(ACTION_COMPOSE_FOLLOWUP));
 	gtk_toolbar_append_space (toolbar);
 
-	g_snprintf (buf, sizeof(buf), "%s:", _("Get"));
-	w = gtk_label_new (buf);
-	gtk_misc_set_alignment (GTK_MISC(w), 0.5f, 0.5f);
-	gtk_misc_set_padding (GTK_MISC(w), GUI_PAD, 0);
-	gtk_toolbar_append_widget (toolbar, w, NULL, NULL);
+	if (show_labels) {
+		g_snprintf (buf, sizeof(buf), "%s:", _("Get"));
+		w = gtk_label_new (buf);
+		gtk_misc_set_alignment (GTK_MISC(w), 0.5f, 0.5f);
+		gtk_misc_set_padding (GTK_MISC(w), GUI_PAD, 0);
+		gtk_toolbar_append_widget (toolbar, w, NULL, NULL);
+	}
 	_toolbar_buttons[ACTION_GET_NEW_HEADERS_FOR_SUBSCRIBED] = gtk_toolbar_append_item (
 		toolbar, NULL, _("Get New Headers in Subscribed Groups"), NULL,
 		create_image (icon_get_subscribed),
@@ -1736,17 +1717,15 @@ create_toolbar (void)
 		toolbar, NULL, _("Get Selected Articles"), NULL,
 		create_image (icon_get_selected),
 		G_CALLBACK(toolbar_cb), GINT_TO_POINTER(ACTION_GET_BODIES_FOR_SELECTED));
-/*	_toolbar_buttons[ACTION_GET_BODIES_FOR_FLAGGED] = gtk_toolbar_append_item (
-		toolbar, NULL, _("Get Flagged Articles"), NULL,
-		create_image (icon_get_flagged),
-		G_CALLBACK(toolbar_cb), GINT_TO_POINTER(ACTION_GET_BODIES_FOR_FLAGGED)); */
 	gtk_toolbar_append_space (toolbar);
 
-	g_snprintf (buf, sizeof(buf), "%s:", _("Reading"));
-	w = gtk_label_new (buf);
-	gtk_misc_set_alignment (GTK_MISC(w), 0.5f, 0.5f);
-	gtk_misc_set_padding (GTK_MISC(w), GUI_PAD, 0);
-	gtk_toolbar_append_widget (toolbar, w, NULL, NULL);
+	if (show_labels) {
+		g_snprintf (buf, sizeof(buf), "%s:", _("Reading"));
+		w = gtk_label_new (buf);
+		gtk_misc_set_alignment (GTK_MISC(w), 0.5f, 0.5f);
+		gtk_misc_set_padding (GTK_MISC(w), GUI_PAD, 0);
+		gtk_toolbar_append_widget (toolbar, w, NULL, NULL);
+	}
 	_toolbar_buttons[ACTION_READ_MORE] = gtk_toolbar_append_item (
 		toolbar, NULL, _("Read More"), NULL,
 		create_image (icon_read_more),
@@ -1771,11 +1750,13 @@ create_toolbar (void)
 		G_CALLBACK(toolbar_cb), GINT_TO_POINTER(ACTION_SAVE_AS), -1);
 	gtk_toolbar_append_space (toolbar);
 
-	g_snprintf (buf, sizeof(buf), "%s:", _("Filters"));
-	w = gtk_label_new (buf);
-	gtk_misc_set_alignment (GTK_MISC(w), 0.5f, 0.5f);
-	gtk_misc_set_padding (GTK_MISC(w), GUI_PAD, 0);
-	gtk_toolbar_append_widget (toolbar, w, NULL, NULL);
+	if (show_labels) {
+		g_snprintf (buf, sizeof(buf), "%s:", _("Filters"));
+		w = gtk_label_new (buf);
+		gtk_misc_set_alignment (GTK_MISC(w), 0.5f, 0.5f);
+		gtk_misc_set_padding (GTK_MISC(w), GUI_PAD, 0);
+		gtk_toolbar_append_widget (toolbar, w, NULL, NULL);
+	}
 	_toolbar_buttons[ACTION_SHOW_ONLY_NEW] = gtk_toolbar_append_element (
 		toolbar, GTK_TOOLBAR_CHILD_TOGGLEBUTTON, NULL, _("New"), _("Show Only New Articles"), NULL,
 		create_image (icon_filter_only_new), NULL, NULL);
@@ -1793,11 +1774,13 @@ create_toolbar (void)
 		create_image (icon_filter_only_watched), NULL, NULL);
 	gtk_toolbar_append_space (toolbar);
 
-	g_snprintf (buf, sizeof(buf), "%s:", _("Net"));
-	w = gtk_label_new (buf);
-	gtk_misc_set_alignment (GTK_MISC(w), 0.5f, 0.5f);
-	gtk_misc_set_padding (GTK_MISC(w), GUI_PAD, 0);
-	gtk_toolbar_append_widget (toolbar, w, NULL, NULL);
+	if (show_labels) {
+		g_snprintf (buf, sizeof(buf), "%s:", _("Net"));
+		w = gtk_label_new (buf);
+		gtk_misc_set_alignment (GTK_MISC(w), 0.5f, 0.5f);
+		gtk_misc_set_padding (GTK_MISC(w), GUI_PAD, 0);
+		gtk_toolbar_append_widget (toolbar, w, NULL, NULL);
+	}
 	_toolbar_buttons[ACTION_NET_ONLINE] = gui_add_network_button_to_toolbar (toolbar_w);
 	_toolbar_buttons[ACTION_NET_CANCEL_LAST_TASK] = gtk_toolbar_append_item (
 		toolbar, "Cancel", _("Cancel Last Task"), NULL,
@@ -1813,6 +1796,7 @@ gui_construct (const char * geometry)
 	GtkWidget * w;
 	GtkWidget * vbox;
 	GtkWidget * menubar;
+	char fname[PATH_MAX];
 	debug_enter ("gui_construct");
 
 	ttips = gtk_tooltips_new();
@@ -1857,9 +1841,14 @@ gui_construct (const char * geometry)
 	menubar = gtk_item_factory_get_widget (_main_menu_factory, "<main>");
 	gtk_box_pack_start (GTK_BOX(vbox), menubar, FALSE, FALSE, 0);
 
+	/* load accelerators */
+	get_accel_filename (fname, sizeof(fname));
+	gtk_accel_map_load (fname);
+
 	/* create the toolbar */
 	w = create_toolbar ();
-	gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
+	if (pan_config_get_bool ("/Pan/Display/toolbar_visible=TRUE"))
+		gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
 
 	/* create the workarea */
 	Pan.workarea = gtk_vbox_new (TRUE, 0);
@@ -1900,17 +1889,19 @@ gui_construct (const char * geometry)
 	gui_layout_set_mode (pan_config_get_int ("/Pan/State/viewmode=1"));
 
 	pan_callback_add (article_get_articles_changed_callback(), articles_changed_cb, NULL);
-	pan_callback_add (log_get_entry_added_callback(), log_list_appended_cb, NULL);
+	pan_callback_add (articlelist_get_group_changed_callback(), update_menus_cb, NULL);
+	pan_callback_add (articlelist_get_selection_changed_callback(), update_menus_cb, NULL );
+	pan_callback_add (current_article_changed, update_menus_cb, NULL);
+	pan_callback_add (grouplist_group_selection_changed, update_menus_cb, NULL);
+	pan_callback_add (serverlist_get_server_activated_callback(), update_menus_cb, NULL);
+	pan_callback_add (log_get_entry_added_callback(), log_entry_added_cb, NULL);
+	pan_callback_add (queue_get_size_changed_callback(), queue_size_changed_cb, NULL);
+	pan_callback_add (queue_get_connection_size_changed_callback(), queue_connection_size_changed_cb, NULL);
+	pan_callback_add (queue_get_connecting_state_changed_callback(), queue_connecting_state_changed_cb, NULL);
 	pan_callback_add (status_item_get_active_callback(), status_item_active_changed_cb, NULL);
 	pan_callback_add (text_get_fill_body_changed_callback(), text_fill_body_changed_cb, NULL);
 	pan_callback_add (text_get_show_all_headers_changed_callback(), show_all_headers_changed_cb, NULL);
 	pan_callback_add (text_get_mute_quoted_changed_callback(), mute_quoted_text_changed_cb, NULL);
-	pan_callback_add (current_article_changed, update_menus_cb, NULL);
-	pan_callback_add (grouplist_group_selection_changed, update_menus_cb, NULL);
-	pan_callback_add (grouplist_server_set_callback, update_menus_cb, NULL);
-	pan_callback_add (articlelist_get_group_changed_callback(), update_menus_cb, NULL);
-	pan_callback_add (articlelist_selection_changed, update_menus_cb, NULL );
-	pan_callback_add (log_get_entry_added_callback(), log_entry_added_cb, NULL);
 	pan_callback_add (queue_get_online_status_changed_callback(), online_status_changed_cb, NULL);
 
 	menu_set_checked (_main_menu_factory, "/File/Work Online", queue_is_online());
@@ -2115,14 +2106,12 @@ toggle_pane_visibility (gpointer user_data, int action, GtkWidget * w)
 }
 
 static void
-zoom_page_cb (gpointer menu, gpointer data)
+zoom_page_cb (gpointer menu, gpointer page)
 {
-	int page;
 	debug_enter ("zoom_page_cb");
        
-	page = GPOINTER_TO_INT(data);
 	gui_layout_set_mode (GUI_NOTEBOOK);
-	gui_page_set (page);
+	gui_page_set (GPOINTER_TO_INT(page));
 
 	debug_exit ("zoom_page_cb");
 }
@@ -2144,8 +2133,6 @@ zoom_cb (gpointer user_data, int action, GtkWidget * w)
 	debug_exit ("zoom_cb");
 }
 
-static int gui_refresh_connection_label_idle (gpointer data);
-
 static void
 delete_last_task_cb (gpointer user_data, int action, GtkWidget * w)
 {
@@ -2247,12 +2234,6 @@ rules_cb (void)
 	gtk_widget_show_all (rule_dialog);
 }
 
-static void
-gui_read_next (void)
-{
-	articlelist_read_next ();
-}
-
 /****
 *****
 *****   SECTION:  MENU
@@ -2347,7 +2328,7 @@ static GtkItemFactoryEntry main_menu_entries[MAIN_MENU_ENTRIES_QTY] =
 	/* s */ {N_("/_Go/Read Back"), "BackSpace", menu_cb, ACTION_READ_LESS, "<ImageItem>", icon_read_less},
 	/*   */ {N_("/_Go/---"), NULL, NULL, 0, "<Separator>"},
 	/* n */ {N_("/_Go/_Next Unread Article"), "N", menu_cb, ACTION_READ_NEXT_UNREAD_ARTICLE, "<ImageItem>", icon_read_new_article},
-	/* x */ {N_("/_Go/Ne_xt Article"), "<control>N", gui_read_next, 0, NULL},
+	/* x */ {N_("/_Go/Ne_xt Article"), "<control>N", articlelist_read_next, 0, NULL},
 	/* t */ {N_("/_Go/Next Unread _Thread"), "T", menu_cb, ACTION_READ_NEXT_UNREAD_THREAD, "<ImageItem>", icon_read_new_thread},
 	/* d */ {N_("/_Go/Next Threa_d"), "<shift>T", articlelist_read_next_thread, 0, NULL},
 	/*   */ {N_("/_Go/---"), NULL, NULL, 0, "<Separator>"},
