@@ -142,7 +142,7 @@ create_identity_tab (IdentityEditDialog *d)
 		gtk_widget_set_usize (w, 12u, 0u);
 		gtk_table_attach_defaults (GTK_TABLE(t), w, 2, 3, row, row+3);
 
-		l = gtk_label_new_with_mnemonic (_("_Profile Name:"));
+		l = gtk_label_new_with_mnemonic (_("_Profile name:"));
 		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
 		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
 
