@@ -17,9 +17,9 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <glib.h>
+#include <config.h>
 
-#include <pan/prefs.h>
+#include <glib.h>
 
 #include <pan/base/base-prefs.h>
 #include <pan/base/debug.h>
