@@ -199,13 +199,12 @@ text_inserted_cb (GtkTextBuffer   * text_buffer,
 		&& experimental_toggle_active
 		&& wrap_column<gtk_text_iter_get_line_offset(insert_pos))
 	{
-		int text_len;
 		int pos;
 		int del_start_pos;
 		int del_end_pos;
 		char * text;
 		char * pch;
-		char * del_start_pch;
+		char * del_start_pch = NULL;
 		GtkTextMark * mark;
 		GtkTextIter line_pos;
 		GtkTextIter del_start_iter;
@@ -217,10 +216,9 @@ text_inserted_cb (GtkTextBuffer   * text_buffer,
 
 		/* get the contents of the line */
 		text = gtk_text_buffer_get_text (text_buffer, &line_pos, insert_pos, FALSE);
-		text_len = text ? strlen(text) : 0;
 
 		/* try to find the last space before the wrap column */
-		del_start_pos=pos=0;
+		del_start_pos = del_end_pos = pos = 0;
 		for (pch=text; *pch; pch=g_utf8_next_char (pch), ++pos) {
 			gunichar ch;
 			if (pos > wrap_column)
@@ -237,7 +235,7 @@ text_inserted_cb (GtkTextBuffer   * text_buffer,
 			return;
 
 		/* find the end of the whitespace */
-		for (pch=del_start_pch; *pch; pch=g_utf8_next_char (pch), ++del_end_pos) {
+		for (pch=del_start_pch; is_nonempty_string(pch); pch=g_utf8_next_char(pch), ++del_end_pos) {
 			gunichar ch = g_utf8_get_char (pch);
 			if (!g_unichar_isspace(ch))
 				break;
@@ -296,7 +294,7 @@ static GtkItemFactoryEntry post_menu_entries [] =
 	{N_("/_File/Send Now"), "<control>Return", send_now_cb, 0, "<ImageItem>", icon_compose_send},
 	{N_("/_File/Send Later"), NULL, send_later_cb, 0, "<ImageItem>", icon_stock_timer},
 	{N_("/_File/---"), NULL, NULL, 0, "<Separator>"},
-	{N_("/_File/Save As..."), "<control>S", message_window_save_cb, 0, "<StockItem>", GTK_STOCK_SAVE_AS},
+	{N_("/_File/Save _As..."), "<control>S", message_window_save_cb, 0, "<StockItem>", GTK_STOCK_SAVE_AS},
 	{N_("/_File/---"), NULL, NULL, 0, "<Separator>"},
 	{N_("/_File/_Close"), NULL, message_window_close, 0, "<StockItem>", GTK_STOCK_CLOSE},
 	{N_("/_Edit"), NULL, NULL, 0, "<Branch>"},
@@ -618,6 +616,7 @@ update_title_with_subject (GtkWidget * widget, Compose * compose)
 ****
 ***/
 
+#if 0
 typedef enum
 {
 	ENCODING_BASE64,
@@ -626,7 +625,6 @@ typedef enum
 }
 PanEncoding;
 
-#if 0
 static PanEncoding
 get_encoding_type (Compose * compose)
 {
@@ -643,7 +641,6 @@ get_encoding_type (Compose * compose)
 
 	return retval;
 }
-#endif
 
 static gulong
 get_estimated_encoded_size (gulong raw_size, PanEncoding encoding)
@@ -763,9 +760,6 @@ static void
 attachment_add_button_clicked_cb (GtkButton * button, gpointer user_data)
 {
 	GtkWidget * w = gtk_file_selection_new(_("Select the file to attach."));
-#if 0
-	gtk_file_selection_set_show_dotfiles (GTK_FILE_SELECTION(w), TRUE);
-#endif
 	g_signal_connect (w, "response", G_CALLBACK(attachments_filesel_response_cb), user_data);
     	gtk_widget_show (w);
 }
@@ -842,7 +836,6 @@ attachment_page (Compose *mw)
 	g_signal_connect (w, "unselect_row",
 	                  G_CALLBACK(attachment_clist_selection_changed_cb), mw);
         w = gtk_scrolled_window_new (NULL, NULL);
-	//gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(w), GTK_SHADOW_IN);
         gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(w),
                 GTK_POLICY_AUTOMATIC,
                 GTK_POLICY_AUTOMATIC);
@@ -915,6 +908,7 @@ attachment_page (Compose *mw)
 	refresh_attachment_page (mw);
 	return top;
 }
+#endif
 
 /***
 ****
@@ -922,22 +916,38 @@ attachment_page (Compose *mw)
 ****
 ***/
 
+static void
+get_charset_partfunc(GMimeObject * obj, gpointer charset_gpointer)
+{
+	GMimePart * part;
+	const GMimeContentType * type;
+	char ** charset;
+
+	if (!GMIME_IS_PART (obj))
+		return;
+
+	part = GMIME_PART (obj);
+	type = g_mime_object_get_content_type (GMIME_OBJECT (part));
+	charset = (char **) charset_gpointer;
+
+	if (g_mime_content_type_is_type (type, "text", "*")) {
+		*charset = (char *) g_mime_object_get_content_type_parameter (GMIME_OBJECT (part), "charset");
+
+		/*odebug1 ("found charset in article: %s", *charset);*/
+	}
+}
+
 static const char *
 determine_charset_for_message (const Compose *mw)
 {
 	const char * charset = NULL; 
-   
-#warning need to first look at the article charset	
-#if 0	
+  
+	/* does the article specify a charset ? */
 	if (mw && mw->message)
-		charset = g_mime_message_get_header (mw->message, PAN_CHARSET);
-#endif
+		g_mime_message_foreach_part (mw->message, get_charset_partfunc, &charset);
 
-	if (charset == NULL) {
-		/*
-		 * FIXME: quick merge from pan-0-12-fix to HEAD. 
-		 * Better approach ?
-		 */
+	/* fallback: use the group's charset */
+	if (charset == NULL || !pan_charset_picker_has (charset)) {
 		const Group * group = articlelist_get_group ();
 
 		charset = group ? group_get_default_charset (group) :
@@ -1410,6 +1420,9 @@ append_signature (GtkTextBuffer * body_buffer, const char *sig_file)
 		/* expand the home directory character */
 		if (*pch=='~')
 			replace_gstr (&pch, g_strconcat (g_get_home_dir(), pch+1, NULL));
+		/* if not an absolute path, assume in home directory */
+		if (*pch!=G_DIR_SEPARATOR)
+			replace_gstr (&pch, g_strdup_printf ("%s%c%s", g_get_home_dir(), G_DIR_SEPARATOR, pch));
 
 		if (!g_shell_parse_argv (pch, &argc, &argv, &err))
 		{
@@ -1466,14 +1479,10 @@ append_signature (GtkTextBuffer * body_buffer, const char *sig_file)
 	/* we assume the contents is in the user's locale's charset.        */
 	if (sig!=NULL)
 	{
-		gchar * freeme;
-
+		char * freeme = NULL;
 		pan_utf8ize (sig, -1, &freeme);
 		if (freeme != NULL)
-		{
-			g_free (sig);
-			sig = freeme;
-		}
+			replace_gstr (&sig, freeme);
 	}
 	 
 	/* append the signature */
@@ -2087,7 +2096,6 @@ create_body_pane_nolock (Compose * mw)
 {
 	GtkTextBuffer * buf;
 	GtkWidget * scrolled_window;
-	GtkStyle * style;
 
 	g_return_val_if_fail (mw!=NULL, NULL);
 
@@ -2103,13 +2111,7 @@ create_body_pane_nolock (Compose * mw)
 	gtk_text_buffer_create_tag (buf, "quote_2", "foreground_gdk", NULL, NULL);
 	gtk_text_buffer_create_tag (buf, "quote_3", "foreground_gdk", NULL, NULL);
 
-	/* make the compose window look like the text window, except force a monospace font */
-	style = gtk_style_copy (gtk_widget_get_style (Pan.text));
-	pango_font_description_free (style->font_desc);
-	style->font_desc = pango_font_description_from_string (message_body_font_fixed);
-	gtk_widget_set_style (GTK_WIDGET(mw->body_view), style);
-	g_object_unref (G_OBJECT(style));
-
+	pan_widget_set_font (mw->body_view, message_body_font_fixed);
 	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW(mw->body_view), GTK_WRAP_NONE);
 	gtk_text_view_set_editable (GTK_TEXT_VIEW(mw->body_view), TRUE);
 	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
@@ -2155,25 +2157,9 @@ message_post_window_create (Compose * compose)
 	pan_unlock();
 }
 
-static const char *
-determine_charset_for_text (const char * text)
-{
-	size_t       len;
-	const char * default_charset = get_charset_from_locale ();
-	const char * charset = NULL;
-	
-	g_return_val_if_fail (text!=NULL, default_charset);
-
-	len = strlen (text);
-	charset = g_mime_charset_best (text, len);
-
-	return charset ? charset : default_charset;
-}
-
 static void
 make_reply_window (ComposeType type)
 {
-	const char   * charset;
 	char         * body;
 	GMimeMessage * message;
 
@@ -2191,8 +2177,6 @@ make_reply_window (ComposeType type)
 	/* open up a populated reply window. */
 	body = text_get_message_to_reply_to ();
 	g_object_set_data_full (G_OBJECT(message), PAN_REPLY_PORTION, g_strdup(body), g_free);
-	charset = determine_charset_for_text (body);
-	g_mime_message_set_header (message, PAN_CHARSET, body);
 	message_window_new (message, type);
 	g_free (body);
 }
@@ -2611,6 +2595,45 @@ populate_message_from_mw (GMimeMessage* message, const Compose * mw)
 	pan_object_unref(PAN_OBJECT(id));
 }
 
+static gboolean
+check_charset_for_body (Compose * mw)
+{
+	GtkTextIter start, end;
+	const char * utf8_body = NULL;
+	const char * charset = NULL;
+	char       * body = NULL;
+	gboolean     ok = TRUE;
+
+
+	/* get settings */
+	gtk_text_buffer_get_bounds (mw->body_buffer, &start, &end);
+	utf8_body = gtk_text_buffer_get_text (mw->body_buffer, &start, &end, FALSE);
+	charset = pan_charset_picker_get_charset (mw->charset_om);
+
+	/* attempt to convert */
+	body = g_convert (utf8_body, -1, charset, "UTF-8", NULL, NULL, NULL);
+
+	/* succeeded? */
+	if (body == NULL)
+	{
+		const char * best_charset;
+	      
+	       	ok = FALSE;	
+		best_charset = g_mime_charset_best (utf8_body, g_utf8_strlen (utf8_body, -1));
+		/*odebug1 ("best charset: %s", best_charset);*/
+		if (best_charset == NULL)
+			best_charset = "UTF-8";
+
+		pan_error_dialog (_("ERROR: message uses characters not specified in charset '%s' - possibly use '%s' instead?"), charset, best_charset);
+
+	}
+
+	/* cleanup */
+	g_free (body);
+
+	return ok;
+}
+
 static void
 save_cb (gpointer user_data, int action, GtkWidget * w)
 {
@@ -2621,6 +2644,10 @@ save_cb (gpointer user_data, int action, GtkWidget * w)
 	/* sanity clause */
 	g_return_if_fail (mw!=NULL);
 
+	/* validate the charset */
+	if (!check_charset_for_body (mw))
+		return;
+
 	/* update contents of mw->message. */
 	old_message = mw->message;
 	new_message = g_mime_message_new (FALSE);
@@ -2698,6 +2725,10 @@ post (Compose * mw, SendMode mode)
 
 	debug_enter ("post");
 
+	/* Validate the charset */
+	if (!check_charset_for_body (mw))
+		return;
+
 	/* Get the article pointer, either through repopulating the
 	 * current article (if we're editing an existing message) or
 	 * by creating a new article (if this is a new message).
@@ -2795,7 +2826,11 @@ message_window_save_cb (gpointer user_data, int action, GtkWidget * w)
 {
 	Compose * compose = (Compose*) user_data;
 	GtkWidget *file_entry = NULL;
-	
+
+	/* validate the charset first */
+	if (!check_charset_for_body (compose))
+		return;
+
 	pan_lock();
 	
 	file_entry = gtk_file_selection_new (_("Save message to file"));
@@ -2841,7 +2876,7 @@ warn_wrong_charset (gpointer p)
 static void*
 run_external_editor (void * compose_voidp)
 {
-	FILE * fp;
+	FILE * fp = NULL;
 	char * fname = NULL;
 	char ** argv = NULL;
 	gboolean ok = TRUE;
@@ -2910,8 +2945,10 @@ run_external_editor (void * compose_voidp)
 		}
 	}
 
-	fclose (fp);
-	fp = NULL;
+	if (fp != NULL) {
+		fclose (fp);
+		fp = NULL;
+	}
 	g_free (chars);
 
 	/* parse the command line */
@@ -2949,7 +2986,7 @@ run_external_editor (void * compose_voidp)
 
 		g_file_get_contents (fname, &body, &body_len, &err);
 		if (err != NULL) {
-			log_add_va (LOG_ERROR, _("Error reading edited file: %s"), err->message);
+			log_add_va (LOG_ERROR, _("Error reading file \"%s\": %s"), err->message, g_strerror(errno));
 			g_error_free (err);
 			ok = FALSE;
 		}
