@@ -33,6 +33,7 @@
 #include <pan/base/log.h>
 #include <pan/base/pan-glib-extensions.h>
 #include <pan/base/pan-i18n.h>
+#include <pan/base/serverlist.h>
 #include <pan/base/util-mime.h>
 
 #include <pan/nntp.h>
@@ -599,14 +600,29 @@ nntp_article_download (StatusItem         * status,
 
 	/* request the article from the server */
 	val = nntp_command_va (status, sock, &response, "ARTICLE %lu", source->number);
-	if (val != 220) {
-		char * pch = g_strdup_printf (_("Getting article \"%s\" body failed: %s"), mid->message_id, response);
+	if (val != 220)
+	{
+		/* log the error */
+		char * pch = g_strdup_printf (_("Getting article \"%s\" body failed: %s"), mid->readable_name, response);
 		status_item_emit_error (status, pch);
 		g_free (pch);
+
+		/* if -1, just fail... we might be able to retry later */
 		if (val==-1)
 			return TASK_FAIL;
-		/*FIXME articles_add_flag (&article, 1, STATE_ERROR);*/
-		return TASK_FAIL_HOPELESS;
+
+		/* if it can't be done, try to mark the article as `error' and return hopeless */
+		else {
+			Server * server = serverlist_get_named_server (source->server_name);
+			Group * group = server_get_named_group (server, source->group_name);
+			if (group_ref_articles_if_loaded (group)) {
+				Article * a = group_get_article_by_message_id (group, mid->message_id);
+				if (a != NULL)
+					articles_add_flag (&a, 1, STATE_ERROR);
+				group_unref_articles (group, NULL);
+			}
+			return TASK_FAIL_HOPELESS;
+		}
 	}
 
 	/* try to read the article... */
@@ -818,6 +834,7 @@ nntp_download_headers (StatusItem       * status,
 		/* get subject */
 		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
 			((char*)tok)[tok_len] = '\0';
+			g_strstrip ((char*)tok);
 			cvt = pan_header_to_utf8 (tok, tok_len, charset);
 			article_init_header (a, HEADER_SUBJECT, cvt, DO_CHUNK_SHARE);
 			g_free (cvt);
@@ -826,12 +843,14 @@ nntp_download_headers (StatusItem       * status,
 		/* get author */
 		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
 			((char*)tok)[tok_len] = '\0';
+			g_strstrip ((char*)tok);
 			article_init_author_from_header (a, tok);
 		}
 
 		/* get date */
 		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
 			((char*)tok)[tok_len] = '\0';
+			g_strstrip ((char*)tok);
 			a->date = is_nonempty_string(tok)
 				? g_mime_utils_header_decode_date (tok, NULL)
 				: (time_t)0;
@@ -840,12 +859,14 @@ nntp_download_headers (StatusItem       * status,
 		/* get message id */
 		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
 			((char*)tok)[tok_len] = '\0';
+			g_strstrip ((char*)tok);
 			if (is_nonempty_string(tok) && *tok=='<')
 				article_init_header (a, HEADER_MESSAGE_ID, tok, DO_CHUNK);
 		}
 
 		/* get references */
 		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
+			g_strstrip ((char*)tok);
 			if (tok_len>0 ) {
 				((gchar*)tok)[tok_len] = '\0';
 				if (is_nonempty_string(tok) && *tok=='<')
@@ -862,6 +883,7 @@ nntp_download_headers (StatusItem       * status,
 		/* get crossref */
 		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
 			((char*)tok)[tok_len] = '\0';
+			g_strstrip ((char*)tok);
 			if (tok_len>6 && !strncmp(tok,"Xref: ", 6)) {
 				const char * header = tok + 6;
 				const char * pch;
@@ -982,7 +1004,7 @@ nntp_download_bodies (StatusItem                * status,
                       int                       * index,
                       gboolean                    abort_if_one_fails)
 {
-	int result;
+	int result = TASK_SUCCESS;
 	int i;
 	debug_enter ("nntp_download_bodies");
 
@@ -997,7 +1019,7 @@ nntp_download_bodies (StatusItem                * status,
 
 	/* find the first message we don't have cached */
 	for (i=*index; i<mid_qty; ++i)
-		if (!acache_has_message (mids[i]->message_id))
+		if (!acache_has_message (ACACHE_DEFAULT_KEY, mids[i]->message_id))
 			break;
 
 	/* are we done? */
@@ -1033,13 +1055,13 @@ nntp_download_bodies (StatusItem                * status,
 
 		if (mid_qty == 1)
 			status_item_emit_status_va (status, _("Getting \"%s\""),
-							      mid->message_id);
+							      mid->readable_name);
 		else
 			status_item_emit_status_va (status, _("Getting %d of %d"),
 							      1+*index, mid_qty);
 
 		/* download if we have to. */
-		if (acache_has_message (mid->message_id))
+		if (acache_has_message (ACACHE_DEFAULT_KEY, mid->message_id))
 		{
 			result = TASK_SUCCESS;
 			status_item_emit_inc_step (status, mid->line_qty);
@@ -1057,7 +1079,7 @@ nntp_download_bodies (StatusItem                * status,
 		}
 	}
 
-	return TASK_SUCCESS;
+	return result;
 }
 
 
@@ -1077,12 +1099,12 @@ nntp_cancel (StatusItem     * status,
 	g_return_val_if_fail (sock != NULL, TASK_FAIL);
 	g_return_val_if_fail (status != NULL, TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (is_nonempty_string(message_id), TASK_FAIL_HOPELESS);
-	g_return_val_if_fail (acache_has_message(message_id), TASK_FAIL_HOPELESS);
+	g_return_val_if_fail (acache_has_message(ACACHE_DEFAULT_KEY, message_id), TASK_FAIL_HOPELESS);
 
 	/* get info */
 	retval = TASK_FAIL;
 	mid = message_identifier_new (message_id);
-	message = acache_get_message (&mid, 1);
+	message = acache_get_message (ACACHE_DEFAULT_KEY, &mid, 1);
 	newsgroups = g_mime_message_get_header (message, HEADER_NEWSGROUPS);
 	author = g_mime_message_get_sender (message);
 	if (is_nonempty_string(newsgroups) && is_nonempty_string(author))
