@@ -247,8 +247,10 @@ main (int argc, char *argv[])
 		g_message ("Running in mute mode: messages will be sent to "
 		           "stdout instead of smtp/nntp server");
 
-	queue_init (pan_config_get_int("/Pan/General/Max_Connections_Total=4"),
-	            pan_config_get_int("/Pan/General/Max_Task_Tries_Total=4"));
+	queue_init (pan_config_get_bool ("/Pan/General/Online=TRUE"),
+	            pan_config_get_int  ("/Pan/General/Max_Connections_Total=4"),
+	            pan_config_get_int  ("/Pan/General/Max_Task_Tries_Total=4"),
+	            pan_config_get_bool ("/Pan/General/remove_failed_tasks=TRUE"));
 
 	/* if it's a new user, pop up the newuser dialog.
 	   Otherwise show the main window. */
@@ -308,54 +310,57 @@ pan_shutdown (void)
 
 /***
 ****
+****
 ***/
 
+static void
+restore_tasks_dialog_response_cb (GtkDialog * dialog, int response, gpointer user_data)
+{
+	char * filename = (char*) user_data;
+
+	if (response == GTK_RESPONSE_ACCEPT)
+	{
+		GPtrArray * tasks = g_ptr_array_new ();
+		task_xml_read (filename, tasks);
+		if (tasks->len)
+		{
+			guint i;
+			GSList * l = NULL;
+			for (i=0; i<tasks->len; ++i)
+				l = g_slist_prepend (l, g_ptr_array_index(tasks,i));
+			l = g_slist_reverse (l);
+			queue_insert_tasks (l, -1);
+			log_add_va (LOG_INFO, _("Restored %d tasks from \"%s\""), (int)tasks->len, filename);
+		}
+		g_ptr_array_free (tasks, TRUE);
+	}
+
+	unlink (filename);
+	g_free (filename);
+	gtk_widget_destroy (GTK_WIDGET(dialog));
+}
+
 static int
 restore_tasks_dialog (gpointer filename_gpointer)
 {
-	int response;
-	GPtrArray * tasks;
-	GtkWidget * dialog;
+	GtkWidget * w;
 	char * filename = (char*) filename_gpointer;
 
 	pan_lock ();
-	dialog = gtk_message_dialog_new (GTK_WINDOW(Pan.window),
-	                                 GTK_DIALOG_DESTROY_WITH_PARENT,
-	                                 GTK_MESSAGE_QUESTION,
-	                                 GTK_BUTTONS_NONE,
-					 _("Pan appears to have crashed or been killed the last time it was run.  Do you want to revert to the last session's task list?"));
-	gtk_dialog_add_buttons (GTK_DIALOG(dialog),
-	                        GTK_STOCK_REVERT_TO_SAVED,
-	                        GTK_RESPONSE_ACCEPT,
-	                        GTK_STOCK_NO,
-	                        GTK_RESPONSE_REJECT,
+	w = gtk_message_dialog_new (GTK_WINDOW(Pan.window),
+	                            GTK_DIALOG_DESTROY_WITH_PARENT,
+	                            GTK_MESSAGE_QUESTION,
+	                            GTK_BUTTONS_NONE,
+	                            _("Pan appears to have crashed or been killed the last time it was run.  Do you want to revert to the last session's task list?"));
+	gtk_dialog_add_buttons (GTK_DIALOG(w),
+	                        GTK_STOCK_NO, GTK_RESPONSE_REJECT,
+	                        GTK_STOCK_REVERT_TO_SAVED, GTK_RESPONSE_ACCEPT,
 	                        NULL);
-	gtk_widget_show_all (dialog);
-	response = gtk_dialog_run (GTK_DIALOG(dialog));
-	gtk_widget_destroy (dialog);
+	gtk_widget_show_all (w);
+	gtk_dialog_set_default_response (GTK_DIALOG(w), GTK_RESPONSE_ACCEPT);
+	g_signal_connect (w, "response", G_CALLBACK(restore_tasks_dialog_response_cb), filename);
+	gtk_widget_show_all (w);
 	pan_unlock ();
 
-	/* if the user wants to restore, load the old task file */
-	tasks = g_ptr_array_new ();
-	if (response == GTK_RESPONSE_ACCEPT)
-		task_xml_read (filename, tasks);
-
-	/* delete the old task file */
-	unlink (filename);
-
-	/* if tasks were read, add them to the queue */
-	if (tasks->len) {
-		guint i;
-		GSList * l = NULL;
-		for (i=0; i<tasks->len; ++i)
-			l = g_slist_prepend (l, g_ptr_array_index(tasks,i));
-		l = g_slist_reverse (l);
-		queue_insert_tasks (l, -1);
-		log_add_va (LOG_INFO, _("Restored %d tasks from \"%s\""), (int)tasks->len, filename);
-	}
-
-	/* cleanup */
-	g_ptr_array_free (tasks, TRUE);
-	g_free (filename);
 	return 0;
 }
