@@ -70,6 +70,7 @@ typedef struct
 	GtkWidget * flush_cache_on_exit_check;
 
 	/* general */
+	GtkWidget * online_on_startup_cbutton;
 	GtkWidget * single_click_selects_cbutton;
 	GtkWidget * remove_failed_tasks_cbutton;
 	GtkWidget * fetch_new_on_group_load_cbutton;
@@ -126,8 +127,6 @@ extern GtkTooltips * ttips;
 
 static char * text_quote_chars                = NULL;
 gboolean collapse_group_names                 = FALSE;
-gboolean use_system_bg                        = TRUE;
-gboolean use_system_fg                        = TRUE;
 gboolean text_window_smooth_scrolling         = TRUE;
 gboolean do_spellcheck                        = TRUE;
 int      text_window_smooth_scrolling_speed   = 10;
@@ -145,7 +144,6 @@ gboolean fetch_new_on_group_load              = TRUE;
 gboolean fetch_new_on_startup                 = FALSE;
 gboolean fetch_new_and_bodies_on_startup      = FALSE;
 gboolean header_pane_is_threaded              = TRUE;
-gboolean remove_failed_tasks                  = TRUE;
 gboolean single_click_selects                 = FALSE;
 gboolean show_group_pane                      = TRUE;
 gboolean show_header_pane                     = TRUE;
@@ -155,7 +153,6 @@ char * group_pane_font = NULL;
 char * header_pane_font = NULL;
 char * message_body_font = NULL;
 char * message_body_font_fixed = NULL;
-char * attribution_line = NULL;
 char * external_editor = NULL;
 char * external_web_browser = NULL;
 char * mail_server_address = NULL;
@@ -427,11 +424,6 @@ prefs_apply (void)
 		text_set_font ();
 	}
 
-	update_entry_and_bool_from_toggle_button (
-		&remove_failed_tasks,
-		"/Pan/General/remove_failed_tasks",
-		win->remove_failed_tasks_cbutton);
-
 	update_entry_and_bool_from_toggle_button (
 		&fetch_new_on_group_load,
 		"/Pan/General/fetch_new_on_group_load",
@@ -447,6 +439,9 @@ prefs_apply (void)
 		"/Pan/General/single_click_selects",
 		win->single_click_selects_cbutton);
 
+	pan_config_set_bool ("/Pan/General/Online",
+	                     gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(win->online_on_startup_cbutton)));
+
 	update_entry_and_bool_from_toggle_button (
 		&fetch_new_and_bodies_on_startup,
 		"/Pan/General/fetch_new_and_bodies_from_subscribed_on_startup",
@@ -557,6 +552,18 @@ prefs_apply (void)
 		"/Pan/User/external_web_browser",
 		GTK_COMBO(win->external_browser_combo)->entry);
 
+	/* remove failed tasks */
+	if (1) {
+		const gboolean old_val = queue_get_remove_failed_tasks ();
+		gboolean new_val = old_val;
+		update_entry_and_bool_from_toggle_button (
+			&new_val,
+			"/Pan/General/remove_failed_tasks",
+			win->remove_failed_tasks_cbutton);
+		if (old_val != new_val)
+			queue_set_remove_failed_tasks (new_val);
+	}
+
 	/* maximum number of connections */
 	if (1) {
 		GtkSpinButton * sb = GTK_SPIN_BUTTON (win->max_connections);
@@ -648,6 +655,9 @@ prefs_init (void)
 	show_group_pane = pan_config_get_bool("show_group_pane=TRUE");
 	show_header_pane = pan_config_get_bool("show_header_pane=TRUE");
 	show_body_pane = pan_config_get_bool("show_body_pane=TRUE");
+	/* don't let the user turn off _all_ the panes; that's too confusing. */
+	if (!show_group_pane && !show_header_pane && !show_body_pane)
+		show_group_pane = show_header_pane = show_body_pane = TRUE;
 	text_set_wrap (pan_config_get_bool("do_wrap=FALSE"));
 	collapse_group_names = pan_config_get_bool("collapse_group_names=FALSE");
 	text_window_smooth_scrolling = pan_config_get_bool("smooth_scrolling=TRUE");
@@ -662,7 +672,6 @@ prefs_init (void)
 	/* general preferences */
 	pan_config_push_prefix ("/Pan/General/");
 	single_click_selects = pan_config_get_bool ("single_click_selects=FALSE");
-	remove_failed_tasks = pan_config_get_bool ("remove_failed_tasks=TRUE");
 	wrap_column = pan_config_get_int ("wrap_column=74");
 	fetch_new_on_group_load = pan_config_get_bool ("fetch_new_on_group_load=TRUE");
 	fetch_new_on_startup = pan_config_get_bool ("fetch_new_from_subscribed_on_startup=FALSE");
@@ -727,8 +736,6 @@ prefs_init (void)
 	***  Colors
 	**/
 
-	use_system_bg = pan_config_get_bool ("/Pan/Display/use_system_bg=true");
-	use_system_fg = pan_config_get_bool ("/Pan/Display/use_system_fg=true");
 	set_color (&killfile_color,        "killfile_color",        35723, 17733,  4883);
 	set_color (&watched_color,         "watched_color",             0, 52428,     0);
 	set_color (&text_url_color,        "text_url_color",            0,     0, 40000);
@@ -780,7 +787,7 @@ tasks_page (void)
 	char * p;
 
 	row = 0;
-	t = gtk_table_new (4, 15, FALSE);
+	t = gtk_table_new (4, 16, FALSE);
 	gtk_table_set_row_spacings (GTK_TABLE(t), 6);
 
 	/* title for profile info section */
@@ -852,7 +859,7 @@ tasks_page (void)
 
 		/* remove failed tasks */
 		w = gtk_check_button_new_with_mnemonic (_("Automatically remove _failed tasks from the task manager"));
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), remove_failed_tasks);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), queue_get_remove_failed_tasks());
 		connect_signal_to_prefs_changed (w, "toggled");
 		win->remove_failed_tasks_cbutton = w;
 		gtk_table_attach_defaults (GTK_TABLE(t), w, 1, 4, row, row+1);
@@ -915,6 +922,35 @@ tasks_page (void)
 		gtk_table_attach_defaults (GTK_TABLE(t), w, 1, 4, row, row+1);
 		++row;
 
+	/* section spacer */
+	w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
+	gtk_widget_set_usize (w, 0u, 6u);
+	gtk_table_attach_defaults (GTK_TABLE(t), w, 0, 4, row, row+1);
+	++row;
+
+	/* title for profile info section */
+	g_snprintf (buf, sizeof(buf), "<b>%s</b>", _("Online"));
+	w = gtk_label_new (buf);
+	gtk_misc_set_alignment (GTK_MISC(w), 0.0f, 0.5f);
+	gtk_label_set_use_markup (GTK_LABEL(w), TRUE);
+	gtk_table_attach_defaults (GTK_TABLE(t), w, 0, 4, row, row+1);
+	++row;
+
+		items_in_section = 1;
+	
+		/* spacer to move the fields a little to the right of the name header */
+		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
+		gtk_widget_set_usize (w, 18u, 0u);
+		gtk_table_attach_defaults (GTK_TABLE(t), w, 0, 1, row, row+items_in_section);
+
+		/* download new headers */
+		w = gtk_check_button_new_with_mnemonic (_("_Start Pan in online mode"));
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), pan_config_get_bool ("/Pan/General/Online=TRUE"));
+		connect_signal_to_prefs_changed (w, "toggled");
+		win->online_on_startup_cbutton = w;
+		gtk_table_attach_defaults (GTK_TABLE(t), w, 1, 4, row, row+1);
+		++row;
+
 	return t;
 }
 
@@ -1440,7 +1476,7 @@ layout_get_new_string (GtkWidget * page)
 {
 	int i;
 	int layout = -1;
-	const char * order;
+	const char * order = "tag";
 	GtkWidget ** layout_buttons = (GtkWidget**) g_object_get_data (G_OBJECT(page), "layout_buttons");
 	GSList * l = (GSList*) g_object_get_data (G_OBJECT(page), "layout_radio_group");
 
@@ -1685,7 +1721,6 @@ cache_page (void)
 		adj = gtk_adjustment_new  (acache_max_megs, 1.0, 20000, 1.0, 1.0, 1.0);
 		w = win->cache_megs_sb = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1.0, 0);
 		connect_signal_to_prefs_changed (w, "changed");
-		//gtk_widget_set_usize (GTK_WIDGET (w), 100u, 0u);
 		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
 		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
 		++row;
