@@ -44,11 +44,8 @@
 #include <pan/base/pan-i18n.h>
 #include <pan/base/server.h>
 
-#include <pan/gui.h>
-#include <pan/prefs.h>
 #include <pan/nntp.h>
 #include <pan/queue.h>
-#include <pan/server-menu-ui.h>
 #include <pan/task-xml.h>
 
 /*********************
@@ -97,8 +94,10 @@ SocketInfo;
 **********************  Private Function Prototypes
 *********************/
 
+static void fire_queue_size_changed (int running_qty, int total_qty);
+static void fire_connecting_state_changed (gboolean connecting);
+static void fire_connection_size_changed (int increment);
 static void fire_online_status_changed (gboolean online);
-
 static void fire_message_id_status_changed (const char ** message_ids, int message_id_qty);
 
 static void queue_run_thread (gpointer data, gpointer user_data);
@@ -185,7 +184,7 @@ static int max_tries = 5;
 static guint running_tasks = 0;
 
 static gboolean _online = TRUE;
-
+static gboolean _remove_failed_tasks = TRUE;
 static gboolean _dirty = FALSE;
 
 /*********************
@@ -218,7 +217,7 @@ queue_wakeup (void)
 }
 
 void
-queue_init (int socket_qty, int tries)
+queue_init (gboolean online, int socket_qty, int tries, gboolean remove_failed_tasks)
 {
 	debug_enter ("queue_init");
 
@@ -240,6 +239,9 @@ queue_init (int socket_qty, int tries)
 
 	qcond = g_cond_new ();
 
+	_online = online;
+
+	_remove_failed_tasks = remove_failed_tasks;
 	max_tries = tries;
 	max_socket_qty = socket_qty;
 	_tpool = g_thread_pool_new (queue_run_thread, NULL, max_socket_qty, TRUE, NULL);
@@ -455,7 +457,7 @@ real_queue_shutdown (void)
 	len = g_slist_length (sockets_tmp);
 	for (l=sockets_tmp; l!=NULL; l=l->next)
 		pan_object_unref (PAN_OBJECT(((SocketInfo*)l->data)->socket));
-	gui_inc_connection_size (-len);
+	fire_connection_size_changed (-len);
 	g_slist_foreach (sockets_tmp, (GFunc)g_free, NULL);
 	g_slist_free (sockets_tmp);
 }
@@ -667,14 +669,14 @@ real_queue_insert_tasks (GSList * new_tasks, int index, gboolean * do_wakeup)
 			g_hash_table_insert (task_to_status, task, GINT_TO_POINTER(QUEUE_TASK_STATUS_QUEUED));
 		}
 
-		/* we need this for gui_set_queue_size */
+		/* we need this for fire_queue_size_changed */
 		task_len = g_slist_length (task_list);
 	}
 	_dirty = TRUE;
 	g_mutex_unlock (task_lock);
 
 	/* cleanup */
-	gui_set_queue_size (running_tasks, task_len);
+	fire_queue_size_changed (running_tasks, task_len);
 	pan_callback_call (queue_tasks_added, new_tasks, GINT_TO_POINTER(index));
 	*do_wakeup = TRUE;
 	debug_exit ("real_queue_insert_tasks");
@@ -725,7 +727,7 @@ real_queue_remove_tasks (GSList * tasks, gboolean * do_wakeup)
 	{
 		removed = g_slist_reverse (removed);
 
-		gui_set_queue_size (running_tasks, g_slist_length(task_list));
+		fire_queue_size_changed (running_tasks, g_slist_length(task_list));
 		*do_wakeup = TRUE;
 		pan_callback_call (queue_tasks_removed, removed, NULL);
 
@@ -845,6 +847,22 @@ real_requeue_failed_tasks (GSList * tasks, gboolean * do_wakeup)
 ***
 **/
 
+gboolean
+queue_get_remove_failed_tasks (void)
+{
+	return _remove_failed_tasks;
+}
+
+void
+queue_set_remove_failed_tasks (gboolean b)
+{
+	_remove_failed_tasks = b;
+}
+
+/**
+***
+**/
+
 int
 queue_get_max_tries (void)
 {
@@ -938,9 +956,9 @@ queue_run_thread (gpointer data, gpointer user_data)
 	status_item_set_active (STATUS_ITEM(task), TRUE);
 	if (task->sock)
 		pan_socket_reset_statistics (task->sock);
-	gui_set_queue_size (++running_tasks, queue_get_length());
+	fire_queue_size_changed (++running_tasks, queue_get_length());
 	result = task_run (task);
-	gui_set_queue_size (--running_tasks, queue_get_length());
+	fire_queue_size_changed (--running_tasks, queue_get_length());
 	debug3 (DEBUG_QUEUE, "queue task[%p] (%s) returned: %d", task, desc, result);
 	g_free(desc);
 
@@ -964,7 +982,7 @@ queue_run_thread (gpointer data, gpointer user_data)
 		/* success, remove the task */
 		queue_remove_tasks (g_slist_append(NULL,task));
 	}
-	else if (result==TASK_FAIL_HOPELESS && remove_failed_tasks)
+	else if (result==TASK_FAIL_HOPELESS && _remove_failed_tasks)
 	{
 		/* failed, and requeueing won't help, so remove it now */
 		queue_remove_tasks (g_slist_append(NULL,task));
@@ -1101,7 +1119,7 @@ queue_get_socket_for_task (Task *task)
 			PanSocket *sock = NULL;
 			debug1 (DEBUG_QUEUE, "making a new socket to %s", server->address);
 
-			gui_set_connecting_flag (TRUE);
+			fire_connecting_state_changed (TRUE);
 			sock = pan_socket_new (server->name, server->address, server->port);
 			pan_socket_set_nntp_auth (sock, server->need_auth, server->username, server->password); 
 
@@ -1109,7 +1127,7 @@ queue_get_socket_for_task (Task *task)
 			if (sock == NULL || sock->sockfd < 0)
 			{
 				if (is_nonempty_string (server->address))
-					errmsg = g_strdup_printf (_("Unable to connect to server %s:%d\n"), server->address, server->port);
+					errmsg = g_strdup_printf (_("Unable to connect to server %s:%d"), server->address, server->port);
 				else
 					errmsg = g_strdup (_("No server address specified in the Preferences"));
 			}
@@ -1147,10 +1165,10 @@ queue_get_socket_for_task (Task *task)
 				task->sock = sock;
 				sockets = g_slist_prepend (sockets, si);
 				retval = 0;
-				gui_inc_connection_size (1);
+				fire_connection_size_changed (1);
 			}
 
-			gui_set_connecting_flag (FALSE);
+			fire_connecting_state_changed (FALSE);
 		}
 	}
 
@@ -1292,7 +1310,7 @@ socket_cleanup (Task* task)
 			/* otherwise throw it away */
 			nntp_disconnect (NULL, socket);
 			pan_object_unref (PAN_OBJECT(socket));
-			gui_inc_connection_size (-1);
+			fire_connection_size_changed (-1);
 			sockets = g_slist_remove (sockets, info);
 			g_free (info);
 
@@ -1400,7 +1418,7 @@ sockets_upkeep (void)
 
 		/* destroy the socket */
 		pan_object_unref (PAN_OBJECT(sock));
-		gui_inc_connection_size (-1);
+		fire_connection_size_changed (-1);
 		sockets = g_slist_remove (sockets, info);
 
 		/* cleanup local */
@@ -1518,8 +1536,6 @@ queue_mainloop (void* unused)
 			}
 		}
 	}
-
-	return NULL;
 }
 
 
@@ -1553,3 +1569,51 @@ fire_message_id_status_changed (const char ** message_ids, int message_id_qty)
 	                   message_ids,
 	                   GINT_TO_POINTER(message_id_qty));
 }
+
+PanCallback*
+queue_get_connection_size_changed_callback (void)
+{
+	static PanCallback * cb = NULL;
+	if (cb==NULL) cb = pan_callback_new ();
+	return cb;
+}
+
+static void
+fire_connection_size_changed (int increment)
+{
+	pan_callback_call (queue_get_connection_size_changed_callback(),
+	                   GINT_TO_POINTER(increment),
+	                   NULL);
+}
+
+PanCallback*
+queue_get_connecting_state_changed_callback (void)
+{
+	static PanCallback * cb = NULL;
+	if (cb==NULL) cb = pan_callback_new ();
+	return cb;
+}
+
+static void
+fire_connecting_state_changed (gboolean connecting)
+{
+	pan_callback_call (queue_get_connecting_state_changed_callback(),
+	                   GINT_TO_POINTER(connecting),
+	                   NULL);
+}
+
+PanCallback*
+queue_get_size_changed_callback (void)
+{
+	static PanCallback * cb = NULL;
+	if (cb==NULL) cb = pan_callback_new ();
+	return cb;
+}
+
+static void
+fire_queue_size_changed (int running_qty, int total_qty)
+{
+	pan_callback_call (queue_get_size_changed_callback(),
+	                   GINT_TO_POINTER(running_qty),
+	                   GINT_TO_POINTER(total_qty));
+}
