@@ -36,6 +36,7 @@
 #include <pan/base/serverlist.h>
 
 #include <pan/pan-file-entry.h>
+#include <pan/queue.h>
 #include <pan/server-ui.h>
 #include <pan/server-menu-ui.h>
 #include <pan/util.h>
@@ -244,6 +245,8 @@ server_edit_response_cb (GtkDialog * w, int response, gpointer user_data)
 
 		if (d->new_server)
 			serverlist_add_server (d->server);
+
+		queue_wakeup ();
 	}
 	else /* cancel, destroy */
 	{
@@ -524,7 +527,6 @@ get_public_server_array (void)
 static void
 server_tree_view_refresh (ServerListDialog * d)
 {
-
 	GtkTreeSelection * selection = gtk_tree_view_get_selection(GTK_TREE_VIEW (d->server_tree_view));
 	gboolean found_selected = FALSE;
 	GtkTreeIter selected_iter;
@@ -573,7 +575,6 @@ server_list_dialog_response_cb (GtkDialog * dialog, int response, gpointer user_
 static void
 server_list_dialog_destroy_cb (GtkWidget * w, gpointer user_data)
 {
-	g_free (user_data);
 	server_menu_update ();
 }
 
@@ -598,17 +599,20 @@ remove_button_clicked_cb (GtkButton * button, gpointer data)
 static void
 server_edit_dialog_destroy_cb (GtkWidget * w, gpointer user_data)
 {
-	ServerListDialog * d = (ServerListDialog*) user_data;
-	server_tree_view_refresh (d);
+	if (GTK_IS_WIDGET (user_data))
+	{
+		ServerListDialog * d = (ServerListDialog*) g_object_get_data (G_OBJECT(user_data), "dialog");
+		server_tree_view_refresh (d);
+	}
 }
 
 static void
 add_button_clicked_cb (GtkButton * button, gpointer data)
 {
-	ServerListDialog * f = (ServerListDialog*) data;
-	GtkWidget * dialog = server_edit_dialog (f->dialog, NULL);
-	g_signal_connect (dialog, "destroy", G_CALLBACK(server_edit_dialog_destroy_cb), f);
-	gtk_widget_show_all (dialog);
+	GtkWidget * list_dialog = GTK_WIDGET (data);
+	GtkWidget * edit_dialog = server_edit_dialog (list_dialog, NULL);
+	g_signal_connect (edit_dialog, "destroy", G_CALLBACK(server_edit_dialog_destroy_cb), list_dialog);
+	gtk_widget_show_all (edit_dialog);
 }
 
 /**
@@ -618,12 +622,13 @@ add_button_clicked_cb (GtkButton * button, gpointer data)
 static void
 edit_button_clicked_cb (GtkButton * button, gpointer user_data)
 {
-	ServerListDialog * d = (ServerListDialog*) user_data;
+	GtkWidget * list_dialog = GTK_WIDGET (user_data);
+	ServerListDialog * d = (ServerListDialog*) g_object_get_data (G_OBJECT(list_dialog), "dialog");
 	Server * selected_server = get_selected_server (d);
 	if (selected_server != NULL) {
-		GtkWidget * dialog = server_edit_dialog (d->dialog, selected_server);
-		g_signal_connect (GTK_OBJECT(dialog), "destroy", G_CALLBACK(server_edit_dialog_destroy_cb), d);
-		gtk_widget_show_all (dialog);
+		GtkWidget * edit_dialog = server_edit_dialog (list_dialog, selected_server);
+		g_signal_connect (GTK_OBJECT(edit_dialog), "destroy", G_CALLBACK(server_edit_dialog_destroy_cb), list_dialog);
+		gtk_widget_show_all (edit_dialog);
 	}
 }
 
@@ -664,6 +669,7 @@ server_dialog_new (GtkWindow * parent)
 	gtk_window_set_policy (GTK_WINDOW(w), TRUE, TRUE, TRUE);
 	g_signal_connect (GTK_OBJECT(w), "response", G_CALLBACK(server_list_dialog_response_cb), d);
 	g_signal_connect (GTK_OBJECT(w), "destroy", G_CALLBACK(server_list_dialog_destroy_cb), d);
+	g_object_set_data_full (G_OBJECT(w), "dialog", d, g_free);
 
 	/* workarea */
 	hbox = gtk_hbox_new (FALSE, GUI_PAD);
@@ -694,7 +700,7 @@ server_dialog_new (GtkWindow * parent)
 	/* Add callbacks */
 	g_signal_connect (GTK_TREE_VIEW (d->server_tree_view), "row-activated",
  			  G_CALLBACK (server_tree_view_row_activated_cb),
- 			  d);
+ 			  d->dialog);
 	g_signal_connect (G_OBJECT (selection), "changed",
 			  G_CALLBACK (server_tree_view_selection_changed_cb),
 			  d);
@@ -714,13 +720,13 @@ server_dialog_new (GtkWindow * parent)
 	w = gtk_button_new_from_stock (GTK_STOCK_ADD);
 	gtk_box_pack_start (GTK_BOX (bbox), w, FALSE, FALSE, 0);
 	gtk_tooltips_set_tip (tips, w, _("Add a new server"), NULL);
-	g_signal_connect (w, "clicked", G_CALLBACK(add_button_clicked_cb), d);
+	g_signal_connect (w, "clicked", G_CALLBACK(add_button_clicked_cb), d->dialog);
 
 	/* edit button */
 	w = gtk_button_new_from_stock (GTK_STOCK_OPEN);
 	gtk_box_pack_start (GTK_BOX (bbox), w, FALSE, FALSE, 0);
 	gtk_tooltips_set_tip (tips, w, _("Edit the selected server"), NULL);
-	g_signal_connect (w, "clicked", G_CALLBACK(edit_button_clicked_cb), d);
+	g_signal_connect (w, "clicked", G_CALLBACK(edit_button_clicked_cb), d->dialog);
 	d->edit_button = w;
 
 	/* remove button */
