@@ -257,8 +257,13 @@ smtp_send_message (StatusItem    * item,
 		const char * line_start = NULL;
 
 		while ((get_next_token_run (march, '\n', &march, &line_start, &line_len)))
-			okay = okay && !smtp_send_line_va (item, sock, "%*.*s\n", line_len, line_len, line_start);
+		{
+			char * line = g_alloca (line_len+1);
+			memcpy (line, line_start, line_len);
+			line[line_len] = '\0';
 
+			okay = okay && !smtp_send_line_va (item, sock, "%s\n", line);
+		}
 		g_free (pch);
 	}
 
