@@ -215,12 +215,11 @@ pan_socket_set_nntp_auth (PanSocket            * socket,
 
 
 /* simple read buffering to hopefully speed things up. */
-static gint
+static int
 read_from_socket (PanSocket* ps)
 {
-	gint read_count = 0;
-	gint retval = 0;
-	gint my_errno = 0;
+	int read_count = 0;
+	int my_errno = 0;
 	struct timeval tv;
 	fd_set read_fds;
 
@@ -235,7 +234,7 @@ read_from_socket (PanSocket* ps)
 	errno = 0;
 	tv.tv_sec = 60;
 	tv.tv_usec = 0;
-	retval = select (ps->sockfd+1, &read_fds, NULL, NULL, &tv);
+	select (ps->sockfd+1, &read_fds, NULL, NULL, &tv);
 	my_errno = errno;
 
 	/* did select fail? */
