@@ -1,11 +1,11 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #ifndef __PAN_SOCKET_H__
@@ -52,7 +51,8 @@ struct _PanSocket
 	GArray * line_buffer;
 
 	/* statistics */
-	gchar * host;
+	char * host_address;
+	char * host_name;
 	unsigned int bytes_read;
 	unsigned int bytes_written;
 	time_t byte_count_start_time;
@@ -64,7 +64,8 @@ struct _PanSocket
 
 void         pan_socket_constructor          (PanSocket            * sock,
                                               PanObjectDestructor    dtor,
-                                              const gchar          * address,
+                                              const char           * server_name,
+                                              const char           * server_address,
                                               int                    port);
 
 void         pan_socket_destructor           (PanObject            * sock);
@@ -73,34 +74,35 @@ void         pan_socket_destructor           (PanObject            * sock);
 ***  PUBLIC - Life Cycle
 **/
 
-PanSocket*   pan_socket_new                  (const gchar          * address,
+PanSocket*   pan_socket_new                  (const char           * server_name,
+                                              const char           * server_address,
                                               int                    port);
 
 void         pan_socket_set_nntp_auth        (PanSocket            * socket,
                                               gboolean               need_auth,
-                                              const gchar          * username,
-                                              const gchar          * password);
+                                              const char           * username,
+                                              const char           * password);
 /**
 ***  PUBLIC - UTIL
 **/
 
-gint         fd_write                        (gint                   fd,
-                                              const gchar          * buf,
-                                              gint                   len);
+int         fd_write                        (int                   fd,
+                                             const char          * buf,
+                                             int                   len);
 
 /**
 ***  PUBLIC - I/O
 **/
 
 int          pan_socket_putline              (PanSocket            * sock,
-                                              const gchar          * line);
+                                              const char           * line);
 
 int          pan_socket_putline_va           (PanSocket            * sock,
-                                              const gchar          * fmt,
+                                              const char           * fmt,
                                               ...);
 
 int          pan_socket_getline              (PanSocket            * sock,
-                                              const gchar         ** setme);
+                                              const char          ** setme);
 
 void         pan_socket_flush                (PanSocket            * sock);
 
