@@ -55,6 +55,7 @@ typedef struct
 	guint signal_id_status;
 	guint signal_id_error;
 
+	GMutex * last_status_mutex;
 	GString * last_status;
 	StatusItem * item;
 
@@ -100,63 +101,44 @@ extern GtkTooltips * ttips;
 ******  Private Utilities
 *****/
 
-static void
-string_remove_char (char * str, int index)
-{
-	char *p = str + index;
-	for (;;) {
-		++p;
-		p[-1] = *p;
-		if (!*p)
-			break;
-	}
-}
-
 static char*
 string_shorten_nolock (GdkFont          * font,
                        int                max_width,
                        const char       * original,
                        int                original_len)
 {
-	char *clone = NULL;
-	size_t len;
-	int i, j;
 	gboolean i_turn = TRUE;
-	const int DOTLEN = 3;
-	int target_width = max_width - 4 - 30; /* 30 is just a magic safety number */
+	int i, j, original_midpoint;
+	const int target_width = max_width - 4 - 30; /* 30 is just a magic safety number */
+	GString * tmp;
 
-	/* if original is NULL, so is shortened */
+	/* handle cases where the string isn't too long */
 	if (!original)
 		return NULL;
-
-	/* if original fits, use it */
 	if (gdk_text_width (font, original, original_len) < target_width)
-		return g_strdup (original);
+		return g_strndup (original, original_len);
 
-	/* add in the middle "..." */
-	len = original_len;
-	clone = g_malloc (original_len + DOTLEN + 1);
-	i = original_len/2;
-	memcpy (clone, original, i);
-	memcpy (clone+i, "...", DOTLEN);
-	j = i+3;
-	memcpy (clone+j, original+i, original_len+1-i);
-	original = NULL;
+	/* build the clone with elipses in the middle */
+	original_midpoint = original_len / 2;
+       	tmp = g_string_sized_new (128);
+	g_string_append_len (tmp, original, original_midpoint);
+	g_string_append (tmp, "...");
+	g_string_append (tmp, original+original_midpoint);
 
-	/* start trimming out text until it fits */	
-	len = original_len + DOTLEN;
-	while (gdk_text_width(font, clone, len) >= target_width)
+	/* start trimming out the characters next to the ellipses */
+	i = original_midpoint - 1;
+	j = original_midpoint + 3; /* "..." */
+	while (gdk_text_width (font, tmp->str, tmp->len) >= target_width)
 	{
 		int *pi = i_turn ? &i : &j;
 		*pi -= 1;
-		if (*pi<0 || *pi>=len)
+		if (*pi<0 || *pi>=tmp->len)
 			break;
-		string_remove_char (clone, *pi);
+		g_string_erase (tmp, *pi, 1);
 		i_turn = !i_turn;
-		--len;
 	}
 
-	return clone;
+	return g_string_free (tmp, FALSE);
 }
 
 /*****
@@ -172,7 +154,7 @@ set_percentage_mainthread (gpointer user_data)
 
 	/* update the percentage */
 	pan_lock ();
-	gtk_progress_set_percentage (progress, (gfloat)(of_100/100.0));
+	gtk_progress_set_percentage (progress, of_100/100.0);
 	pan_unlock ();
 
 	/* cleanup */
@@ -200,10 +182,14 @@ status_cb_mainthread (gpointer data)
 	if (impl!=NULL && impl->item!=NULL)
 	{
 		GtkProgress * prog = GTK_PROGRESS(impl->progress);
-		char * s = string_shorten_nolock (gtk_style_get_font(gtk_widget_get_style(impl->progress)),
+		char * s;
+
+		g_mutex_lock (impl->last_status_mutex);
+		s = string_shorten_nolock (gtk_style_get_font(gtk_widget_get_style(impl->progress)),
 		                                   impl->progress->allocation.width,
 		                                   impl->last_status->str,
 		                                   impl->last_status->len);
+		g_mutex_unlock (impl->last_status_mutex);
 
 		/* update progressbar w/o a resize */
 		replace_gstr (&prog->format, s);
@@ -218,7 +204,11 @@ static void
 status_cb (gpointer call_object, gpointer call_arg, gpointer user_data)
 {
 	Impl * impl = (Impl*) user_data;
+
+	g_mutex_lock (impl->last_status_mutex);
 	g_string_assign (impl->last_status, (const char*) call_arg);
+	g_mutex_unlock (impl->last_status_mutex);
+
 	gui_queue_add (status_cb_mainthread, impl);
 }
 
@@ -266,6 +256,12 @@ destroy_cb (GtkWidget* w)
 			impl->last_status = NULL;
 		}
 
+		if (impl->last_status_mutex != NULL)
+		{
+			g_mutex_free (impl->last_status_mutex);
+			impl->last_status_mutex = NULL;
+		}
+
 		g_free (impl);
 		g_object_set_data (G_OBJECT(w), "impl", NULL);
 	}
@@ -298,6 +294,7 @@ status_item_view_new (void)
 
 	impl = g_new0 (Impl, 1);
 	impl->last_status = g_string_new (NULL);
+	impl->last_status_mutex = g_mutex_new ();
 	impl->progress = progress;
 	g_object_set_data (G_OBJECT(w), "impl", impl);
 	return w;
@@ -322,7 +319,9 @@ status_item_view_set_item_nolock (GtkWidget * w, StatusItem* item)
 		/* update last_status; update progressbar w/o a resize */
 		gui_queue_add (set_percentage_mainthread, argset_new2 (impl->progress, NULL));
 		prog = GTK_PROGRESS(impl->progress);
+		g_mutex_lock (impl->last_status_mutex);
 		g_string_assign (impl->last_status, "");
+		g_mutex_unlock (impl->last_status_mutex);
 		replace_gstr (&prog->format, g_strdup(""));
 		GTK_PROGRESS_GET_CLASS (GTK_OBJECT(prog))->update (prog);
 		gtk_tooltips_set_tip (ttips, w, NULL, NULL);
