@@ -82,14 +82,17 @@ static char* task_bodies_describe (const StatusItem* item);
 static void
 task_bodies_destructor (PanObject * obj)
 {
+	TaskBodies * task;
 	debug_enter ("task_bodies_destructor");
 
 	/* sanity clause */
 	g_return_if_fail (obj!=NULL);
 
 	/* task-bodies dtor */
-	acache_checkin ((MessageIdentifier **)TASK(obj)->identifiers->pdata, TASK(obj)->identifiers->len);
+	task = TASK_BODIES(obj);
+	acache_checkin (task->acache_key, (MessageIdentifier **)TASK(obj)->identifiers->pdata, TASK(obj)->identifiers->len);
 	TASK_BODIES(obj)->working_index = 0;
+	g_free (task->acache_key);
 
 	/* destroy parent class */
 	task_destructor (obj);
@@ -109,6 +112,7 @@ task_bodies_new (Server               * server,
 	int i;
 	TaskBodies* bodies = NULL;
 	gboolean needs_socket;
+	const char * acache_key = ACACHE_DEFAULT_KEY;
 	debug_enter ("task_bodies_new");
 
 	/* sanity clause */
@@ -116,12 +120,12 @@ task_bodies_new (Server               * server,
 	g_return_val_if_fail (identifier_qty!=0, NULL);
 
 	/* check out all the articles */
-	acache_checkout (identifiers, identifier_qty);
+	acache_checkout (acache_key, identifiers, identifier_qty);
 
 	/* do we need a connection? */
 	needs_socket = FALSE;
 	for (i=0; i<identifier_qty && !needs_socket; ++i)
-		if (!acache_has_message (identifiers[i]->message_id))
+		if (!acache_has_message (acache_key, identifiers[i]->message_id))
 			needs_socket = TRUE;
 
 	/* create the object */
@@ -139,6 +143,7 @@ task_bodies_new (Server               * server,
 
 	/* initialize the task-bodies */
 	bodies->working_index = 0;
+	bodies->acache_key = g_strdup (acache_key);
 	task_add_identifiers (TASK(bodies), identifiers, identifier_qty);
 
 	debug_exit ("task_bodies_new");
@@ -177,7 +182,17 @@ task_bodies_new_from_articles (const Article       ** articles,
 static char*
 task_bodies_describe (const StatusItem* si)
 {
-	return g_strdup_printf (_("Downloading %d articles"), TASK(si)->identifiers->len);
+	char * retval;
+	GPtrArray * mids = TASK(si)->identifiers;
+
+	if (mids->len == 1u) {
+		MessageIdentifier * mid = MESSAGE_IDENTIFIER (g_ptr_array_index (mids, 0));
+		retval = g_strdup (message_identifier_get_readable_name (mid));
+	} else {
+		retval = g_strdup_printf (_("Downloading %u articles"), mids->len);
+	}
+
+	return retval;
 }
 
 /*****
