@@ -156,16 +156,12 @@ task_grouplist_run (Task *item)
 {
 	TaskGroupList *item_g = TASK_GROUPLIST(item);
 	Server *server = item->server;
-	gint result;
+	int result;
 
-	if (1) {
-		char buf[32];
-		g_snprintf (buf, sizeof(buf), "%lu", (gulong)item_g->groups->len);
-		status_item_emit_status_va (STATUS_ITEM(item),
-			_("Got %s groups from server \"%s\""),
-			buf,
-			server_get_name (server));
-	}
+	status_item_emit_status_va (STATUS_ITEM(item),
+		_("Got %d groups from server \"%s\""),
+		(int)item_g->groups->len,
+		server_get_name (server));
 
 	/**
 	***  Try to get the groups from the server
@@ -286,13 +282,10 @@ nntp_grouplist_download_all (TaskGroupList * item)
 
 			/* periodic status feedback */
 			if (groups->len % 256 == 0)
-			{
-				char buf[32];
-				g_snprintf (buf, sizeof(buf), "%lu", (gulong)groups->len);
 				status_item_emit_status_va (status,
-					_("Got %s groups from server \"%s\""),
-					buf, server_get_name (server));
-			}
+					_("Got %d groups from server \"%s\""),
+					(int)groups->len,
+					server_get_name (server));
 
 			/* cleanup */
 			g_free (name);
@@ -359,14 +352,10 @@ nntp_grouplist_download_all (TaskGroupList * item)
 		/* let the user know we're still on the job */
 		++description_count;
 		if (description_count % 43 == 0)
-		{
-			char buf[32];
-			g_snprintf (buf, sizeof(buf), "%d", description_count);
-			status_item_emit_status_va (
-				STATUS_ITEM(item),
-				_("Got %s descriptions from server \"%s\""),
-				buf, server_get_name (server));
-		}
+			status_item_emit_status_va (STATUS_ITEM(item),
+			                            _("Got %d descriptions from server \"%s\""),
+			                            description_count,
+			                            server_get_name (server));
 
 		/* try to update the group's description field */
 		if (is_nonempty_string(name) &&
@@ -482,7 +471,7 @@ nntp_grouplist_download_new (TaskGroupList * item)
 
 	if (1) {
 		char buf[512];
-		g_snprintf (buf, sizeof(buf), _("Got %d new groups from server \"%s\""), count, server_get_name(server));
+		g_snprintf (buf, sizeof(buf), _("Got %d groups from server \"%s\""), count, server_get_name(server));
 		log_add (LOG_INFO, buf);
 		status_item_emit_status (STATUS_ITEM(item), buf);
 	}
