@@ -65,14 +65,14 @@ typedef struct
 }
 ManagerUI;
 
-enum
-{
+enum {
 	TASK_MANAGER_STATUS_COLUMN,
 	TASK_MANAGER_PERCENTAGE_COLUMN,
 	TASK_MANAGER_SERVER_COLUMN,
 	TASK_MANAGER_TIME_COLUMN,
 	TASK_MANAGER_DESCRIPTION_COLUMN,
 	TASK_MANAGER_DATA_COLUMN,
+	TASK_MANAGER_TASK_STATE_COLUMN,
 	TASK_MANAGER_N_COLUMNS
 };
 
@@ -257,7 +257,7 @@ get_secs_left (const Task * task)
 		double progress = status_item_get_progress_of_100 (STATUS_ITEM(task)) / 100.0;
 		double safe_progress = MAX(progress, 0.01); /* don't divide by 0 */
 		const double estimated_total_time = elapsed_time/safe_progress;
-		retval = estimated_total_time - elapsed_time;
+		retval = (int)(estimated_total_time - elapsed_time);
 	}
 
 	return retval;
@@ -363,14 +363,14 @@ refresh_row_nolock (GtkListStore * model, GtkTreeIter * iter, Task * task)
 	text[4] = pan_utf8ize (description, -1, freeme+4);
 
 	/* update the row */
-	gtk_list_store_set (model, iter,
-			    TASK_MANAGER_STATUS_COLUMN, text[0],
-			    TASK_MANAGER_PERCENTAGE_COLUMN, text[1],
-			    TASK_MANAGER_SERVER_COLUMN, text[2],
-			    TASK_MANAGER_TIME_COLUMN, text[3],
-			    TASK_MANAGER_DESCRIPTION_COLUMN, text[4],
-			    TASK_MANAGER_DATA_COLUMN, task,
-			    -1);
+	gtk_list_store_set (model, iter, TASK_MANAGER_STATUS_COLUMN, text[0],
+	                                 TASK_MANAGER_PERCENTAGE_COLUMN, text[1],
+	                                 TASK_MANAGER_SERVER_COLUMN, text[2],
+	                                 TASK_MANAGER_TIME_COLUMN, text[3],
+	                                 TASK_MANAGER_DESCRIPTION_COLUMN, text[4],
+	                                 TASK_MANAGER_DATA_COLUMN, task,
+	                                 TASK_MANAGER_TASK_STATE_COLUMN, (int)queue_get_task_status(task),
+	                                 -1);
 						
 	/* cleanup */
 	g_free (description);
@@ -548,13 +548,14 @@ tasks_moved_cb (gpointer call_obj, gpointer call_arg, gpointer client_arg)
 	pan_lock ();
 	for (l=tasks; l!=NULL; l=l->next) {
 		const int index = find_task_index (ui->list, l->data);
-
 		if (index != -1) {
+			Task * task = NULL;
 			gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (ui->list), &iter, NULL, index);
-			gtk_list_store_remove (ui->list, &iter);			
+			gtk_tree_model_get (GTK_TREE_MODEL (ui->list), &iter, TASK_MANAGER_DATA_COLUMN, &task, -1);
+			if (task != NULL)
+				pan_object_unref (PAN_OBJECT(task));
+			gtk_list_store_remove (ui->list, &iter);
 		}
-		
-
 	}
 	pan_unlock ();
 
@@ -564,14 +565,9 @@ tasks_moved_cb (gpointer call_obj, gpointer call_arg, gpointer client_arg)
 	/* select the new rows */
 	pan_lock ();
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (ui->task_manager_tree_view));
-
-	if (index == -1) {
+	if (index == -1)
 		index = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (ui->list), NULL) - g_slist_length(tasks);
-	}
-	
-
 	for (l=tasks, i=index; l!=NULL; l=l->next, ++i) {
-
 		gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (ui->list), &iter, NULL, i);
 		gtk_tree_selection_select_iter (selection, &iter);
 	}
@@ -588,7 +584,7 @@ update_titlebar_lock (ManagerUI * ui)
 	int running, queued, failed;
 	guint i;
 	GPtrArray * tasks;
-	gchar* pch;
+	char title[1024] = { '\0' };
 
 	g_return_if_fail (ui!=NULL);
 
@@ -609,17 +605,16 @@ update_titlebar_lock (ManagerUI * ui)
 
 	/* create a titlebar */
 	if (failed)
-		pch = g_strdup_printf (_("Pan %s Task Manager (%d Queued, %d Running, %d Failed)"), VERSION, queued, running, failed);
+		g_snprintf (title, sizeof(title), _("Pan %s Task Manager (%d Queued, %d Running, %d Failed)"), VERSION, queued, running, failed);
 	else if (queued || running)
-		pch = g_strdup_printf (_("Pan %s Task Manager (%d Queued, %d Running)"), VERSION, queued, running);
+		g_snprintf (title, sizeof(title), _("Pan %s Task Manager (%d Queued, %d Running)"), VERSION, queued, running);
 	else 
-		pch = g_strdup_printf (_("Pan %s Task Manager"), VERSION);
+		g_snprintf (title, sizeof(title), _("Pan %s Task Manager"), VERSION);
 
 	/* show the titlebar */
 	pan_lock ();
-	gtk_window_set_title (GTK_WINDOW(ui->window), pch);
+	gtk_window_set_title (GTK_WINDOW(ui->window), title);
 	pan_unlock ();
-	g_free (pch);
 
 	/* cleanup */
 	pan_g_ptr_array_foreach (tasks, (GFunc)pan_object_unref, NULL);
@@ -627,14 +622,14 @@ update_titlebar_lock (ManagerUI * ui)
 }
 
 static gboolean
-refresh_row_nolock_foreach (GtkTreeModel  * model,
-                            GtkTreePath   * path,
-                            GtkTreeIter   * iter,
-                            gpointer        gpointer_visible_rect)
+periodic_update_timer_foreach (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer unused)
 {
 	Task * task;
-	gtk_tree_model_get (model, iter, TASK_MANAGER_DATA_COLUMN, &task, -1);
-	refresh_row_nolock ((GtkListStore*)model, iter, task);
+	int old_state, new_state;
+	gtk_tree_model_get (model, iter, TASK_MANAGER_DATA_COLUMN, &task, TASK_MANAGER_TASK_STATE_COLUMN, &old_state, -1);
+	new_state = queue_get_task_status(task);
+	if (new_state!=old_state || new_state==QUEUE_TASK_STATUS_RUNNING)
+		refresh_row_nolock ((GtkListStore*)model, iter, task);
 
 	return FALSE;
 }
@@ -648,7 +643,7 @@ periodic_update_timer (gpointer user_data)
 
 	/* update the grid */
 	pan_lock ();
-	gtk_tree_model_foreach (GTK_TREE_MODEL(ui->list), refresh_row_nolock_foreach, NULL);
+	gtk_tree_model_foreach (GTK_TREE_MODEL(ui->list), periodic_update_timer_foreach, NULL);
 	pan_unlock ();
 
 	/* update the titlebar */	
@@ -668,6 +663,16 @@ window_delete_event_cb (GtkWidget * w, GdkEvent * e, gpointer data)
 	gui_save_column_widths_tree_view (ui->task_manager_tree_view, "task_manager_2");
 	return FALSE;
 } 
+
+static gboolean
+ui_destroy_cb_tree_model_foreach_func (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
+{
+	Task * task = NULL;
+	gtk_tree_model_get (model, iter, TASK_MANAGER_DATA_COLUMN, &task, -1);
+	if (task != NULL)
+		pan_object_unref (PAN_OBJECT(task));
+	return FALSE;
+}
 static void
 ui_destroy_cb (GtkObject * o, gpointer user_data)
 {
@@ -679,6 +684,8 @@ ui_destroy_cb (GtkObject * o, gpointer user_data)
 
 	g_source_remove (ui->timeout_id);
 
+	gtk_tree_model_foreach (GTK_TREE_MODEL(ui->list), ui_destroy_cb_tree_model_foreach_func, ui);
+
 	pan_warn_if_fail (ui == manager_ui);
 	g_free (manager_ui);
 	manager_ui = NULL;
@@ -749,13 +756,13 @@ task_manager_build_ui (GtkWidget * window)
 	gtk_box_pack_start (GTK_BOX(vbox), toolbar, FALSE, FALSE, 0);
 
 	/* Create the list_store */
-	ui->list = gtk_list_store_new (TASK_MANAGER_N_COLUMNS,
-				       G_TYPE_STRING,
-				       G_TYPE_STRING,
-				       G_TYPE_STRING,
-				       G_TYPE_STRING,
-				       G_TYPE_STRING,
-				       G_TYPE_POINTER);
+	ui->list = gtk_list_store_new (TASK_MANAGER_N_COLUMNS, G_TYPE_STRING,
+	                                                       G_TYPE_STRING,
+	                                                       G_TYPE_STRING,
+	                                                       G_TYPE_STRING,
+	                                                       G_TYPE_STRING,
+	                                                       G_TYPE_POINTER,
+	                                                       G_TYPE_INT);
 		
 	/* Create the tree_view */
 	w = ui->task_manager_tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (ui->list));
