@@ -38,7 +38,6 @@ typedef struct
 
 	/* private */
 	int            runval;
-	char         * readable_name;
 
 	gboolean       save_attachments;
 	char         * path_attachments;
@@ -56,12 +55,10 @@ TaskSave;
 **/
 
 PanObject* task_save_new                (Server              * server,
-                                         const char          * readable_name_for_description,
                                          MessageIdentifier  ** identifiers,
                                          int                   identifier_qty);
 
-PanObject* task_save_new_from_articles  (const char          * readable_name_for_description,
-                                         const Article      ** articles,
+PanObject* task_save_new_from_articles  (const Article      ** articles,
                                          int                   article_qty);
 
 void task_save_set_attachments        (TaskSave       * task,
