@@ -1,11 +1,11 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 /*********************
@@ -104,31 +103,77 @@ task_constructor (Task *item,
 	item->tries = 0;
 	item->run = run;
 	item->task_ran_callback = pan_callback_new ();
+	item->identifiers = g_ptr_array_new ();
 
         debug1 (DEBUG_PAN_OBJECT, "task constructor: %p", item);
 }
 
 void
-task_destructor (PanObject* item)
+task_destructor (PanObject* obj)
 {
-	g_return_if_fail (item != NULL);
+	Task * task;
+
+	/* sanity clause */
+	g_return_if_fail (obj != NULL);
 
 	/* clean up task bits */
-        debug1 (DEBUG_PAN_OBJECT, "task destructor: %p", item);
-	pan_callback_free (TASK(item)->task_ran_callback);
+        debug1 (DEBUG_PAN_OBJECT, "task destructor: %p", obj);
+	task = TASK(obj);
+	pan_callback_free (task->task_ran_callback);
+	pan_g_ptr_array_foreach (task->identifiers, (GFunc)g_object_unref, NULL);
+	g_ptr_array_free (task->identifiers, TRUE);
 
 	/* clean up parent's bits */
-	status_item_destructor (PAN_OBJECT(item));
+	status_item_destructor (obj);
 }
 
 /*****
 ******
 *****/
 
-gint
+void
+task_add_identifiers (Task                * task,
+                      MessageIdentifier  ** identifiers,
+                      int                   identifier_qty)
+{
+	guint i;
+
+	/* sanity clause */
+	g_return_if_fail (task!=NULL);
+	g_return_if_fail (identifiers!=NULL);
+	g_return_if_fail (identifier_qty>0);
+	for (i=0; i<identifier_qty; ++i)
+		g_return_if_fail (PAN_IS_MESSAGE_IDENTIFIER(identifiers[i]));
+
+	/* add the message-ids */	
+	for (i=0; i<identifier_qty; ++i) {
+		g_object_ref (identifiers[i]);
+		g_ptr_array_add (task->identifiers, identifiers[i]);
+	}
+}
+
+GPtrArray*
+task_get_identifiers (Task * task)
+{
+	GPtrArray * retval = g_ptr_array_new ();
+
+	/* sanity clause */
+	g_return_val_if_fail (task!=NULL, retval);
+
+	/* get the message-ids */
+	pan_g_ptr_array_assign (retval, task->identifiers->pdata, task->identifiers->len);
+	return retval;
+}
+
+
+/*****
+******
+*****/
+
+int
 task_run (Task * task)
 {
-	gint retval = TASK_FAIL_HOPELESS;
+	int retval = TASK_FAIL_HOPELESS;
 
 	if (task != NULL)
 	{
