@@ -53,114 +53,31 @@
 #include <pan/text.h>
 #include <pan/util.h>
 
-static char welcome[] =
-{
-"Pan " VERSION "\n"
-"Copyright  2002 by Charles Kerr \n"
-"\n"
-"If you find a bug, please report it.\n"
-"\n"
-"http://pan.rebelbase.com/ - Pan Homepage\n"
-"http://pan.rebelbase.com/bugs/ - Report a Bug\n"
-"http://pan.rebelbase.com/download/ - Upgrade\n"
-"\n"
-"This program is free software; you can redistribute it \n"
-"and/or modify it under the terms of the GNU General Public \n"
-"License as published by the Free Software Foundation; \n"
-"version 2 of the License.\n"
-"\n"
-"This program is distributed in the hope that it will be \n"
-"useful, but WITHOUT ANY WARRANTY; without even the implied \n"
-"warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR \n"
-"PURPOSE.  See the GNU General Public License for more \n"
-"details. \n"
-"\n"
-"The GNU Public License can be found from the menu above \n"
-"in Help|About|License. \n"
-};
 
 static GtkTextBuffer * _text_buffer = NULL;
 static GtkWidget * header_table = NULL;
-static GtkWidget * text_vbox = NULL;
 static GtkWidget * scrolled_window = NULL;
+static GtkTextMark * _begin_mark = NULL;
 
-#if 0
-GdkColor text_fg_color;
-GdkColor text_bg_color;
-#endif
 GdkColor text_quoted_color[3];
 GdkColor text_url_color;
 PanCallback * current_article_changed = NULL;
 
-static void clear_text_buffer_nolock (GtkTextBuffer * buffer);
-static void append_text_buffer_nolock (GtkTextBuffer*,
-                                       const char*,
-                                       gboolean mute_quotes);
-
 /****
 *****
 *****    LOW-LEVEL TEXT UPDATE
 *****
 ****/
 
-static char*
-get_url_from_location (GtkWidget * w, int x, int y)
-{
-	char * retval = NULL;
-	gboolean clicked_on_url;
-	GtkTextBuffer * text_buffer;
-	static GtkTextTag * url_tag = NULL;
-	GtkTextIter pos;
-	int old_x = x, old_y = y;
-
-	/* get the buffer */
-	g_return_val_if_fail (GTK_IS_TEXT_VIEW(w), FALSE);
-	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(w));
-
-	/* translate coordinates */
-	gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW(w),GTK_TEXT_WINDOW_TEXT, old_x, old_y, &x, &y);
-
-	/* did the user click on a url? */
-	gtk_text_view_get_iter_at_location (GTK_TEXT_VIEW(w), &pos, x, y);
-	if (url_tag == NULL) { 
-		GtkTextTagTable * tag_table = gtk_text_buffer_get_tag_table (text_buffer);
-		url_tag = gtk_text_tag_table_lookup (tag_table, "url");
-	}
-	clicked_on_url = gtk_text_iter_has_tag (&pos, url_tag);
-
-	if (clicked_on_url)
-	{
-		GtkTextIter begin;
-		GtkTextIter end;
-
-		/* get the URL */
-		begin = end = pos;
-		gtk_text_iter_backward_to_tag_toggle (&begin, NULL);
-		gtk_text_iter_forward_to_tag_toggle (&end, NULL);
-		retval = gtk_text_iter_get_text (&begin, &end);
-	}
-
-	return retval;
-}
-
-static gboolean
-text_button_pressed (GtkWidget * w, GdkEventButton * event, gpointer unused)
-{
-	g_return_val_if_fail (GTK_IS_TEXT_VIEW(w), FALSE);
-
-	if (event->button==1 || event->button==2)
-	{
-		char * url = get_url_from_location (w, (int)event->x, (int)event->y);
-		if (url != NULL)
-		{
-			pan_url_show (url);
-			g_free (url);
-		}
-	}
-
-	return FALSE;
-}
-
+/**
+ * Returns the quote tag ("quote_0", "quote_1", etc.) appropriate for the line.
+ * The right tag is calculated by adding up the number of quote characters
+ * at the beginning of the line.
+ *
+ * @param utf8_line the line whose quote status we're checking
+ * @param utf8_byte_len the byte length of utf8_line
+ * @return a const string for the line's quote tag.  Never NULL.
+ */
 static const char *
 get_quote_tag (const char * utf8_line, int utf8_byte_len)
 {
@@ -200,6 +117,9 @@ get_quote_tag (const char * utf8_line, int utf8_byte_len)
 	return retval;
 }
 
+/**
+ * Convenience function for clearing the specified GtkTextBuffer.
+ */
 static void
 clear_text_buffer_nolock (GtkTextBuffer * buffer)
 {
@@ -209,9 +129,16 @@ clear_text_buffer_nolock (GtkTextBuffer * buffer)
 	gtk_text_buffer_delete (buffer, &start, &end);
 }
 
+/**
+ * Returns the pointer to the beginning of the next URL
+ * in searchme, or NULL if no URL is found.
+ * @param searchme the zero-terminated string to search for URLS.
+ */
 static const char*
-find_next_url (const char * pch)
+find_next_url (const char * searchme)
 {
+	register const char * pch = searchme;
+
 	for (; pch && *pch; ++pch)
 	{
 		if (*pch == 'h')
@@ -226,6 +153,11 @@ find_next_url (const char * pch)
 	return NULL;
 }
 
+/**
+ * Appends the specified body into the text buffer.
+ * This function takes care of muting quotes and marking
+ * quoted and URL areas in the GtkTextBuffer.
+ */
 static void
 append_text_buffer_nolock (GtkTextBuffer * text_buffer,
                            const char    * body,
@@ -311,6 +243,12 @@ append_text_buffer_nolock (GtkTextBuffer * text_buffer,
 	debug_exit ("append_text_buffer_nolock");
 }
 
+/**
+ * Clears out the old buffer contents and populates it with body.
+ * @param text_buffer the text buffer to repopulate.
+ * @param body the new text to be placed in the buffer
+ * @param mute_quotes true if quotes are to be muted in the buffer, false otherwise.
+ */
 void
 update_body_pane (GtkTextBuffer  * text_buffer,
                   const char     * body,
@@ -379,13 +317,13 @@ sylpheed_textview_smooth_scroll_do (GtkAdjustment  * vadj,
 		up = TRUE;
 	}
 
-	//FIXME? gdk_key_repeat_disable ();
+	/*FIXME? gdk_key_repeat_disable (); */
 
 	for (i=step; i<=change_value; i+=step)
 		gtk_adjustment_set_value (vadj, old_value+(up?-i:i));
 	gtk_adjustment_set_value (vadj, last_value);
 
-	//gdk_key_repeat_restore ();
+	/*gdk_key_repeat_restore (); */
 }
 
 static void
@@ -405,42 +343,33 @@ text_read_more_impl (gboolean more)
 	Group * grouplist_group = grouplist_get_selected_group ();
 	const Group * articlelist_group = articlelist_get_group ();
 	const Article * a = articlelist_get_selected_article_nolock ();
-	const char * art_subject = a ? article_get_subject(a) : NULL;
-	const char * cur_subject = message ? message->subject : NULL;
+	const char * art_msgid = a ? article_get_message_id (a) : NULL;
+	const char * cur_msgid = message ? g_mime_message_get_message_id (message) : NULL;
 	GtkAdjustment * v = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW(scrolled_window));
 	const float inc = v->page_size - arbitrary_font_height_pixels_hack;
 	const gfloat val = CLAMP (v->value + (more ? inc : -inc),
 	                          v->lower,
 	                          MAX(v->upper,v->page_size)-MIN(v->upper,v->page_size));
 
-	if (grouplist_group!=NULL && grouplist_group!=articlelist_group)
-	{
+	if (v->upper>=v->page_size && val!=v->value) {
+		/* this article has some lines left to read */
+		if (Pan.text->parent==scrolled_window && text_window_smooth_scrolling)
+			sylpheed_textview_smooth_scroll_do (v, v->value, val, text_window_smooth_scrolling_speed);
+		else
+			gtk_adjustment_set_value (v, val);
+	}
+	else if (grouplist_group!=NULL && grouplist_group!=articlelist_group) {
 		/* if user has changed group selection, change group. */
 		articlelist_set_group (grouplist_group);
 	}
-	else if (queue_is_online() && pan_strcmp (art_subject, cur_subject))
-	{
-		if (!queue_get_message_id_status (article_get_message_id (a)))
-			articlelist_read_selected ();
-		else
-			text_read_next (more);
+	else if (pan_strcmp (art_msgid, cur_msgid)) {
+		/* if the user has selected a different article, jump to that. */
+		articlelist_read_selected ();
 	}
-	else if ((v->upper < v->page_size) || (val == v->value))
-	{
-		/* or if we're at the end of a page, go to the next page */
+	else {
+		/* just go to the next article */
 		text_read_next (more);
 	}
-	else if (Pan.text->parent==scrolled_window && text_window_smooth_scrolling)
-	{
-		/* or if smooth scrolling is turned on, scroll down a page */
-		sylpheed_textview_smooth_scroll_do (v, v->value, val,
-		                                    text_window_smooth_scrolling_speed);
-	}
-	else
-	{
-		/* or jump down a page */
-		gtk_adjustment_set_value (v, val);
-	}
 
 	if (message != NULL)
 		g_object_unref (message);
@@ -576,8 +505,7 @@ insert_part_partfunc (GMimeObject * obj, gpointer ips_gpointer)
 			g_object_unref (pixbuf);
 		}
 	}
-	else if (g_mime_content_type_is_type (type, "text", "plain")
-	      || g_mime_content_type_is_type (type, "text", "html"))
+	else if (g_mime_content_type_is_type (type, "text", "*"))
 	{
 		char * str = pan_body_to_utf8 (part, ips->default_charset);
 
@@ -594,7 +522,7 @@ static void
 set_text_from_message_nolock (GMimeMessage * message)
 {
 	const char * default_charset = (const char*) message ? g_object_get_data (G_OBJECT(message), "default_charset") : NULL;
-	debug_enter ("set_text_from_message");
+	debug_enter ("set_text_from_message_nolock");
 
 	/* upper headers */
 	gui_headers_set_default_nolock (header_table, message);
@@ -631,14 +559,13 @@ set_text_from_message_nolock (GMimeMessage * message)
 		g_mime_message_foreach_part (message, insert_part_partfunc, &ips);
 	}
 
-	gtk_adjustment_set_value (
-			gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW(scrolled_window)),
-			0.0);
+	gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW(Pan.text), _begin_mark, 0.0, FALSE, 0.0, 0.0);
 
 	/* cleanup */
 	debug_exit ("set_text_from_message_nolock");
 }
 
+
 /****
 *****
 *****  CURRENT MESSAGE
@@ -728,8 +655,10 @@ static void
 set_current_message (GMimeMessage * message)
 {
 	if (message != NULL) {
-		gui_page_set (BODY_PANE);
 		g_object_ref (message); /* pair with set_current_message_mainthread */
+		/* must be called _before_ set_current_message_mainthread 
+		 * so that that function's scroll-to-mark "start" works right*/
+		gui_page_set (BODY_PANE);
 	}
 	gui_queue_add ((GSourceFunc)set_current_message_mainthread, message);
 }
@@ -757,14 +686,16 @@ text_refresh (void)
 }
 
 static void
-task_bodies_ran_cb (gpointer call_obj, gpointer call_arg, gpointer data)
+task_bodies_ran_cb (gpointer call_obj, gpointer call_arg, gpointer user_data)
 {
 	Task * task = TASK (call_obj);
 	const int status = GPOINTER_TO_INT (call_arg);
+	const char * acache_key = (const char*) user_data;
 
 	if (status == TASK_SUCCESS)
 	{
 		GMimeMessage * message = acache_get_message (
+			acache_key,
 			(MessageIdentifier**)task->identifiers->pdata,
 			task->identifiers->len);
 
@@ -778,31 +709,45 @@ task_bodies_ran_cb (gpointer call_obj, gpointer call_arg, gpointer data)
 		}
 	}
 }
+
 void
 text_set_from_identifiers (Server             * server,
+                           const char         * acache_key,
                            const char         * default_charset,
                            MessageIdentifier ** mids,
                            int                  mid_qty)
 {
-	GMimeMessage * message;
-
-	message = acache_get_message (mids, mid_qty);
-
-	if (message)
+	if (mid_qty == 0)
 	{
-		g_object_set_data_full (G_OBJECT(message), "default_charset", g_strdup(default_charset), g_free);
-		set_current_message (message);
-		g_object_unref (message);
-		message = NULL;
+		set_current_message (NULL);
 	}
 	else
 	{
-		Task * task = TASK (task_bodies_new (server, mids, mid_qty));
-		if (task != NULL)
+		GMimeMessage * message;
+
+		g_return_if_fail (is_nonempty_string(acache_key));
+		g_return_if_fail (server != NULL);
+		g_return_if_fail (mids != NULL);
+		g_return_if_fail (mid_qty > 0);
+
+		message = acache_get_message (acache_key, mids, mid_qty);
+
+		if (message)
 		{
-			PAN_OBJECT(task)->user_data = g_strdup (default_charset);
-			pan_callback_add (task->task_ran_callback, task_bodies_ran_cb, NULL);
-			queue_add (task);
+			g_object_set_data_full (G_OBJECT(message), "default_charset", g_strdup(default_charset), g_free);
+			set_current_message (message);
+			g_object_unref (message);
+			message = NULL;
+		}
+		else
+		{
+			Task * task = TASK (task_bodies_new (server, mids, mid_qty));
+			if (task != NULL)
+			{
+				PAN_OBJECT(task)->user_data = g_strdup (default_charset);
+				pan_callback_add (task->task_ran_callback, task_bodies_ran_cb, (gpointer)acache_key);
+				queue_add (task);
+			}
 		}
 	}
 }
@@ -926,6 +871,72 @@ text_key_pressed (GtkWidget * w, GdkEventKey * event, gpointer unused)
 *****
 ****/
 
+static char*
+get_url_from_location (GtkWidget * w, int x, int y)
+{
+	char * retval = NULL;
+	gboolean clicked_on_url;
+	GtkTextBuffer * text_buffer;
+	static GtkTextTag * url_tag = NULL;
+	GtkTextIter pos;
+	int old_x = x, old_y = y;
+
+	/* get the buffer */
+	g_return_val_if_fail (GTK_IS_TEXT_VIEW(w), FALSE);
+	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(w));
+
+	/* translate coordinates */
+	gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW(w), GTK_TEXT_WINDOW_TEXT, old_x, old_y, &x, &y);
+
+	/* did the user click on a url? */
+	gtk_text_view_get_iter_at_location (GTK_TEXT_VIEW(w), &pos, x, y);
+	if (url_tag == NULL) { 
+		GtkTextTagTable * tag_table = gtk_text_buffer_get_tag_table (text_buffer);
+		url_tag = gtk_text_tag_table_lookup (tag_table, "url");
+	}
+	clicked_on_url = gtk_text_iter_has_tag (&pos, url_tag);
+
+	if (clicked_on_url)
+	{
+		GtkTextIter begin;
+		GtkTextIter end;
+
+		/* get the URL */
+		begin = end = pos;
+		gtk_text_iter_backward_to_tag_toggle (&begin, NULL);
+		gtk_text_iter_forward_to_tag_toggle (&end, NULL);
+		retval = gtk_text_iter_get_text (&begin, &end);
+	}
+
+	return retval;
+}
+
+/**
+ * Listen to button presses to see if the user has clicked on a URL.
+ * If they have, we pass the URL to pan_url_show().
+ */
+static gboolean
+text_button_pressed (GtkWidget * w, GdkEventButton * event, gpointer unused)
+{
+	g_return_val_if_fail (GTK_IS_TEXT_VIEW(w), FALSE);
+
+	if (event->button==1 || event->button==2)
+	{
+		char * url = get_url_from_location (w, (int)event->x, (int)event->y);
+		if (url != NULL)
+		{
+			pan_url_show (url);
+			g_free (url);
+		}
+	}
+
+	return FALSE;
+}
+
+/**
+ * Listen to button presses to see if the user mouses onto or off of a URL.
+ * If they have, change the text widget's pointer icon appropriately.
+ */
 static gboolean
 motion_notify_event (GtkWidget * w, GdkEventMotion * event, gpointer user_data)
 {
@@ -940,13 +951,13 @@ motion_notify_event (GtkWidget * w, GdkEventMotion * event, gpointer user_data)
 		GdkCursor * cursor_new;
 		GdkModifierType state;
 
-		// initialize static variables
+		/* initialize static variables */
 		if (!cursor_ibeam)
 			cursor_ibeam = gdk_cursor_new (GDK_XTERM);
 		if (!cursor_href)
 			cursor_href = gdk_cursor_new (GDK_HAND2);
 
-		// pump out x, y, and state
+		/* pump out x, y, and state */
 		if (event->is_hint)
 			gdk_window_get_pointer (event->window, &x, &y, &state);
 		else {
@@ -955,7 +966,7 @@ motion_notify_event (GtkWidget * w, GdkEventMotion * event, gpointer user_data)
 			state = event->state;
 		}
 
-		// decide what cursor we should be using
+		/* decide what cursor we should be using */
 		url = get_url_from_location (w, (int)event->x, (int)event->y);
 		if (!url)
 			cursor_new = cursor_ibeam;
@@ -964,7 +975,7 @@ motion_notify_event (GtkWidget * w, GdkEventMotion * event, gpointer user_data)
 			g_free (url);
 		}
 
-		// change the cursor if needed
+		/* change the cursor if needed */
 		if (cursor_new != cursor_current)
 			gdk_window_set_cursor (event->window, cursor_current=cursor_new);
 	}
@@ -972,13 +983,39 @@ motion_notify_event (GtkWidget * w, GdkEventMotion * event, gpointer user_data)
 	return FALSE;
 }
 
+static char welcome[] =
+{
+"Pan " VERSION "\n"
+"Copyright  2002 by Charles Kerr \n"
+"\n"
+"If you find a bug, please report it.\n"
+"\n"
+"http://pan.rebelbase.com/ - Pan Homepage\n"
+"http://pan.rebelbase.com/bugs/ - Report a Bug\n"
+"http://pan.rebelbase.com/download/ - Upgrade\n"
+"\n"
+"This program is free software; you can redistribute it \n"
+"and/or modify it under the terms of the GNU General Public \n"
+"License as published by the Free Software Foundation; \n"
+"version 2 of the License.\n"
+"\n"
+"This program is distributed in the hope that it will be \n"
+"useful, but WITHOUT ANY WARRANTY; without even the implied \n"
+"warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR \n"
+"PURPOSE.  See the GNU General Public License for more \n"
+"details. \n"
+"\n"
+"The GNU Public License can be found from the menu above \n"
+"in Help|About|License. \n"
+};
+
 GtkWidget *
 text_create (void)
 {
 	GtkWidget * text_view;
 
 	/* create the text view */
-        text_view = gtk_text_view_new_with_buffer (_text_buffer);
+        text_view = gtk_text_view_new ();
 	gtk_widget_add_events (text_view, GDK_POINTER_MOTION_MASK|GDK_POINTER_MOTION_HINT_MASK);
 
 	g_signal_connect (text_view, "motion_notify_event",
@@ -987,8 +1024,6 @@ text_create (void)
 	                  G_CALLBACK(text_button_pressed), NULL);
 	g_signal_connect (text_view, "key_press_event",
 	                  G_CALLBACK(text_key_pressed), NULL);
-	/*if (!use_system_bg)
-		gtk_widget_modify_base (text_view, GTK_STATE_NORMAL, &text_bg_color);*/
         gtk_container_set_border_width (GTK_CONTAINER(text_view), GUI_PAD_SMALL);
         gtk_text_view_set_editable (GTK_TEXT_VIEW(text_view), FALSE);
 	gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW(text_view), FALSE);
@@ -1003,23 +1038,20 @@ text_create (void)
 	                            "underline", PANGO_UNDERLINE_SINGLE,
 	                            "foreground_gdk", &text_url_color,
 				    NULL);
-	/*if (use_system_fg)*/
-		gtk_text_buffer_create_tag (_text_buffer, "quote_0",
-					    NULL);
-	/*else
-		gtk_text_buffer_create_tag (_text_buffer, "quote_0",
-					    "foreground_gdk", &text_fg_color,
-					    NULL);*/
-	gtk_text_buffer_create_tag (_text_buffer, "quote_1",
-	                            "foreground_gdk", &text_quoted_color[0],
-				    NULL);
-	gtk_text_buffer_create_tag (_text_buffer, "quote_2",
-	                            "foreground_gdk", &text_quoted_color[1],
-				    NULL);
-	gtk_text_buffer_create_tag (_text_buffer, "quote_3",
-	                            "foreground_gdk", &text_quoted_color[2],
-				    NULL);
-#warning need some way to refresh these tags when user changes text color via prefs
+
+	/* set a beginning mark that we can always scroll to */
+	if (1) {
+		GtkTextIter start;
+		gtk_text_buffer_get_start_iter (_text_buffer, &start);
+		_begin_mark = gtk_text_buffer_create_mark  (_text_buffer, "start", &start, TRUE);
+	}
+
+	/* set the quote colors */
+	gtk_text_buffer_create_tag (_text_buffer, "quote_0", NULL);
+	gtk_text_buffer_create_tag (_text_buffer, "quote_1", "foreground_gdk", &text_quoted_color[0], NULL);
+	gtk_text_buffer_create_tag (_text_buffer, "quote_2", "foreground_gdk", &text_quoted_color[1], NULL);
+	gtk_text_buffer_create_tag (_text_buffer, "quote_3", "foreground_gdk", &text_quoted_color[2], NULL);
+	/* FIXME need some way to refresh these tags when user changes text color via prefs */
 
 	current_article_changed = pan_callback_new ();
 
@@ -1029,7 +1061,6 @@ text_create (void)
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
-	text_vbox = gtk_vbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER(scrolled_window), text_view);
 
 	if (1) {
