@@ -303,7 +303,9 @@ thread_shutdown_module (void)
 		char * filename = get_filename ();
 		FILE * fp = fopen (filename, "w");
 
-		if (fp != NULL)
+		if (fp == NULL)
+			g_warning (_("Error opening file \"%s\": %s"), filename, g_strerror(errno));
+		else
 		{
 			fprintf (fp, "# one record per line, comma-delimited fields\n");
 			fprintf (fp, "# each record has three fields:\n");
@@ -313,11 +315,7 @@ thread_shutdown_module (void)
 			g_hash_table_foreach (hash, thread_write_line, fp);
 			fclose (fp);
 		}
-		else
-		{
-			g_warning (_("Could not open \"%s\" for writing: %s\n"),
-				filename, g_strerror(errno));
-		}
+
 		g_free (filename);
 		dirty = FALSE;
 	}
