@@ -34,6 +34,7 @@
 #include <pan/base/log.h>
 
 #include <pan/globals.h>
+#include <pan/gui.h>
 #include <pan/prefs.h>
 #include <pan/util.h>
 
@@ -43,21 +44,49 @@
 ****
 ***/
 
-static void
-pan_dialog (const gchar * str, GtkMessageType type, GtkWindow * window)
+typedef struct
+{
+	GtkWindow * window;
+	GtkMessageType type;
+	char * text;
+}
+IdleDialogStruct;
+
+static int
+pan_dialog_idle (gpointer user_data)
 {
 	GtkWidget * w;
+	IdleDialogStruct * info = (IdleDialogStruct*) user_data;
+
+	/* build & show the dialog */
 	pan_lock();
-	w = gtk_message_dialog_new (GTK_WINDOW(window),
+	if (!GTK_IS_WINDOW(info->window)) /* make info->window is alive */
+		info->window = GTK_WINDOW(Pan.window);
+	w = gtk_message_dialog_new (GTK_WINDOW(info->window),
 	                            GTK_DIALOG_DESTROY_WITH_PARENT,
-	                            type,
+	                            info->type,
 				    GTK_BUTTONS_CLOSE,
-				    "%s", str);
+				    "%s", info->text);
 	g_signal_connect_swapped (GTK_OBJECT(w), "response",
 	                          G_CALLBACK (gtk_widget_destroy),
 	                          GTK_OBJECT(w));
 	gtk_widget_show_all (w);
 	pan_unlock();
+
+	/* cleanup */
+	g_free (info->text);
+	g_free (info);
+	return 0;
+}
+
+static void
+pan_dialog (char * i_own_it_str, GtkMessageType type, GtkWindow * window)
+{
+	IdleDialogStruct * info = g_new (IdleDialogStruct, 1);
+	info->text = i_own_it_str;
+	info->type = type;
+	info->window = window;
+	gui_queue_add (pan_dialog_idle, info);
 }
 void
 pan_info_dialog (const char *format, ...)
@@ -71,8 +100,6 @@ pan_info_dialog (const char *format, ...)
 	g_warning (str);
 
 	pan_dialog (str, GTK_MESSAGE_INFO, GTK_WINDOW(Pan.window));
-
-	g_free (str);
 }
 
 void
@@ -87,8 +114,6 @@ pan_error_dialog_parented (gpointer parent, const char *format, ...)
 	g_warning (str);
 
 	pan_dialog (str, GTK_MESSAGE_ERROR, GTK_WINDOW(parent));
-
-	g_free (str);
 }
 
 void
@@ -103,8 +128,6 @@ pan_error_dialog (const char *format, ...)
 	g_warning (str);
 
 	pan_dialog (str, GTK_MESSAGE_ERROR, GTK_WINDOW(Pan.window));
-
-	g_free (str);
 }
 
 /***
@@ -413,7 +436,7 @@ pan_gtk_entry_set_text (GtkWidget            * w,
 
 	g_return_if_fail (GTK_IS_ENTRY(w));
 	pch = pan_header_to_utf8 (text, -1, NULL);
-	gtk_entry_set_text (GTK_ENTRY(w), pch);
+	gtk_entry_set_text (GTK_ENTRY(w), pch ? pch : "");
 	g_free (pch);
 }
 
@@ -430,6 +453,7 @@ pan_hig_frame_new        (const char           * title,
 
 	pch = g_strdup_printf ("<b>%s</b>", title);
 	frame = gtk_frame_new (pch);
+	g_free (pch);
 	gtk_frame_set_shadow_type (GTK_FRAME(frame), GTK_SHADOW_NONE);
 	gtk_label_set_use_markup (GTK_LABEL(gtk_frame_get_label_widget (GTK_FRAME(frame))), TRUE);
 
