@@ -1,5 +1,8 @@
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
+
+#include <gmime/gmime.h>
 
 #include <pan/base/article.h>
 #include <pan/base/base-prefs.h>
@@ -10,7 +13,7 @@
 #define PRINT_ERRORS \
 	if (1) { \
 		gint i; \
-		g_message ("[%d] errors", errors->len); \
+		g_message ("[%d] errors, goodness [%d]", errors->len, (int)goodness); \
 		for (i=0; i<errors->len; ++i) \
 			g_message ("[%d][%s]", i, (gchar*)g_ptr_array_index(errors,i)); \
 	}
@@ -25,16 +28,17 @@ int main (void)
 	Article * a;
 	gchar * pch;
 	GPtrArray * errors = g_ptr_array_new ();
+	GMimePart * part;
+	GMimeMessage * msg;
 	GoodnessLevel goodness = OKAY;
 
 	/* init panbase */
 	g_thread_init (NULL);
+	g_mime_init (0);
 	server_saving_enabled = FALSE;
 	if (1) {
-		gchar * data_dir = g_strdup_printf ("%s/data/", getenv("PWD"));
 		gchar * download_dir = g_strdup_printf ("%s/data/save/", getenv("PWD"));
-		base_prefs_init (data_dir, download_dir, ">", 1000, FALSE, FALSE);
-		g_free (data_dir);
+		base_prefs_init (download_dir, ">", 1000, FALSE, FALSE);
 		g_free (download_dir);
 	}
 
@@ -48,18 +52,22 @@ int main (void)
 
 	/* populate a simple article */
 	++test;
-	a->number = 1;
-	article_set_header (a, HEADER_FROM, "\"Charles Kerr\" <charles@rebelbase.com>", DO_CHUNK);
+	goodness = OKAY;
+	msg = g_mime_message_new (FALSE);
+	g_mime_message_set_sender (msg, "\"Charles Kerr\" <charles@rebelbase.com>");
 	pch = gnksa_generate_message_id ("rebelbase.com");
-	article_set_header (a, HEADER_MESSAGE_ID, pch, DO_CHUNK);
-	article_set_header (a, HEADER_SUBJECT, "MAKE MONEY FAST", DO_CHUNK);
-	article_set_header (a, HEADER_ORGANIZATION, "Lazars Android Works", DO_CHUNK);
-	article_set_header (a, HEADER_NEWSGROUPS, "alt.test", DO_CHUNK);
-	article_set_header (a, PAN_ATTRIBUTION, "Someone wrote", DO_CHUNK);
-	article_set_header (a, PAN_BODY, "Hello World!", DO_CHUNK);
-
+	g_mime_message_set_message_id (msg, pch);
+	g_mime_message_set_subject (msg, "MAKE MONEY FAST");
+	g_mime_message_set_subject (msg, "MAKE MONEY FAST");
+	g_mime_message_set_header (msg, HEADER_ORGANIZATION, "Lazars Android Works");
+	g_mime_message_set_header (msg, HEADER_NEWSGROUPS, "alt.test");
+	g_mime_message_set_header (msg, PAN_ATTRIBUTION, "Someone wrote");
+	part = g_mime_part_new_with_type ("text", "plain");
+	pch = "Hello World!";
+	g_mime_part_set_content (part, pch, strlen(pch));
+	g_mime_message_set_mime_part (msg, GMIME_OBJECT(part));
 	/* this should pass the tests */
-	check_article (a, s, errors, &goodness);
+	message_check (msg, s, errors, &goodness);
 	if (errors->len!=0) return test;
 	if (goodness!=OKAY) return test;
 	g_message ("PASS simple message");
@@ -68,8 +76,9 @@ int main (void)
 	++test;
 	goodness = OKAY;
 	g_ptr_array_set_size (errors, 0);
-	article_set_header (a, PAN_BODY, "> Hello World!\n> All quoted text", DO_CHUNK);
-	check_article (a, s, errors, &goodness);
+	pch = "> Hello World!\n> All quoted text.";
+	g_mime_part_set_content (part, pch, strlen(pch));
+	message_check (msg, s, errors, &goodness);
 	if (errors->len!=2) return test;
 	if (goodness!=REFUSE) return test;
 	if (pan_strcmp((gchar*)g_ptr_array_index(errors,0), "WARNING: The message is entirely quoted text!")) return test;
@@ -80,8 +89,9 @@ int main (void)
 	++test;
 	goodness = OKAY;
 	g_ptr_array_set_size (errors, 0);
-	article_set_header (a, PAN_BODY, "> Hello World!\n> quoted\n> text\n> foo\n> bar\nnew text", DO_CHUNK);
-	check_article (a, s, errors, &goodness);
+	pch = "> Hello World!\n> quoted\n> text\n> foo\n> bar\nnew text";
+	g_mime_part_set_content (part, pch, strlen(pch));
+	message_check (msg, s, errors, &goodness);
 	if (errors->len!=1) return test;
 	if (goodness!=WARN) return test;
 	if (pan_strcmp((gchar*)g_ptr_array_index(errors,0), "WARNING: The message is mostly quoted text.")) return test;
@@ -91,9 +101,9 @@ int main (void)
 	++test;
 	goodness = OKAY;
 	g_ptr_array_set_size (errors, 0);
-	article_set_header (a, PAN_BODY, "> Hello World!\n> quoted\n> text\n> foo\nnew text", DO_CHUNK);
-	check_article (a, s, errors, &goodness);
-PRINT_ERRORS
+	pch = "> Hello World!\n> quoted\n> text\n> foo\nnew text";
+	g_mime_part_set_content (part, pch, strlen(pch));
+	message_check (msg, s, errors, &goodness);
 	if (errors->len!=0) return test;
 	if (goodness!=OKAY) return test;
 	g_message ("PASS 20%% quoted text");
@@ -102,29 +112,31 @@ PRINT_ERRORS
 	++test;
 	goodness = OKAY;
 	g_ptr_array_set_size (errors, 0);
-	article_set_header (a, PAN_BODY, "Hello!\n\n-- \nThis\nSig\nIs\nToo\nLong\n", DO_CHUNK);
-	check_article (a, s, errors, &goodness);
+	pch = "Hello!\n\n-- \nThis\nSig\nIs\nToo\nLong\n";
+	g_mime_part_set_content (part, pch, strlen(pch));
+	message_check (msg, s, errors, &goodness);
 	if (errors->len!=1) return test;
 	if (goodness!=WARN) return test;
-	if (pan_strcmp((gchar*)g_ptr_array_index(errors,0), "WARNING: signature is more than 4 lines long")) return test;
+	if (pan_strcmp((gchar*)g_ptr_array_index(errors,0), "WARNING: Signature is more than 4 lines long")) return test;
 	g_message ("PASS sig too long");
 
 	/* sig check: too wide */
 	++test;
 	goodness = OKAY;
 	g_ptr_array_set_size (errors, 0);
-	article_set_header (a, PAN_BODY,
+	pch =
 "Hello!\n"
 "\n"
 "-- \n"
 "This sig line is exactly 80 characters wide.  I'll keep typing until I reach 80.\n"
 "This sig line is greater than 80 characters wide.  In fact, it's 84 characters wide.\n"
 "This sig line is greater than 80 characters wide.  In fact, it measures 95 characters in width!\n"
-"This sig line is less than 80 characters wide.", DO_CHUNK);
-	check_article (a, s, errors, &goodness);
+"This sig line is less than 80 characters wide.";
+	g_mime_part_set_content (part, pch, strlen(pch));
+	message_check (msg, s, errors, &goodness);
 	if (errors->len!=1) return test;
 	if (goodness!=WARN) return test;
-	if (pan_strcmp((gchar*)g_ptr_array_index(errors,0), "WARNING: 2 lines too wide in signature")) return test;
+	if (pan_strcmp((char*)g_ptr_array_index(errors,0), "WARNING: Signature is more than 80 characters wide.")) return test;
 	g_message ("PASS sig too wide");
 
 
@@ -132,8 +144,9 @@ PRINT_ERRORS
 	++test;
 	goodness = OKAY;
 	g_ptr_array_set_size (errors, 0);
-	article_set_header (a, PAN_BODY, "Hello!\n\n-- \n", DO_CHUNK);
-	check_article (a, s, errors, &goodness);
+	pch = "Hello!\n\n-- \n";
+	g_mime_part_set_content (part, pch, strlen(pch));
+	message_check (msg, s, errors, &goodness);
 	if (errors->len!=1) return test;
 	if (goodness!=WARN) return test;
 	if (pan_strcmp((gchar*)g_ptr_array_index(errors,0),
@@ -145,9 +158,9 @@ PRINT_ERRORS
 	++test;
 	goodness = OKAY;
 	g_ptr_array_set_size (errors, 0);
-	article_set_header (a, PAN_BODY,
-		"Hello!\n\n-- \nThis is a short, narrow sig.\nIt should pass.\n", DO_CHUNK);
-	check_article (a, s, errors, &goodness);
+	pch = "Hello!\n\n-- \nThis is a short, narrow sig.\nIt should pass.\n";
+	g_mime_part_set_content (part, pch, strlen(pch));
+	message_check (msg, s, errors, &goodness);
 	if (errors->len!=0) return test;
 	if (goodness!=OKAY) return test;
 	g_message ("PASS good sig");
@@ -157,8 +170,7 @@ PRINT_ERRORS
 	++test;
 	goodness = OKAY;
 	g_ptr_array_set_size (errors, 0);
-	article_set_header (a, PAN_BODY,
-		">>>>>>>>>>>> I think A\n"
+	pch =   ">>>>>>>>>>>> I think A\n"
 		">>>>>>>>>>> No, it's not\n"
 		">>>>>>>>>> But B => C\n"
 		">>>>>>>>> What's that got to do with A?\n"
@@ -174,8 +186,9 @@ PRINT_ERRORS
 		"I don't know wether I am amused or annoyed. Apparently the funny side\n"
 		"prevailed so far, as I'm still reading.\n"
 		"\n"
-		"-- vbi", DO_CHUNK);
-	check_article (a, s, errors, &goodness);
+		"-- vbi";
+	g_mime_part_set_content (part, pch, strlen(pch));
+	message_check (msg, s, errors, &goodness);
 	if (errors->len!=0) return test;
 	if (goodness!=OKAY) return test;
 	g_message ("PASS Adrian's Fake Followup");
@@ -186,66 +199,67 @@ PRINT_ERRORS
 	++test;
 	goodness = OKAY;
 	g_ptr_array_set_size (errors, 0);
-	article_set_header (a, PAN_BODY,
-	"Hello!\n"
-	"This sig line is exactly 80 characters wide.  I'll keep typing until I reach 80.\n"
-	"This sig line is greater than 80 characters wide.  In fact, it's 84 characters wide.\n"
-	"This sig line is greater than 80 characters wide.  In fact, it measures 95 characters in width!\n"
-	"This sig line is less than 80 characters wide.", DO_CHUNK);
-	check_article (a, s, errors, &goodness);
+	pch = "Hello!\n"
+	      "This sig line is exactly 80 characters wide.  I'll keep typing until I reach 80.\n"
+	      "This sig line is greater than 80 characters wide.  In fact, it's 84 characters wide.\n"
+	      "This sig line is greater than 80 characters wide.  In fact, it measures 95 characters in width!\n"
+	      "This sig line is less than 80 characters wide.";
+	g_mime_part_set_content (part, pch, strlen(pch));
+	message_check (msg, s, errors, &goodness);
 	if (errors->len!=1) return test;
 	if (goodness!=WARN) return test;
-	if (pan_strcmp((gchar*)g_ptr_array_index(errors,0), "WARNING: 2 lines >80 characters in the message.")) return test;
+	if (pan_strcmp((char*)g_ptr_array_index(errors,0), "WARNING: 2 lines are more than 80 characters wide.")) return test;
 	g_message ("PASS body too wide");
 
 	/* body empty */
 	++test;
 	goodness = OKAY;
 	g_ptr_array_set_size (errors, 0);
-	article_set_header (a, PAN_BODY, "\n\t\n   \n-- \nThis is the sig.", DO_CHUNK);
-	check_article (a, s, errors, &goodness);
+	pch = "\n\t\n   \n-- \nThis is the sig.";
+	g_mime_part_set_content (part, pch, strlen(pch));
+	message_check (msg, s, errors, &goodness);
 	if (errors->len!=2) return test;
 	if (goodness!=REFUSE) return test;
 	if (pan_strcmp((gchar*)g_ptr_array_index(errors,0), "ERROR: Message is empty.")) return test;
 	if (pan_strcmp((gchar*)g_ptr_array_index(errors,1), "ERROR: Message appears to have no new content.")) return test;
 	g_message ("PASS empty body");
-	article_set_header (a, PAN_BODY, "Innocent body.", DO_CHUNK);
+	pch = "Innocent body.";
+	g_mime_part_set_content (part, pch, strlen(pch));
 
 	/* empty subject */
 	++test;
 	goodness = OKAY;
 	g_ptr_array_set_size (errors, 0);
-	article_set_header (a, HEADER_SUBJECT, "", DO_CHUNK);
-	check_article (a, s, errors, &goodness);
+	g_mime_message_set_subject (msg, "");
+	message_check (msg, s, errors, &goodness);
 	if (errors->len<1) return test;
 	if (goodness!=REFUSE) return test;
 	g_message ("PASS refuse to post with no subject");
-	article_set_header (a, HEADER_SUBJECT, "Happy Lucky Feeling", DO_CHUNK);
+	g_mime_message_set_subject (msg, "Happy Lucky Feeling");
 
 	/* newsgroups */
 	++test;
 	goodness = OKAY;
 	g_ptr_array_set_size (errors, 0);
-	article_set_header (a, HEADER_NEWSGROUPS, "alt.test,unknown.group", DO_CHUNK);
-	check_article (a, s, errors, &goodness);
+	g_mime_message_set_header (msg, HEADER_NEWSGROUPS, "alt.test,unknown.group");
+	message_check (msg, s, errors, &goodness);
 	if (errors->len!=1) return test;
 	if (goodness!=WARN) return test;
 	if (pan_strcmp((gchar*)g_ptr_array_index(errors,0), "WARNING: Unknown group: \"unknown.group\".")) return test;
 	g_message ("PASS unknown group");
-	article_set_header (a, HEADER_NEWSGROUPS, "alt.test", DO_CHUNK);
+	g_mime_message_set_header (msg, HEADER_NEWSGROUPS, "alt.test");
 
 	/* newsgroups */
 	++test;
 	goodness = OKAY;
 	g_ptr_array_set_size (errors, 0);
-	article_set_header (a, HEADER_FOLLOWUP_TO, "alt.test,unknown.group", DO_CHUNK);
-	check_article (a, s, errors, &goodness);
+	g_mime_message_set_header (msg, HEADER_FOLLOWUP_TO, "alt.test,unknown.group");
+	message_check (msg, s, errors, &goodness);
 	if (errors->len!=1) return test;
 	if (goodness!=WARN) return test;
 	if (pan_strcmp((gchar*)g_ptr_array_index(errors,0), "WARNING: Unknown group: \"unknown.group\".")) return test;
 	g_message ("PASS unknown followup-to group");
 
-
 	/* success */
 	g_message ("Passed all message-check tests");
 	return 0;
